/**
 *  Reload Dweezil Library - Handles GUI, Undo and Utility functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.dweezil.gui;

import java.io.*;

import javax.swing.*;

/**
 * An implementation of JFileChooser that remembers the last directory.
 *
 * @author Phillip Beauvoir
 * @version 2003-09-17
 */
public class ReloadFileChooser
extends JFileChooser
{
    /**
     * The initial default directory
     */
    protected static File _lastFolder;

    /**
     * Set the initial folder
     */
    public static void setDefaultFolder(File defFolder) {
        _lastFolder = defFolder;
    }

    /**
     * Default constructor for last saved folder
     */
    public ReloadFileChooser() {
        this(_lastFolder);
    }

    /**
     * Constructor for a given folder
	 * @param folder the directory to set
     */
    public ReloadFileChooser(File folder) {
        super(folder);
    }

    /**
    * Returns the selected file and updates the local dir
    * @return The selected file
    */
    public File getSelectedFileAndStore() {
        File file = getSelectedFile();
        if(file != null) _lastFolder = file.getParentFile();
        return file;
    }

    /**
    * Returns the selected files and updates the local dir
    * @return The selected files
    */
    public File[] getSelectedFilesAndStore() {
        File[] files = getSelectedFiles();
        if(files != null) _lastFolder = files[0].getParentFile();
        return files;
    }

    /**
    * Store the current directory.
    * @param folder the directory to set
    */
    public void setStoredFolder(File folder) {
        _lastFolder = folder;
    }

	/**
	 * Get the stored directory
	 * @return the stored directory
	 */
    public File getStoredFolder() {
        return _lastFolder;
    }
}
