/**
  *  Reload Dweezil Library - Handles GUI, Undo and Utility functions
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.dweezil.gui.tree;

import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;



/**
 * Extend and over-ride the DefaultTreeCellRenderer so that we can highlight the dragged over node
 *
 * @author Phillip Beauvoir
 * @version 2003-06-30
 */
public class ReloadTreeRenderer
extends DefaultTreeCellRenderer
{

	/**
	 * Configures the renderer based on the passed in components. The value is set from messaging
	 * the tree with convertValueToText, which ultimately invokes toString on value. The foreground
	 * color is set based on the selection and the icon is set based on on leaf and expanded.
	 *
	 * @param tree the JTree the receiver is being configured for
	 * @param value the value to be set for the current tree cell
	 * @param selected if true, the current tree cell will be drawn as if selected
	 * @param expanded if true, the node is currently expanded
	 * @param leaf if true, the node represents a leaf
	 * @param row the current row of the tree table
	 * @param hasFocus if true, the node currently has focus
	 * @return the Component that the renderer uses to draw the value
	 */
    public Component getTreeCellRendererComponent(JTree tree, Object value,
            boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {

        if(value instanceof ReloadTreeNode) {
            ReloadTreeNode selNode = (ReloadTreeNode) value;
            if(selNode != null && selNode.isHiLited) selected = true;
        }

        return super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
    }
}