/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.dweezil.gui.widgets;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import org.jdom.*;

import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.gui.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.editor.metadata.formview.*;
import uk.ac.reload.editor.metadata.vdexBrowser.*;
import uk.ac.reload.editor.vocabulary.*;
import uk.ac.reload.editor.vocabulary.formview.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.schema.*;


/**
 * A Combination of JTextField and JButton.
 * This is used for providing Vdex Browsing facility from the Metadata Editor.
 *
 * @author Tina Manoharan
 * @version 2003-09-10
 */
public class ReloadButtonTextField
extends JPanel
implements ReloadWidget
{
    /**
     * The maximum number of characters allowed
     */
    protected int _maxLength;

    /**
     * A Textfield to display the value
     */
    protected JTextField _textField;

    /**
     * A button for browing Vdex vocabulary values
     */
    protected JButton _button;


    /**
     * The text to display on button
     */
    protected String _text;

    /**
     * The button type
     */
    protected int _type;

    /**
     * The Vdex Source Path, that is the source element that has the Vocabulary Name
     * or Identifier of the Vdex file that we have to use for browsing
     */
    protected String _vdexSourcePath;


    /**
     * The Vdex Source Value, that is the Vocabulary Name or Identifier of the Vdex
     * file that we will use for browsing
     */
    protected String _vdexSourceValue;


    /**
     * Type of button - BROWSE_TYPE
     */
    public static int VDEX_BROWSE_TYPE = 1;

    /**
     * The Reload Schema Document
     */
    protected ReloadSchemaDocument _reloadSchemaDocument;

	/**
	 * The Reload Schema Element bound to the JTextPane
	 */
    protected ReloadSchemaElement _schemaElement;

	/**
	 * The Element bound to the TextArea
	 */
    protected Element _element;

	/**
	 * The ReloadFormField bound to this TextArea
	 */
    protected ReloadFormField _formField = null;

    /**
     * Constructor
     * @param reloadSchemaDocument the owner document
     * @param type the button type (for example Vdex Browse Button)
     * @param text the text to display on the button
     */
    public ReloadButtonTextField(ReloadSchemaDocument reloadSchemaDocument, int type, String text) {
        super(new BorderLayout());

		_reloadSchemaDocument = reloadSchemaDocument;
        _type = type;
        _text = text;

        // Add text field
        _textField = new JTextField();
        _textField.setFont(ReloadUIManager.plainFont11);
        add(_textField, BorderLayout.CENTER);

        // Add Button
        _button = new JButton(_text, ReloadUIManager.getIcon(ReloadEditor.ICON_VXBROWSE));
        add(_button, BorderLayout.EAST);
        setupAction();
    }

    /**
     * Set up the button action. If the button type is Vdex Browse, then it opens a MetadataVdexBrowser.
     */
    protected void setupAction() {
        _button.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {

				if(_type == VDEX_BROWSE_TYPE) {

						Thread thread = new Thread() {
		        	    	public void run() {
		        	    	    try {

		        	    	        // ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

				       	            // first get the Vdex Source Value
				       	            handleVdexBrowse();

		        	    	        // New Vdex Browser Window
		        	    	        final MetadataVdexBrowser mdVdexBrowser = new MetadataVdexBrowser(ReloadButtonTextField.this, _vdexSourceValue, "Browse Vdex Vocabulary");

		        	    	        // Show the Window later in the Event queue
		        	    	        SwingUtilities.invokeLater(new Runnable() {
		        	    	            public void run() {
		        	    	                //ReloadEditor.getInstance().addInternalFrame(mdVdexBrowser);

		        	    	                mdVdexBrowser.show();
		        	    	            }
		        	    	        });

		        	    	    }
		        	    	    catch(Exception ex) {
		        	    	        ex.printStackTrace();
		        	    	    }
		        	    	    //finally {
		        	    	        //ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
		        	    	    //}
		        	    	}
		        		};

		        		thread.start();
				}

				//ReloadEditor.getInstance().setCursor(ReloadUIManager.TEXT_CURSOR);
			}
		});
    }


    /**
     * Get the Text Field
     * @return the TextField
     */
    public JTextField getTextField() {
        return _textField;
    }

    /**
     * Set the Value on the TextField
     * @param value the value to be set on the TextField
     */
    public void setTextValue(String value) {
        _textField.setText(value);
    }

    /**
     * Insert some text into the TextField
     * @param text the text to be inserted into the TextField
     */
    public void insertTextValue(String text) {

    }

    /**
     * Get the Value on the TextField
     * @return the value on the TextField
     */
    public String getTextValue() {
        return _textField.getText();
    }

    /**
     * Set the maximum allowed number of characters
     * @param maxLength the maximum allowed number of characters
     */
    public void setMaxLength(int maxLength) {
        _maxLength = maxLength;
    }

    /**
     * Determine whether this widget only needs a single line to display itself
     * @return true if this widget only needs a single line to display itself, false otherwise
     */
    public boolean isSingleLine() {
        return true;
    }

    /**
     * Add a DocumentListener to the Text Box
     * @param dl the DocumentListener to add
     */
    public void addDocumentListener(DocumentListener dl) {
        _textField.getDocument().addDocumentListener(dl);
    }

    /**
     * Remove a DocumentListener from the Text Box
     * @param dl the DocumentListener to remove
     */
    public void removeDocumentListener(DocumentListener dl) {
        _textField.getDocument().removeDocumentListener(dl);
    }

    /**
     * Set the components enabled/disabled
     * @param enabled if true enables the component, and if false disables it
     */
    public void setEnabled(boolean enabled) {
        _textField.setEnabled(enabled);
        _button.setEnabled(enabled);
    }


    /**
     * Set the columns allowed for the textfield
     * @param cols the columns allowed for the text field
     */
    public void setColumns(int cols) {
        _textField.setColumns(cols);
    }

    /**
     * Set the Vdex Source Path
     * @param vdexSourcePath the vdex source path
     */
    public void setVdexSourcePath(String vdexSourcePath) {

		_vdexSourcePath = vdexSourcePath;
	}


    /**
     * Get the Vdex Source Path
     * @return the vdex source path
     */
    public String getVdexSourcePath() {

		return _vdexSourcePath;
	}

    /**
     * Set the Vdex Source Value
     * @param vdexSourceValue
     */
    public void setVdexSourceValue(String vdexSourceValue) {

		_vdexSourceValue = vdexSourceValue;
	}

    /**
     * Get the Vdex Source Value
     * @return the vdex source value
     */
    public String getVdexSourceValue() {

		return _vdexSourceValue;
	}

	/**
	 * Set the schema element that is bound to this TextField
	 * @param schemaElement the schema element that is bound to this TextField
	 */
	public void setSchemaElement(ReloadSchemaElement schemaElement) {

		_schemaElement = schemaElement;
	}

	/**
	 * Get the schema element that is bound to this TextField
	 * @return the schema element that is bound to this TextField
	 */
	public ReloadSchemaElement getSchemaElement() {

		return _schemaElement;
	}

	/**
	 * Set the element that is bound to this TextArea
	 * @param element the element that is bound to this TextArea
	 */
	public void setElement(Element element) {

		_element = element;
	}

	/**
	 * Set the form field that is bound to this TextArea
	 * @param formField the form field that is bound to this TextArea
	 */
	public void setFormField(ReloadFormField formField) {

		_formField = formField;
	}

    /**
     * Handle Vdex Browsing. Gets the Vdex Source Value using the Vdex Source Path information.
     */
    public void handleVdexBrowse() {

		if(_formField != null && _formField instanceof MetadataTextField) {

			_element = ((MetadataTextField)_formField).getElement();
		}

		if(_element == null) _element = _reloadSchemaDocument.addElementUnique(this, _schemaElement, true);

		if(_element != null && _vdexSourcePath != null) {

			DomPath domPath = new DomPath(_vdexSourcePath);

			Element[] elements = _reloadSchemaDocument.getAllElements(domPath);

			if(elements == null) {

				showWarningMessage("Choose a Vdex source file");
			}
			else {

				this.elementWithSameAncestor(_element.getParent(), elements);
			}
		}
	}

	private Element elementWithSameAncestor(Element ancestor, Element[] elements) {

		boolean found = false;

		if(ancestor != null) {

			for(int i = 0; i < elements.length; i++) {

				if(elements[i].isAncestor(ancestor)) {

					if(elements[i].getText() != null) {

						this.setVdexSourceValue(elements[i].getText());
					}

					found = true;

					return elements[i];
				}
			}

			if(!found) elementWithSameAncestor(ancestor.getParent(), elements);
		}

		return null;
	}


    /**
     * Warn the user that a source vdex file was not choosen. You need a Vdex Source Path to get the
     * Vdex Source Value.
     * @param msg the warning message to display in the dialog
     */
    protected void showWarningMessage(String msg) {
        JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                                      msg,
                                      "Reload Editor",
                                      JOptionPane.WARNING_MESSAGE);
    }


    /**
     * Set Vdex Vocabulary Values selected on the Browser to ReloadDocument
     * @param vals an array of string values
     */
	public void setVdexValuesToDocument(String[] vals) {

		StringTokenizer st;

		String val1 = null, val2 = null;

		Element parentElement = null, firstElement = null, secondElement = null, newParentElement = null;

		int index;

		ReloadSchemaElement parentSchemaElement = null, newSchemaElement = null, secondSchemaElement = null;


		for(int i = 0; i < vals.length; i++) {

			val1 = null; val2 = null;

			if(vals[i].indexOf(':') != -1) {

				st = new StringTokenizer(vals[i], ":");

				while(st.hasMoreTokens()) {

					val1 = st.nextToken().trim();

					val2 = st.nextToken().trim();
				}
			}
			else {

				val1 = vals[i];
			}

			// For the element bound to this component

			if(i == 0) {

				// setting the element

				if(_formField != null && _formField instanceof MetadataTextField) {

					_element = ((MetadataTextField)_formField).getElement();
				}

				if(_element == null) _element = _reloadSchemaDocument.addElementUnique(this, _schemaElement, true);


				// Case1: First part is for the ID element

				if(val1 != null) {

					if(_element != null) {

						_element.setText(val1);

						_reloadSchemaDocument.changedElement(this, _element);
					}
				}

				// setting the parent element

				if(_schemaElement.getName().equals("langstring")) {

					parentSchemaElement = (ReloadSchemaElement) _schemaElement.getParent().getParent();

					if(_element != null) parentElement = _element.getParent().getParent();
				}
				else {

					parentSchemaElement = (ReloadSchemaElement) _schemaElement.getParent();

					if(_element != null) parentElement = _element.getParent();
				}


				if(parentSchemaElement != null) {

					index = parentSchemaElement.indexofChild(_schemaElement);

					if(index != -1) {

						index++;

						secondSchemaElement = parentSchemaElement.getChildAt(index);
					}
				}

				if(secondSchemaElement != null && parentElement != null) {

					if(secondSchemaElement.hasChild("langstring")) {

						Element[] elements = _reloadSchemaDocument.getAllElements(secondSchemaElement.getChild("langstring"));

						secondElement = this.elementWithSameAncestor(parentElement, elements);
					}
					else {

						Element[] elements = _reloadSchemaDocument.getAllElements(secondSchemaElement);

						secondElement = this.elementWithSameAncestor(parentElement, elements);
					}

					if(secondElement == null) {

						secondElement = _reloadSchemaDocument.addElementBySchemaUndoable(this, parentElement, secondSchemaElement, true);

						if(secondSchemaElement.hasChild("langstring")) {

							secondElement = secondElement.getChild("langstring", secondElement.getNamespace());
						}
					}
				}

				// Assumption: 2nd element if exists is at the same level as the first

				// Case2: Second part is for the Entry element

				if(val2 != null) {

					if(secondElement != null) {

						secondElement.setText(val2);

						_reloadSchemaDocument.changedElement(this, secondElement);
					}
				}
			}
			else { // if (i > 0)

				// Note: All elements (e.g. Taxons) are in a hierarchy, this is according to Reload IMS Metadata Schema

				// Case1: First part is for the ID element

				if(parentElement != null && parentSchemaElement != null) {

					if(parentSchemaElement.hasChild(parentSchemaElement.getName())) {

						// hierarchy
						// TaxonPath -> taxon -> taxon

						newSchemaElement = parentSchemaElement.getChild(parentSchemaElement.getName(), parentSchemaElement.getNamespace());
					}
					else {

						// same level
						// TaxonPath -> taxon
						// 			 -> taxon

						newSchemaElement = parentSchemaElement;

						parentElement = parentElement.getParent();
					}

					if(newSchemaElement != null) {

						newParentElement = _reloadSchemaDocument.addElementBySchemaUndoable(this, parentElement, newSchemaElement, true);

						if(_schemaElement.getName().equals("langstring")) {

							firstElement = _reloadSchemaDocument.addElementBySchemaUndoable(this, newParentElement, (ReloadSchemaElement) _schemaElement.getParent(), true);

							firstElement = firstElement.getChild("langstring", firstElement.getNamespace());
						}
						else {

							firstElement = _reloadSchemaDocument.addElementBySchemaUndoable(this, newParentElement, _schemaElement, true);
						}
					}

				}

				if(val1 != null) {

					if(firstElement != null) {

						firstElement.setText(val1);

						_reloadSchemaDocument.changedElement(this, firstElement);
					}
				}


				// Case2: Second part is for the Entry element

				if(newParentElement != null && secondSchemaElement != null) {

					secondElement = _reloadSchemaDocument.addElementBySchemaUndoable(this, newParentElement, secondSchemaElement, true);

					if(secondSchemaElement.hasChild("langstring")) {

						secondElement = secondElement.getChild("langstring", secondElement.getNamespace());
					}
				}

				if(val2 != null) {

					if(secondElement != null) {

						secondElement.setText(val2);

						_reloadSchemaDocument.changedElement(this, secondElement);
					}
				}

				// setting parentElement
				parentElement = newParentElement;

				// setting parentSchemaElement
				parentSchemaElement = newSchemaElement;
			}
		}
	}

}



/*

=================================================================

	public void setVdexValuesToDocument(String[] vals) {

		StringTokenizer st;

		String val1 = null, val2 = null;

		Element element, parentElement = null, newParentElement = null;

		int index;

		ReloadSchemaElement parentSchemaElement = null, newSchemaElement = null, secondSchemaElement = null;


		for(int i = 0; i < vals.length; i++) {

			val1 = null; val2 = null;

			if(vals[i].indexOf(':') != -1) {

				st = new StringTokenizer(vals[i], ":");

				while(st.hasMoreTokens()) {

					val1 = st.nextToken().trim();

					val2 = st.nextToken().trim();
				}
			}
			else {

				val1 = vals[i];
			}

			// For the element bound to this component

			if(i == 0) {

				// Case1: First part is for the ID element

				if(val1 != null) {

					// gets the 'first' found element for the given _schemaElement

					element = _reloadSchemaDocument.getElement(_schemaElement);

					if(element != null) {

						element.setText(val1);

						_reloadSchemaDocument.changedElement(this, element);
					}
					else {

						element = _reloadSchemaDocument.addElementUnique(this, _schemaElement, true);

						if(element != null) {

							element.setText(val1);

							_reloadSchemaDocument.changedElement(this, element);

						}
					}

					// setting the parentElement
					if(_schemaElement.getName().equals("langstring")) {

						parentSchemaElement = (ReloadSchemaElement) _schemaElement.getParent().getParent();

						parentElement = element.getParent().getParent();
					}
					else {

						parentSchemaElement = (ReloadSchemaElement) _schemaElement.getParent();

						parentElement = element.getParent();
					}

				}

				// Assumption: 2nd element if exists is at the same level as the first

				// Case2: Second part is for the Entry element

				if(val2 != null) {

					index = parentSchemaElement.indexofChild(_schemaElement);

					index++;

					secondSchemaElement = parentSchemaElement.getChildAt(index);

					if(secondSchemaElement != null) {

						if(secondSchemaElement.hasChild("langstring")) {

							element = _reloadSchemaDocument.getElement(secondSchemaElement.getChild("langstring"));
						}
						else element = _reloadSchemaDocument.getElement(secondSchemaElement);

						if(element != null) {

							element.setText(val2);

							_reloadSchemaDocument.changedElement(this, element);
						}
						else {

							element = _reloadSchemaDocument.addElementUnique(this, secondSchemaElement, true);

							if(secondSchemaElement.hasChild("langstring")) {

								element = element.getChild("langstring", element.getNamespace());
							}

							if(element != null) {

								element.setText(val2);

								_reloadSchemaDocument.changedElement(this, element);

							}
						}
					}
				}
			}
			else { // if (i > 0)

				// Note: All elements (e.g. Taxons) are in a hierarchy, this is according to Reload IMS Metadata Schema

				// Case1: First part is for the ID element

				if(val1 != null && parentElement != null && parentSchemaElement != null) {

					if(parentSchemaElement.hasChild(parentSchemaElement.getName())) {

						// hierarchy
						// TaxonPath -> taxon -> taxon

						newSchemaElement = parentSchemaElement.getChild(parentSchemaElement.getName(), parentSchemaElement.getNamespace());
					}
					else {

						// same level
						// TaxonPath -> taxon
						// 			 -> taxon

						newSchemaElement = parentSchemaElement;

						parentElement = parentElement.getParent();
					}

					if(_schemaElement.getName().equals("langstring")) {

						newParentElement = _reloadSchemaDocument.addElementBySchemaUndoable(this, parentElement, newSchemaElement, true);


						element = _reloadSchemaDocument.addElementBySchemaUndoable(this, newParentElement, (ReloadSchemaElement)_schemaElement.getParent(), true);


						element = element.getChild("langstring", element.getNamespace());
					}
					else {

						newParentElement = _reloadSchemaDocument.addElementBySchemaUndoable(this, parentElement, newSchemaElement, true);


						element = _reloadSchemaDocument.addElementBySchemaUndoable(this, newParentElement, _schemaElement, true);

					}

					if(element != null) {

						element.setText(val1);

						_reloadSchemaDocument.changedElement(this, element);
					}

				}


				// Case2: Second part is for the Entry element

				if(val2 != null && newParentElement != null && parentSchemaElement != null && secondSchemaElement != null) {

					if(secondSchemaElement != null) {

						if(secondSchemaElement.hasChild("langstring")) {

							element = _reloadSchemaDocument.addElementBySchemaUndoable(this, newParentElement, secondSchemaElement, true);


							element = element.getChild("langstring", element.getNamespace());
						}
						else {

							element = _reloadSchemaDocument.addElementBySchemaUndoable(this, newParentElement, secondSchemaElement, true);
						}

						if(element != null) {

							element.setText(val2);

							_reloadSchemaDocument.changedElement(this, element);
						}
					}
				}


				// setting parentElement
				parentElement = newParentElement;

				// setting parentSchemaElement
				parentSchemaElement = newSchemaElement;
			}
		}
	}


=================================================================

*/