/**
  *  Reload Dweezil Library - Handles GUI, Undo and Utility functions
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.dweezil.gui.widgets;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

import uk.ac.reload.dweezil.gui.*;

/**
 * A Reload JTextPane
 *
 * @author Phillip Beauvoir
 * @version 2003-08-04
 */
public class ReloadTextPane
extends JTextPane
implements ReloadWidget
{
    /**
     * The maximum number of characters allowed
     */
    protected int _maxLength;

    /**
     * Default constructor
     */
    public ReloadTextPane() {
        setTabKeyBehaviour();
        setUI();
    }

    /**
     * Set the Value
     * @param value the value to be set to this TextPane
     */
    public void setTextValue(String value) {
        setText(value);
        setCaretPosition(0);
    }

    /**
     * Insert some text
     * @param text the text to insert into this TextPane
     */
    public void insertTextValue(String text) {
        try {
            getDocument().insertString(getCaretPosition(), text, null);
        }
        catch(BadLocationException ex) {
        }
    }

    /**
     * Get the Value
     * @return the value on this TextPane
     */
    public String getTextValue() {
        return getText();
    }

    /**
     * Set the maximum allowed of digits
     * @param maxLength is the maximum allowed digits of this TextPane
     */
    public void setMaxLength(int maxLength) {
        _maxLength = maxLength;
    }

    /**
     * Determine whether this widget only needs a single line to display itself
     * @return true if this widget needs only a single line to display itself, false otherwise
     */
    public boolean isSingleLine() {
        return false;
    }

    /**
     * Add a DocumentListener to the Text Box
     * @param dl the DocumentListner to add
     */
    public void addDocumentListener(DocumentListener dl) {
        getDocument().addDocumentListener(dl);
    }

    /**
     * Remove a DocumentListener from the Text Box
     * @param dl the DocumentListener to remove
     */
    public void removeDocumentListener(DocumentListener dl) {
        getDocument().removeDocumentListener(dl);
    }

    /**
     * Set the UI
     */
    protected void setUI() {
        setFont(ReloadUIManager.plainFont11);
    }

    /**
     * Update the UI
     */
    public void updateUI() {
        super.updateUI();
        setUI();
    }

    /**
     * Make sure we can use the Tab key for traversal
     */
    protected void setTabKeyBehaviour() {
        Set forwardTraversalKeys = new HashSet();
        forwardTraversalKeys.add(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0));
        forwardTraversalKeys.add(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, InputEvent.CTRL_MASK));
        setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, forwardTraversalKeys);
        Set backwardTraversalKeys = new HashSet();
        backwardTraversalKeys.add(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, InputEvent.SHIFT_MASK));
        backwardTraversalKeys.add(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, InputEvent.SHIFT_MASK | InputEvent.CTRL_MASK));
        setFocusTraversalKeys(KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS, backwardTraversalKeys);
    }
}