/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.contentpackaging;

import java.io.*;

import javax.swing.*;

import org.jdom.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.moonunit.*;
import uk.ac.reload.moonunit.dom.contentpackaging.*;
import uk.ac.reload.moonunit.schema.*;

/**
 * The CP ReloadSchema Controller
 *
 * @author Phillip Beauvoir
 * @version 2003-11-04
 */
public class ContentPackageSchemaController
extends ReloadEditorSchemaController
{
    /**
     * Default Constructor with Default Profile
     */
    public ContentPackageSchemaController() {
        super();
    }

    /**
     * Default Constructor with Default Profile
     */
    public ContentPackageSchemaController(File fileSchema, String version) {
        try {
            setVersion(version);

            loadSchema(fileSchema);

            // Get the Default Profile
            ContentPackageHelperProfile defProfile = ContentPackageHelperProfile.getDefaultContentPackageHelperProfile();
            setHelperProfile(defProfile);
        }
        catch(ReloadEditorException ex) {
            ErrorDialogBox.showWarning("Could not load Default Content Package Profile", "Load Profile", ex);
        }
    }

    /**
     * @return the Default ContentPackageSchemaController - 1.1.3
     */
    public static ContentPackageSchemaController getDefaultSchemaController() {
        return (ContentPackageSchemaController)getReloadSchemaController(IMSDocumentHandler.IMSCP_113);
    }

    /**
     * @return the Default MD SchemaController - 1.2.2
     */
    public static MetadataSchemaController getDefaultMetadataSchemaController() {
        return (MetadataSchemaController)getReloadSchemaController(IMSDocumentHandler.IMSMD_122);
    }

    /**
     * @return the Root element name
     */
    public String getRootElementName() {
        return ContentPackageCore.MANIFEST;
    }

    /**
     * @return true if nameSpaceURI is in our list of accepted Namespace URIs
     */
    public static boolean isAcceptedNamespaceURI(String nameSpaceURI) {
        for(int i = 0; i < IMSDocumentHandler.IMSCP_NAMESPACE_URIS.length; i++) {
            if(nameSpaceURI.equals(IMSDocumentHandler.IMSCP_NAMESPACE_URIS[i])) {
                return true;
            }
        }
        return false;
    }

    /**
     * Get The Leaf Icon
     */
    public Icon getLeafIcon(Element element) {
        // Only want to only display icons for our Namespace
        if(!isAcceptedNamespaceURI(element.getNamespace().getURI()))
            return ReloadUIManager.getIcon(ReloadEditor.ICON_MDLEAF);

        String elementName = element.getName();

        // Set some icons - HARD CODED FOR NOW
        if(elementName.equals(ContentPackageCore.MANIFEST)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_CP);
        }
        else if(elementName.equals(ContentPackageCore.ORGANIZATIONS)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_ORGS);
        }
        else if(elementName.equals(ContentPackageCore.ORGANIZATION)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_ORG);
        }
        else if(elementName.equals(ContentPackageCore.ITEM)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_ITEM);
        }
        else if(elementName.equals(ContentPackageCore.FILE)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_FILE);
        }
        else if(elementName.equals(ContentPackageCore.METADATA)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_MD);
        }
        else if(elementName.equals(ContentPackageCore.RESOURCES)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_RESOURCES);
        }
        else if(elementName.equals(ContentPackageCore.RESOURCE)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_RESOURCE);
        }
        //else if(elementName.equals(ContentPackageCore.TITLE)) {
        //    return ReloadUIManager.getIcon(ReloadUIManager.ICON_TITLE);
        //}
        else if(elementName.equals(ContentPackageCore.DEPENDENCY)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_DEPENDENCY);
        }
        else {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_MDLEAF);
        }
    }

    /**
     * Get The Leaf Icon
     */
    public Icon getLeafIcon(ReloadSchemaElement schemaElement) {
        // Only want to only display icons for our Namespace
        if(!isAcceptedNamespaceURI(schemaElement.getNamespace().getURI()))
            return ReloadUIManager.getIcon(ReloadEditor.ICON_MDLEAF);

        String elementName = schemaElement.getName();

        // Set some icons - HARD CODED FOR NOW
        if(elementName.equals(ContentPackageCore.MANIFEST)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_CP);
        }
        else if(elementName.equals(ContentPackageCore.ORGANIZATIONS)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_ORGS);
        }
        else if(elementName.equals(ContentPackageCore.ORGANIZATION)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_ORG);
        }
        else if(elementName.equals(ContentPackageCore.ITEM)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_ITEM);
        }
        else if(elementName.equals(ContentPackageCore.FILE)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_FILE);
        }
        else if(elementName.equals(ContentPackageCore.METADATA)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_MD);
        }
        else if(elementName.equals(ContentPackageCore.RESOURCES)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_RESOURCES);
        }
        else if(elementName.equals(ContentPackageCore.RESOURCE)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_RESOURCE);
        }
        //else if(elementName.equals(ContentPackageCore.TITLE)) {
        //    return ReloadUIManager.getIcon(ReloadEditor.ICON_TITLE);
        //}
        else if(elementName.equals(ContentPackageCore.DEPENDENCY)) {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_DEPENDENCY);
        }
        else {
            return ReloadUIManager.getIcon(ReloadEditor.ICON_MDLEAF);
        }
    }

    /**
     * Get The Open Icon
     */
    public Icon getOpenIcon(Element element) {
        return getLeafIcon(element);
    }

    /**
     * Get The Open Icon
     */
    public Icon getOpenIcon(ReloadSchemaElement schemaElement) {
        return getLeafIcon(schemaElement);
    }

    /**
     * Get The Closed Icon
     */
    public Icon getClosedIcon(Element element) {
        return getLeafIcon(element);
    }

    /**
     * Get The Closed Icon
     */
    public Icon getClosedIcon(ReloadSchemaElement schemaElement) {
        return getLeafIcon(schemaElement);
    }

    /**
     * Copy the Schema Files to the Project Folder
     */
    public void copySchemaFilesToFolder(File projectFolder) throws ReloadEditorException {
        try {
            String version = getVersion();

            // IMS CP 113
            if(version.equals(ReloadEditorDocumentHandler.IMSCP_113)) {
                // Metadata Schema Files
                ReloadUtils.copyFolder(new File(ReloadEditorDocumentHandler.SCHEMASHIPPEDFOLDER, IMSDocumentHandler.IMSMD_122), projectFolder, null);
                // CP schema Files
                ReloadUtils.copyFolder(new File(ReloadEditorDocumentHandler.SCHEMASHIPPEDFOLDER, IMSDocumentHandler.IMSCP_113), projectFolder, null);
            }

            // IMS CP 112
            else if(version.equals(ReloadEditorDocumentHandler.IMSCP_112)) {
                // Metadata Schema Files
                ReloadUtils.copyFolder(new File(ReloadEditorDocumentHandler.SCHEMASHIPPEDFOLDER, IMSDocumentHandler.IMSMD_121), projectFolder, null);
                // CP schema Files
                ReloadUtils.copyFolder(new File(ReloadEditorDocumentHandler.SCHEMASHIPPEDFOLDER, IMSDocumentHandler.IMSCP_112), projectFolder, null);
            }
        }
        catch(IOException ex) {
            throw new ReloadEditorException(ex);
        }
    }
}