/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.contentpackaging;

import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.filechooser.*;

import uk.ac.reload.editor.contentpackaging.htmlparser.*;
import uk.ac.reload.editor.*;

/**
 * A ReloadResource
 *
 * @author Paul Sharples
 * @author Phillip Beauvoir
 * @version 2003-07-01
 */
public class ReloadResource extends File {

    /**
     * Constructor
     * @param resource
     */
    public ReloadResource(File resource) {
        super(resource.toString());
    }

    /**
     * Method to determine what dependencies are needed for a given
     * ReloadResource.
     * @return - an array of File references
     */
    public File[] getDependencies(File projectFolder){
        HTMLParser dep = new HTMLParser(this, projectFolder, false);
/*
        if(ReloadEditor.DEBUG) System.out.println("*********************");
        Iterator listElement;
        listElement = dep.getAllLinks().iterator();
        while (listElement.hasNext()) {
            if(ReloadEditor.DEBUG) System.out.println("returned: "+listElement.next().toString()+"\n");
        }
        if(ReloadEditor.DEBUG) System.out.println("*********************");
 */
        return dep.getAllLinks();
    }

    /**
     * @return a list of child ReloadResources.
     * Returns null if this is not a folder and no children
     * Puts folders first
     */
    public ReloadResource[] getChildren() {
        if(!isDirectory()) return null;

        // Do this in two passes so we can do the folders first

        Vector v = new Vector();

        File[] files = listFiles();

        // Folders
        for(int i = 0; i < files.length; i++) {
            File file = files[i];
            if(file.isDirectory()) {
                v.add(new ReloadResource(files[i]));
            }
        }

        // Files
        for(int i = 0; i < files.length; i++) {
            File file = files[i];
            if(!file.isDirectory()){
                v.add(new ReloadResource(files[i]));
            }
        }

        ReloadResource[] resources = new ReloadResource[v.size()];
        v.copyInto(resources);
        return resources;
    }


    /**
     * Method to check if this reload resources parent folders exist
     * @return boolean
     */
    public boolean doParentDirectoriesExist(){
        return getParentFile().exists();
    }

    /**
     * Method to return the friendly name for a ReloadResource.
     */
    public String getFriendlyName(){
        String s = "";
        if(exists()) {
            FileSystemView view = FileSystemView.getFileSystemView();
            s = view.getSystemTypeDescription(this);
        }
        return s;
    }

    /**
     * Method to retrieve which icon to use in the resources tree
     */
    public Icon getResourceIcon(){
        Icon icon = null;
        if(exists()) {
            FileSystemView view = FileSystemView.getFileSystemView();
            icon = view.getSystemIcon(this);
        }
        return icon;
    }

    /**
     * The string (filename) to show next to the icon in the resources tree
     * @return the filename to show next to the icon in the resource tree
     */
    public String toString(){
        return getName();
    }

}
