/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */


package uk.ac.reload.editor.contentpackaging;

import java.io.*;

import org.jdom.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.moonunit.schema.*;
import uk.ac.reload.moonunit.*;

/**
 * The SCORM 1.2 Content Package Class
 *
 * @author Phillip Beauvoir
 * @version 2003-09-26
 */
public class SCORM1_2Package extends ContentPackage {
    /**
     * Our unique signature
     */
    static final String[] scorm_comments = {
        "This is a Reload version " + ReloadEditor.VERSION + " SCORM 1.2 Content Package document",
        "Spawned from the Reload Content Package Generator - http://www.reload.ac.uk"
    };

    /**
     * Default Constructor
     */
    public SCORM1_2Package() {
        super();
    }

    /**
     * Constructor
     */
    public SCORM1_2Package(File file, Document doc, ContentPackageSchemaController schemaController) {
        super(file, doc, schemaController);
    }

    /**
     * Setup the JDOM Document with Namespaces set and root element set and some
     * default attributes.
     */
    protected void newDocument(File projectFolder, ContentPackageSchemaController cpController, MetadataSchemaController mdController) throws ReloadEditorException {
        super.newDocument(projectFolder, cpController, mdController);

        Element root = getDocument().getRootElement();

        ReloadSchema cpSchema = cpController.getReloadSchema();

        // Add ADL SCORM Namespace
        root.addNamespaceDeclaration(IMSDocumentHandler.ADLCP_NAMESPACE_12);

        // Add Schema Location Attribute
        Attribute att = root.getAttribute(IMSDocumentHandler.XSI_SchemaLocation, IMSDocumentHandler.XSI_Namespace);
        if(att != null) {
            String value = att.getValue();
            if(value == null) value = "";
            StringBuffer schemaLocationURI = new StringBuffer(value);
            schemaLocationURI.append(" ");
            schemaLocationURI.append(IMSDocumentHandler.ADLCP_NAMESPACE_12.getURI());
            schemaLocationURI.append(" ");
            schemaLocationURI.append(cpSchema.getSchemaName(IMSDocumentHandler.ADLCP_NAMESPACE_12.getURI()));
            att.setValue(schemaLocationURI.toString());
        }
    }

    /**
     * @return the Comments to add to the XML Document
     */
    public String[] getComments() {
        return scorm_comments;
    }
}