/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */


package uk.ac.reload.editor.gui;

import java.awt.*;

import org.jdom.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.schema.*;

/**
 * An Edit Field Form Text widget abstract class
 *
 * @author Phillip Beauvoir
 * @version 2003-10-22
 */
public abstract class ReloadFormField {
    /**
     * The owner
     */
    protected ReloadSchemaDocument _reloadDocument;

    /**
     * The Element we represent
     */
    protected Element _element;

    /**
     * Our schema template
     */
    protected ReloadSchemaElement _schemaElement;

    /**
     * Default Constructor
     */
    public ReloadFormField() { }

    /**
     * Constructor
     * @param reloadDocument the owner document
     * @param schemaElement the schema template bound to this element
     */
    public ReloadFormField(ReloadSchemaDocument reloadDocument, ReloadSchemaElement schemaElement) {
        _reloadDocument = reloadDocument;
        _schemaElement = schemaElement;
    }

    /**
     * Constructor
     * @param reloadDocument the owner document
     * @param element the element we represent
     * @param schemaElement the schema template bound to this element
     */
    public ReloadFormField(ReloadSchemaDocument reloadDocument, Element element, ReloadSchemaElement schemaElement) {
        _reloadDocument = reloadDocument;
        _element = element;
        _schemaElement = schemaElement;
    }

    /**
     * Get the value of this Widget
     * @return The value contained in the Widget
     */
    public abstract String getValue();

    /**
     * Set the value of the Widget
     * @param value The value to be set to this widget
     */
    public abstract void setValue(String value);

    /**
     * Get the component used
     * @return the component used
     */
    public abstract Component getComponent();

    /**
     * Clean up
     */
    public abstract void destroy();
}