/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.metadata;

import java.io.*;
import javax.swing.*;

import org.jdom.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.moonunit.*;
import uk.ac.reload.moonunit.dom.metadata.*;
import uk.ac.reload.moonunit.schema.*;

/**
 * The Metadata Schema Controller - this interfaces from the various helpers
 *
 * @author Phillip Beauvoir
 * @version 2003-11-04
 */
public class MetadataSchemaController
extends ReloadEditorSchemaController
{

    /**
     * Constructor with Default Profile
     * @param fileSchema the schema file
     * @param version the schema version
     */
    public MetadataSchemaController(File fileSchema, String version) {
        try {
            setVersion(version);

            loadSchema(fileSchema);

            // Get the Default Profile
            MetadataHelperProfile defProfile = MetadataHelperProfile.getDefaultMetadataHelperProfile();
            setHelperProfile(defProfile);
        }
        catch(ReloadEditorException ex) {
            ErrorDialogBox.showWarning("Could not load Default Metadata Profile", "Load Profile", ex);
        }
    }

    /**
     * Get the default VocabularySchemaController
     * @return the Default MetadataSchemaController
     */
    public static MetadataSchemaController getDefaultSchemaController() {
        return (MetadataSchemaController)ReloadEditorSchemaController.getReloadSchemaController(IMSDocumentHandler.IMSMD_122);
    }

    /**
     * Get the Metadata Root Element name
     * @return the root element name
     */
    public String getRootElementName() {
        return MetadataCore.ROOT_NAME;
    }

    /**
     * Get the top level IMS Element Categories (9 in this case)
     * @return An enumeration of ReloadSchemaElement objects
     */
    public ReloadSchemaElement[] getTopLevelElements() {
        return _reloadSchema.getRootElement().getChildren();
    }

    /**
     * Find out if the given Element is a top level one below "lom"
     * @param schemaElement The Schema Element to query
     * @return true if the parent of schemaElement is the root Element, false otherwise
     */
    public boolean isTopLevelElement(ReloadSchemaElement schemaElement) {
        return schemaElement.getParent() == _reloadSchema.getRootElement();
    }

    /**
     * Get The Leaf Icon (for the Full View Tree)
     * @param element the leaf element
     * @return the leaf icon
     */
    public Icon getLeafIcon(Element element) {
        String elementName = element.getName();
        if(elementName.equals(MetadataCore.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadEditor.ICON_MD);
        else return ReloadUIManager.getIcon(ReloadEditor.ICON_MDLEAF);
    }

    /**
     * Get The Leaf Icon (for the Full View Tree)
     * @param schemaElement the leaf schema element
     * @return the leaf icon
     */
    public Icon getLeafIcon(ReloadSchemaElement schemaElement) {
        String elementName = schemaElement.getName();
        if(elementName.equals(MetadataCore.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadEditor.ICON_MD);
        else return ReloadUIManager.getIcon(ReloadEditor.ICON_MDLEAF);
    }

    /**
     * Get The Open Icon (for the Full View Tree)
     * @param element the open element
     * @return the open icon
     */
    public Icon getOpenIcon(Element element) {
        String elementName = element.getName();
        if(elementName.equals(MetadataCore.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadEditor.ICON_MD);
        else return ReloadUIManager.getIcon(ReloadEditor.ICON_MDFOLDER);
    }

    /**
     * Get The Open Icon (for the Full View Tree)
     * @param schemaElement the open schema element
     * @return the open icon
     */
    public Icon getOpenIcon(ReloadSchemaElement schemaElement) {
        String elementName = schemaElement.getName();
        if(elementName.equals(MetadataCore.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadEditor.ICON_MD);
        else return ReloadUIManager.getIcon(ReloadEditor.ICON_MDFOLDER);
    }

    /**
     * Get The Closed Icon (for the Full View Tree)
     * @param element the closed element
     * @return the close icon
     */
    public Icon getClosedIcon(Element element) {
        return getOpenIcon(element);
    }

    /**
     * Get The Closed Icon (for the Full View Tree)
     * @param schemaElement the closed schemaElement
     * @return the close icon
     */
    public Icon getClosedIcon(ReloadSchemaElement schemaElement) {
        return getLeafIcon(schemaElement);
    }
}