/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.metadata.formview;

import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.moonunit.schema.*;

/**
 * The Metadata Form View Model
 * This reads in a MD User Profile and provides the Elements and form view mappings
 *
 * @author Phillip Beauvoir
 * @version 2003-04-23
 */
public class MetadataFormModel {
    /**
     * The Metadata Document
     */
    protected Metadata _metadata;

    /**
     * The Metadata Schema Controller
     */
    protected MetadataSchemaController _schemaController;

    /**
     * The Metadata Helper Profile
     */
    protected MetadataHelperProfile _mdProfile;


    /**
     * Default constructor
     */
    public MetadataFormModel() {

    }

    /**
     * Update the Document
     * @param metadata the metadata document
     */
    public void setDocument(Metadata metadata) {
        _metadata = metadata;
        _schemaController = (MetadataSchemaController)_metadata.getReloadSchemaController();
        _mdProfile = (MetadataHelperProfile)_schemaController.getReloadHelperProfile();
    }

    /**
     * Get the Root Element of the Metadata Profile
     * @return the root element of the metadata profile
     */
    public MetadataProfileElement getRootElement() {
        return _mdProfile.getRootElement();
    }

    /**
     * Create a suitable component for a Schema Element or Attribute
     * @param profileElement The profileElement for which to make the widget
     * @return The MetadataField Widget
     */
    public MetadataField createWidget(MetadataProfileElement profileElement) {
        ReloadSchemaNode schemaNode = _schemaController.getReloadSchemaNode(profileElement.getDomPath());

        // Attribute
        //if(schemaNode instanceof ReloadSchemaAttribute)
        //    return createAttributeWidget((ReloadSchemaAttribute)schemaNode);

        // Element
        if(schemaNode instanceof ReloadSchemaElement)
            return createElementWidget((ReloadSchemaElement)schemaNode);

        else return null;
    }

    /**
     * Create a widget bound to an Element
     * @param schemaElement The Element to Widgetise
     * @return A MetadataField widget
     */
    protected MetadataField createElementWidget(ReloadSchemaElement schemaElement) {
        // Do we have a vocabulary?
        ReloadVocabularyList rvList = _schemaController.getVocabularyList(schemaElement);

        // If we do, make a combo box
        if(rvList != null) return new MetadataComboBox(_metadata, schemaElement, rvList);

        // Else make a Text Field
        else return new MetadataTextField(_metadata, schemaElement);
    }

    /**
     * Create a widget bound to an Element's Attribute
     * @param att The Attribute to Widgetise
     * @return A MetadataField widget
     */
/*    protected MetadataField createAttributeWidget(ReloadSchemaAttribute att) {
        // Do we have a vocabulary?
        ReloadVocabularyList rvList = _schemaController.getVocabularyList(att);

        // If we do, make a combo box
        if(rvList != null) return new MetadataComboBox(att, metadata, rvList);

        // Else make a StringPanel
        else return new MetadataTextField(att, metadata);
    }
*/
}