/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.metadata.tabbedformview;

import uk.ac.reload.dweezil.gui.layout.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.moonunit.dom.metadata.*;
import uk.ac.reload.moonunit.schema.*;

/**
 * A Metadata Form Grouping Panel for the "General" Element
 *
 * @author Phillip Beauvoir
 * @version 2003-10-25
 */
public class GeneralPanel
extends MetadataGroupPanel
{
    /**
     * Title Table
     */
    protected LangStringTableWidget _titleTable;

    /**
     * Description Table
     */
    protected LangStringTableWidget _descriptionTable;

    /**
     * Keyword Table
     */
    protected LangStringTableWidget _keywordTable;

    /**
     * Constructor
     */
    public GeneralPanel(Metadata metadata, ReloadSchemaElement schemaElement) {
        super(metadata, schemaElement);
        constructView(metadata, schemaElement);
    }

    /**
     * Construct the View
     */
    protected void constructView(Metadata metadata, ReloadSchemaElement schemaElement) {
        setLayout(new XYLayout());

        int x = 10; int y = 10;

        _titleTable = new LangStringTableWidget(metadata, schemaElement.getChild(MetadataCore.TITLE), 1000, 200); // 400 -> 1000, 110 -> 200
        add(_titleTable, new XYConstraints(x, y, 0, 0));
        y += 200;

        _descriptionTable = new LangStringTableWidget(metadata, schemaElement.getChild(MetadataCore.DESCRIPTION), 1000, 200); // 400 -> 1000, 160 -> 200
        add(_descriptionTable, new XYConstraints(x, y, 0, 0));
        y += 200;

        _keywordTable = new LangStringTableWidget(metadata, schemaElement.getChild(MetadataCore.KEYWORD), 1000, 200); // 400 -> 1000, 160 -> 200
        add(_keywordTable, new XYConstraints(x, y, 0, 0));
        y += 200;

    }


    /**
     * Clean up
     */
    public void destroy() {
        _keywordTable.destroy();
    }
}