/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.metadata.tabbedformview;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import org.jdom.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.gui.layout.*;
import uk.ac.reload.dweezil.gui.widgets.*;
import uk.ac.reload.dweezil.menu.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.moonunit.dom.metadata.*;
import uk.ac.reload.editor.metadata.formview.*;
import uk.ac.reload.moonunit.schema.*;

/**
 * A Metadata Form Widget for the Metadata UI - this one allows multiple LangStrings
 *
 * @author Phillip Beauvoir
 * @version 2003-10-25
 */
public class LangStringTableWidget
extends JPanel
implements MetadataFormWidget
{
    /**
     * The Metadata
     */
    protected Metadata _metadata;

    /**
     * The Controller
     */
    protected MetadataSchemaController _schemaController;

    /**
     * The Parent ReloadSchemaElement
     */
    protected ReloadSchemaElement _schemaElement;

    /**
     * The Width
     */
    protected int _width;

    /**
     * The Height
     */
    protected int _height;

    private GroupPanel _groupPanel;

    private JPanel mainPanel;

    private Vector _langFields;



    /**
     * Default Constructor
     */
    public LangStringTableWidget(Metadata metadata, ReloadSchemaElement schemaElement, int width, int height) {
        _metadata = metadata;
        _schemaController = (MetadataSchemaController)_metadata.getReloadSchemaController();
        _schemaElement = schemaElement;

        _width = width;
        _height = height;

        constructView();
    }


    /**
     * Construct the View
     */
    protected void constructView() {
        setLayout(new XYLayout());

        String name = _schemaController.getElementFriendlyName(_schemaElement.getDomPath());
        setBorder(BorderFactory.createTitledBorder(name));

        Action_Plus actionPlus = new Action_Plus();
        // Action_Plus2 actionPlus2 = new Action_Plus2(); // old version
        Action_Minus actionMinus = new Action_Minus();

        ReloadToolBar toolBar = new ReloadToolBar();
        toolBar.add(actionPlus);
        //toolBar.add(actionPlus2);
        toolBar.add(actionMinus);
        add(toolBar, new XYConstraints(0, -5, 0, 0));

        mainPanel = new JPanel(new GridBagLayout());
        mainPanel.setBorder(BorderFactory.createEmptyBorder());

        JScrollPane sp1 = new JScrollPane(mainPanel);
        sp1.setBorder(BorderFactory.createEmptyBorder());
        add(sp1, new XYConstraints(0, 20, _width - 18, _height - 55));

        // Get the Elements to display
        Element[] elements = _metadata.getElements(_schemaElement);

		if(ReloadEditor.DEBUG) System.out.println("schemaElement = " + _schemaElement.getName() + " domPath = " + _schemaElement.getDomPath().getPath());

        // Group
        if(elements != null) {
            for(int i = 0; i < elements.length; i++) {
                // For subPanel
                GridBagConstraints constraints = new GridBagConstraints();
                constraints.insets = new Insets(0, 3, 10, 3);
                constraints.gridx = 0;
                constraints.gridy = i;
                constraints.anchor = GridBagConstraints.WEST;
                constraints.fill = GridBagConstraints.HORIZONTAL;
                constraints.weightx = 1.0;
                constraints.weighty = 0.0;

				// add subPanel to mainPanel
                JPanel subPanel = new JPanel(new GridBagLayout());
                subPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
                mainPanel.add(subPanel, constraints);

                // For text elements
                constraints = new GridBagConstraints();
                constraints.gridy = 0;
                constraints.anchor = GridBagConstraints.WEST;
                constraints.fill = GridBagConstraints.HORIZONTAL;
                constraints.weightx = 1.0;
                constraints.weighty = 0.0;

                // Entry
                Iterator it = elements[i].getChildren(MetadataCore.LANGSTRING, _schemaElement.getNamespace()).iterator();
                while(it.hasNext()) {
                    Element element = (Element) it.next();

                    constraints.gridx = 0;
                    constraints.insets = new Insets(0, 0, 0, 3);

                    ReloadTextArea textArea = new ReloadTextArea();
                    textArea.setTextValue(element.getText());
                    textArea.setBorder(BorderFactory.createLineBorder(Color.gray));
                    subPanel.add(textArea, constraints);

                    constraints.gridx = 1;
                    constraints.insets = new Insets(0, 0, 0, 0);

                    ReloadComboBox combo = new ReloadComboBox(new String[] {"en", "fr", "de", "nl"});
                    combo.setEditable(true);
                    combo.setPreferredSize(new Dimension(40, combo.getPreferredSize().height));
                    subPanel.add(combo, constraints);
                    String att = element.getAttributeValue("lang", Namespace.XML_NAMESPACE);
                    if(att != null) combo.setSelectedItem(att);

                    constraints.gridy++;
                }
            }
        }
        else {
			_groupPanel = new GroupPanel(_schemaElement);
        	revalidate();
		}


    }

    /**
     * @return preferred size
     */
    public Dimension getPreferredSize() {
        return new Dimension(_width, _height);
    }

    /**
     * Clean up
     */
    public void destroy() {

    }

    class Action_Plus extends MenuAction
    {
        public Action_Plus() {
            super("New Entry", ReloadEditor.ICON_PLUS_SM);
        }

        public void actionPerformed(ActionEvent e) {

			if(ReloadEditor.DEBUG) System.out.println("Pressed: +");

			if(ReloadEditor.DEBUG) System.out.println("ADD another GROUP");

            // Get the Children
            ReloadSchemaElement[] children = _schemaElement.getChildren(_schemaElement.getNamespace());
            for(int i = 0; i < children.length; i++) {
                ReloadSchemaElement child = children[i];

                if(ReloadEditor.DEBUG) System.out.println("_schemaElement = " + _schemaElement.getName() + " child = " + child.getName());

                // add a field
                _groupPanel.addField(child);
            }

			revalidate();
        }
    }

	/*
    class Action_Plus2 extends MenuAction
    {
        public Action_Plus2() {
            super("New Group", ReloadEditor.ICON_PLUS2_SM);
            //setEnabled(false);
        }

        public void actionPerformed(ActionEvent e) {

        }
    }
    */

    class Action_Minus extends MenuAction
    {
        public Action_Minus() {
            super("Delete", ReloadEditor.ICON_MINUS_SM);
        }

        public void actionPerformed(ActionEvent e) {

			if(ReloadEditor.DEBUG) System.out.println("Pressed: -");
        }
    }

    /**
     * A Panel that uses GridBagLayout to set out its components
     */
    class GroupPanel extends JPanel {
        /**
         * The Constraints
         */
        protected GridBagConstraints constraints;

        public GroupPanel(ReloadSchemaElement schemaElement) {
            // Set up the constraints
            setLayout(new GridBagLayout());
            constraints = new GridBagConstraints();
            constraints.insets = new Insets(0, 3, 10, 3);
            constraints.gridx = 0;
            constraints.gridy = 0;
            constraints.anchor = GridBagConstraints.WEST;
            constraints.fill = GridBagConstraints.HORIZONTAL;
            constraints.weightx = 1.0;
            constraints.weighty = 0.0;

			// add this panel to mainPanel
            setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
            mainPanel.add(GroupPanel.this, constraints);


            // For text elements
            constraints = new GridBagConstraints();
            constraints.gridy = 1;
            constraints.anchor = GridBagConstraints.WEST;
            constraints.fill = GridBagConstraints.HORIZONTAL;
            constraints.weightx = 1.0;
            constraints.weighty = 0.0;


            // Get the Children
            ReloadSchemaElement[] children = _schemaElement.getChildren(_schemaElement.getNamespace());
            for(int i = 0; i < children.length; i++) {
                ReloadSchemaElement child = children[i];

                if(ReloadEditor.DEBUG) System.out.println("_schemaElement = " + _schemaElement.getName() + " child = " + child.getName());

                // add a field
                addField(child);
            }
        }

        /**
         * Add a single field for an Element
         * @param profileElement
         */
        protected void addField(ReloadSchemaElement schemaElement) {

            constraints.gridx = 0;
            constraints.insets = new Insets(0, 0, 0, 3);

            MetadataTextField textField = new MetadataTextField(_metadata, schemaElement);
            //textField.setBorder(BorderFactory.createLineBorder(Color.gray));
            add(textField.getComponent(), constraints);

            String[] languages = new String[] {"en", "fr", "de", "nl"};
            ReloadToolBar langsToolBar = new ReloadToolBar();

            _langFields = new Vector();

            MetadataTextField textField2;
			constraints.gridx = 1;

            for(int i = 0; i < languages.length; i++) {

	            textField2 = new MetadataTextField(_metadata, schemaElement);

	            //textField2.setBorder(BorderFactory.createLineBorder(Color.gray));
	            add(textField2.getComponent(), constraints);

	            if(i == 0) textField2.getComponent().setVisible(true);
	            else textField2.getComponent().setVisible(false);

	            _langFields.add(textField2.getComponent());

				Action_Lang action_lang = new Action_Lang(languages[i], textField2);
				langsToolBar.add(action_lang);
			}

			constraints.gridy--;
            constraints.insets = new Insets(0, 0, 0, 0);
            add(langsToolBar, constraints);

            constraints.gridy += 2;
        }

    }

    class Action_Lang extends MenuAction
    {
		private MetadataTextField _target;

        public Action_Lang(String text, MetadataTextField target) {
            super(text);
            setButtonText(text);

            _target = target;
        }

        public void actionPerformed(ActionEvent e) {

			for(int i = 0; i < _langFields.size(); i++) {

				JTextArea tf = (JTextArea) _langFields.elementAt(i);

				tf.setVisible(false);
			}

			// makes the target textfield visible
			_target.getComponent().setVisible(true);

			// sets the value of the Reload Widget
			_target.setValue(getButton().getText());
        }
    }

}