/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.metadata.tabbedformview;

import java.awt.*;
import javax.swing.*;

import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.editor.metadata.tableview.*;
import uk.ac.reload.moonunit.dom.metadata.*;
import uk.ac.reload.moonunit.schema.*;

/**
 * A Metadata Form Panel for the Metadata UI
 *
 * @author Phillip Beauvoir
 * @version 2003-10-22
 */
public class MetadataTabbedFormPanel
extends JPanel
{
    /**
     * The Metadata
     */
    protected Metadata _metadata;

    /**
     * The Controller
     */
    protected MetadataSchemaController _schemaController;

    /**
     * The Metadata Profile
     */
    protected MetadataHelperProfile _mdProfile;

    /**
     * The root JTabbedPane
     */
    protected JTabbedPane _tabPane;

    /**
     * TableTree Panel
     */
    protected MetadataTablePanel _tablePanel;

    /**
     * Constructor
     */
    public MetadataTabbedFormPanel(MetadataTablePanel tablePanel) {
        setLayout(new BorderLayout());
        _tabPane = new JTabbedPane();
        add(_tabPane, BorderLayout.CENTER);
        _tablePanel = tablePanel;
    }

    /**
     * Set the view up to display the Metadata Document
     */
    public void setDocument(Metadata metadata) {
        // Clean up
        destroy();

        // Save selected Tab Pane
        int selected = _tabPane.getSelectedIndex();

        // Clear Tab Pane
        _tabPane.removeAll();

        _metadata = metadata;
        _schemaController = (MetadataSchemaController)_metadata.getReloadSchemaController();
        _mdProfile = (MetadataHelperProfile)_schemaController.getReloadHelperProfile();

        // Add Panes
        ReloadSchemaElement[] topElements = _schemaController.getTopLevelElements();
        for(int i = 0; i < topElements.length; i++) {
            String name = _schemaController.getElementFriendlyName(topElements[i].getDomPath());
            String tip = _schemaController.getElementTip(topElements[i].getDomPath());
            MetadataGroupPanel panel = createPanel(metadata, topElements[i]);
            _tabPane.addTab(name, null, panel, tip);
        }

        _tabPane.addTab("Tree View", null, _tablePanel);

        // Restore selected pane
        if(selected != -1) _tabPane.setSelectedIndex(selected);

        revalidate();
    }


    /**
     * @return an appropriate Panel depending on the Schema
     */
    protected MetadataGroupPanel createPanel(Metadata metadata, ReloadSchemaElement schemaElement) {
        MetadataGroupPanel panel = null;

        String name = schemaElement.getName();

        if(name.equals(MetadataCore.GENERAL)) panel = new GeneralPanel(_metadata, schemaElement);
        //else if(name.equals(MetadataCore.LIFECYCLE)) panel = new LifeCyclePanel(_metadata, schemaElement);
        //else if(name.equals(MetadataCore.METAMETADATA)) panel = new MetametadataPanel(_metadata, schemaElement);
        //else if(name.equals(MetadataCore.TECHNICAL)) panel = new TechnicalPanel(_metadata, schemaElement);
        //else if(name.equals(MetadataCore.EDUCATIONAL)) panel = new EducationalPanel(_metadata, schemaElement);
        //else if(name.equals(MetadataCore.RIGHTS)) panel = new RightsPanel(_metadata, schemaElement);
        //else if(name.equals(MetadataCore.RELATION)) panel = new RelationPanel(_metadata, schemaElement);
        //else if(name.equals(MetadataCore.ANNOTATION)) panel = new AnnotationPanel(_metadata, schemaElement);
        //else if(name.equals(MetadataCore.CLASSIFICATION)) panel = new ClassificationPanel(_metadata, schemaElement);

        return panel;
    }

    /**
     * Clean up
     */
    public void destroy() {
        for(int i = 0; i < _tabPane.getTabCount(); i++) {
            Component c = _tabPane.getComponentAt(i);
            if(c instanceof MetadataFormWidget) ((MetadataFormWidget)c).destroy();
        }
    }
}