/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.metadata.vdexBrowser;

import org.jdom.*;

import uk.ac.reload.dweezil.gui.tree.*;


/**
 * A node of the VdexTree. It is used for wrapping a Vdex Term element.
 *
 * @author Tina Manoharan
 * @version 2002-09-11
 */
public class VdexTreeNode
extends ReloadTreeNode
{

	/**
	 * The Vdex Term element
	 */
	private Element element;

    /**
     * Constructor
     * @param nodeName the name of node (term_id:term_caption)
     *		  element the term element
     */
    public VdexTreeNode(String nodeName, Element element) {

        super(nodeName);

        this.element = element;
    }

	/**
	 * Set the term element
	 */
    public void setElement(Element element) {

		this.element = element;
	}

	/**
	 * Get the term element
	 * @return the term element
	 */
	public Element getElement() {

		return element;
	}

	/**
	 * Set the name of the node
	 */
	public void setNodeName(String nodeName) {

		setUserObject(nodeName);
	}

	/**
	 * Get the node name
	 * @return the node name
	 */
	public String getNodeName() {

		return (String) getUserObject();
	}

}