/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.prefs;

import javax.swing.*;

import uk.ac.reload.dweezil.gui.layout.*;
import uk.ac.reload.dweezil.prefs.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.editor.contentpackaging.*;
import java.io.*;
import uk.ac.reload.editor.*;



/**
 * A Content Packaging Settings User Prefs Panel
 *
 * @author Phillip Beauvoir
 * @version 2003-10-02
 */
public class CP_PrefsPanel
extends PrefsPanel
{
    /**
     * Hide Resources check box
     */
    protected JCheckBox _checkbox_hideResources;

    /**
     * Hide SCORM Elements check box
     */
    protected JCheckBox _checkbox_hideSCORM;

    /**
     * Combo Box for Default CP Profile
     */
    protected ReloadComboBox _profileCPCombobox;

    /**
     * Combo Box for Default SCORM Profile
     */
    protected ReloadComboBox _profileSCORMCombobox;

    /**
     * Constructor
     */
    public CP_PrefsPanel() {
        setLayout(new XYLayout());
        constructFields();
    }

    /**
     * Set up the fields
     */
    private void constructFields() {
        int x = 5;
        int y = 10;

        // ============ Hide Resources ==================
        _checkbox_hideResources = new JCheckBox("Hide Resources node (effective the next time you open a Manifest)");
        add(_checkbox_hideResources, new XYConstraints(x, y, 0, CHECKBOX_HEIGHT));

        y += 20;

        // ============ Hide SCORM ==================
        _checkbox_hideSCORM = new JCheckBox("Hide SCORM nodes (effective the next time you open a Manifest)");
        add(_checkbox_hideSCORM, new XYConstraints(x, y, 0, CHECKBOX_HEIGHT));

        y += 30;

        // ============ Default CP Profile ==================

        x = 10;

        JLabel label = new JLabel("Default IMS Profile:");
        add(label, new XYConstraints(x, y + 2, 0, 0));

        _profileCPCombobox = new ReloadComboBox(false);
        add(_profileCPCombobox, new XYConstraints(x += 120, y, 300, COMBOBOX_HEIGHT));

        // Populate it
        ContentPackageHelperProfile[] cp_profiles = ContentPackageHelperProfile.getContentPackageHelperProfiles();
        _profileCPCombobox.setItems(cp_profiles);

        y += 30;

        // ============ Default SCORM Profile ==================

        x = 10;

        label = new JLabel("Default SCORM Profile:");
        add(label, new XYConstraints(x, y + 2, 0, 0));

        _profileSCORMCombobox = new ReloadComboBox(false);
        add(_profileSCORMCombobox, new XYConstraints(x += 120, y, 300, COMBOBOX_HEIGHT));

        // Populate it
        SCORM1_2HelperProfile[] scorm_profiles = SCORM1_2HelperProfile.getSCORM1_2HelperProfiles();
        _profileSCORMCombobox.setItems(scorm_profiles);

        y += 20;
    }

    /**
     * Update UserPrefs according to the controls' settings
     * @param prefs the UserPrefs instance
     * @see UserPrefs
     */
    public void saveToUserPrefs(UserPrefs prefs) {
        prefs.putBooleanValue(ReloadEditorPrefs.CP_HIDE_RESOURCES, _checkbox_hideResources.isSelected());
        prefs.putBooleanValue(ReloadEditorPrefs.CP_HIDE_SCORM, _checkbox_hideSCORM.isSelected());

        // Default CP Profile
        ContentPackageHelperProfile cp_profile = (ContentPackageHelperProfile)_profileCPCombobox.getSelectedItem();
        prefs.putValue(ReloadEditorPrefs.CP_DEFAULT_PROFILE, cp_profile.getFile().getName());

        // Default SCORM Profile
        SCORM1_2HelperProfile scorm_profile = (SCORM1_2HelperProfile)_profileSCORMCombobox.getSelectedItem();
        prefs.putValue(ReloadEditorPrefs.SCORM_DEFAULT_PROFILE, scorm_profile.getFile().getName());
    }

    /**
     * Set the controls' settings from those found in UserPrefs
     * @param prefs the UserPrefs instance
     * @see UserPrefs
     */
    public void setFields(UserPrefs prefs) {
        _checkbox_hideResources.setSelected(prefs.getBooleanValue(ReloadEditorPrefs.CP_HIDE_RESOURCES));
        _checkbox_hideSCORM.setSelected(prefs.getBooleanValue(ReloadEditorPrefs.CP_HIDE_SCORM));

        // Default CP Profile
        String fileName = prefs.getValue(ReloadEditorPrefs.CP_DEFAULT_PROFILE);
        if(fileName != null) {
            File file = new File(ReloadEditorDocumentHandler.folderCPProfile, fileName);
            try {
                ContentPackageHelperProfile profile = ContentPackageHelperProfile.getContentPackageProfile(file);
                if(profile != null) _profileCPCombobox.setSelectedItem(profile);
            }
            catch(ReloadEditorException ex) {
            }
        }

        // Default SCORM Profile
        fileName = prefs.getValue(ReloadEditorPrefs.SCORM_DEFAULT_PROFILE);
        if(fileName != null) {
            File file = new File(ReloadEditorDocumentHandler.folderSCORMProfile, fileName);
            try {
                SCORM1_2HelperProfile profile = SCORM1_2HelperProfile.getSCORM1_2Profile(file);
                if(profile != null) _profileSCORMCombobox.setSelectedItem(profile);
            }
            catch(ReloadEditorException ex) {
            }
        }

    }

}
