/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.prefs;

import javax.swing.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.gui.layout.*;
import uk.ac.reload.dweezil.gui.widgets.*;
import uk.ac.reload.dweezil.prefs.*;
import uk.ac.reload.editor.*;



/**
 * A General Settings User Prefs Panel
 *
 * @author Phillip Beauvoir
 * @version 2003-09-18
 */
public class General_PrefsPanel
extends PrefsPanel
{
    /**
     * Default folder
     */
    private ReloadFileTextField _tfDefaultFolder;

    /**
     * Constructor
     */
    public General_PrefsPanel() {
        setLayout(new XYLayout());
        constructFields();
    }

    /**
     * Set up the fields
     */
    private void constructFields() {
        int x = 10;
        int y = 10;

        // ============ Default Open Folder ==================
        JLabel label = new JLabel("Default Open Folder:");
        add(label, new XYConstraints(x, y + 2, 0, 0));
        _tfDefaultFolder = new ReloadFileTextField(ReloadFileTextField.FOLDER_TYPE, "Select Default Open Folder", ReloadUIManager.getIcon(ReloadEditor.ICON_OPEN));
        add(_tfDefaultFolder, new XYConstraints(x += 110, y, 300, TEXTBOX_HEIGHT));

    }

    /**
     * Update UserPrefs according to the controls' settings
     * @param prefs the UserPrefs instance
     * @see UserPrefs
     */
    public void saveToUserPrefs(UserPrefs prefs) {
        prefs.putValue(ReloadEditorPrefs.GENERAL_PREFS_DEFAULT_FOLDER, _tfDefaultFolder.getTextValue());
    }

    /**
     * Set the fields' settings from those found in UserPrefs
     * @param prefs the UserPrefs instance
     * @see UserPrefs
     */
    public void setFields(UserPrefs prefs) {
        // Set value in field
        _tfDefaultFolder.setTextValue(prefs.getValue(ReloadEditorPrefs.GENERAL_PREFS_DEFAULT_FOLDER));

    }

}
