/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.vocabulary.formview;

import java.awt.*;
import java.awt.event.*;

import uk.ac.reload.editor.vocabulary.*;
import uk.ac.reload.moonunit.schema.*;
import uk.ac.reload.dweezil.gui.*;


/**
 * A Combo Box widget.  Derived from JComboBox.
 * This can hold an Element or an Attribute.
 *
 * @author Tina Manoharan
 * @version 2003-08-11
 */
public class VocabularyComboBox
extends VocabularyField
implements ActionListener
{
	/**
	 * An internal flag to notify listeners of element changes
	 */
    boolean allowNotification = true;

	/**
	 * The combo box for displaying element vocabulary list
	 */
    protected ReloadComboBox _comboBox;

	/**
	 * Constructor
	 * @param vocabulary the vocabulary JDOM document
	 * 	 	  schemaElement the ReloadSchemaElement bound to this VocabularyComboBox
	 *		  rvList the vocabulary list of this element
	 */
    public VocabularyComboBox(Vocabulary vocabulary, ReloadSchemaElement schemaElement, ReloadVocabularyList rvList) {
        super(vocabulary, schemaElement);

        _comboBox = new ReloadComboBox(true);
        _comboBox.setBorder(null);

        _comboBox.setItems(rvList.getList());

        _element = vocabulary.getElement(_schemaElement);
        if(_element != null) setValue(_element.getText());
        else setValue("");

        // We listen to Text Changes
        _comboBox.addActionListener(this);

        // We listen to Element changes
        vocabulary.addReloadDocumentListener(this);
    }

	/**
	 * Clean up
	 */
    public void destroy() {
        _reloadDocument.removeReloadDocumentListener(this);
        _comboBox.removeActionListener(this);
    }

    /**
     * Get the value of this Widget
     * @return The value contained in the Widget
     */
    public String getValue() {
        return (String)_comboBox.getSelectedItem();
    }

    /**
     * Set the value of the Widget
     * @param value the value to be set to the widget
     */
    public void setValue(String value) {
        allowNotification = false;
        _comboBox.setSelectedItem(value);
        allowNotification = true;
    }

	/**
	 * Get the ComboBox component
	 * @return the ComboBox component
	 */
    public Component getComponent() {

        return _comboBox;
    }

	/**
	 * Action to be performed when the ComboBox component's value is changed
	 */
    public void actionPerformed(ActionEvent event) {
        if(allowNotification && _comboBox != null) fireElementChanged(getValue());
    }
}