/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.vocabulary.importexport;

import java.io.*;

import org.jdom.*;

import javax.xml.transform.*;
import javax.xml.transform.stream.*;

//import com.icl.saxon.*;
//import com.icl.saxon.jdom.*;
//import com.icl.saxon.expr.*;
//import com.icl.saxon.om.*;

import org.jdom.transform.*;

import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.vocabulary.*;
import uk.ac.reload.moonunit.dom.*;

/**
 * Export Vdex Vocabulary as Zthes
 *
 * @author Tina Manoharan
 * @version 2003-11-15
 */
public class ZthesExporter {

	/**
	 * Constructor
	 * @param vdexFile the vdex file to export
	 */
	public ZthesExporter(File vdexFile) {

		try {

			// create source document

			Document vdexDoc = XMLUtils.readXMLFile(vdexFile);

	    	// Now do a transformation

    		File stylesheetFile = ReloadEditorDocumentHandler.VDEX_TO_ZTHES_STYLESHEET;

			Transformer transformer = TransformerFactory.newInstance().newTransformer(new StreamSource(stylesheetFile));


			// create result by performing transformation using stylesheet

			JDOMSource source = new JDOMSource(vdexDoc);

			JDOMResult result = new JDOMResult();

			transformer.transform(source, result);

			Document zthesDoc = result.getDocument();


			// save output file

	    	File zthesOutputFile = new File(ReloadEditorDocumentHandler.folderMDZthesVocab, vdexFile.getName());

	    	XMLUtils.write2XMLFile(zthesDoc, zthesOutputFile);

		}
        catch(TransformerConfigurationException tex) {
            if(ReloadEditor.DEBUG) tex.printStackTrace();
        }
        catch(TransformerException tex2) {
            if(ReloadEditor.DEBUG) tex2.printStackTrace();
        }
        catch(Exception rex) {
            if(ReloadEditor.DEBUG) rex.printStackTrace();
            ErrorDialogBox.showWarning("Export Vdex Vocabulary as Zthes", "Error exporting", rex);
        }
	}
}


/* Using Saxon XSLT Processor

	public ZthesExporter(File vdexFile) {

		try {

			Document vdexDoc = XMLUtils.readXMLFile(vdexFile);

			// Give it a Saxon wrapper
	    	DocumentWrapper saxonVdexDocWrap = new DocumentWrapper(vdexDoc, vdexFile.getAbsolutePath());
	    	NamePool pool = NamePool.getDefaultNamePool();
	    	saxonVdexDocWrap.setNamePool(pool);

	    	// Now do a transformation

	    	System.setProperty("javax.xml.transform.TransformerFactory", "com.icl.saxon.TransformerFactoryImpl");
	    	TransformerFactory tfactory = TransformerFactory.newInstance();

	    	String stylesheetFileName = System.getProperty("user.dir") + ReloadUIManager.VDEX_TO_ZTHES_STYLESHEET;

	    	Templates templates = tfactory.newTemplates(new StreamSource(stylesheetFileName));
	    	Transformer transformer = templates.newTransformer();

	    	File zthesOutputFile = new File(ReloadEditorDocumentHandler.folderMDZthesVocab, vdexFile.getName());

	    	transformer.transform(saxonVdexDocWrap, new StreamResult(zthesOutputFile));
		}
        catch(Exception rex) {
            if(ReloadEditor.DEBUG) rex.printStackTrace();
            ErrorDialogBox.showWarning("Export Vdex Vocabulary as Zthes", "Error exporting", rex);
        }
        catch(TransformerConfigurationException tex) {
            if(ReloadEditor.DEBUG) tex.printStackTrace();
        }
        catch(TransformerException tex2) {
            if(ReloadEditor.DEBUG) tex2.printStackTrace();
        }
	}

*/