/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.vocabulary.tableview;

import java.awt.*;
import javax.swing.*;

import uk.ac.reload.editor.gui.*;
import uk.ac.reload.editor.menu.*;
import uk.ac.reload.editor.vocabulary.*;

/**
 * A Vocabulary Table View Panel for the Vocabulary UI
 *
 * @author Tina Manoharan
 * @version 2003-08-11
 */
public class VocabularyTablePanel
extends JPanel
{
    /**
     * The Treetable
     */
    protected VocabularyTreeTable _treeTable;

    /**
     * The info Panel
     */
    protected ReloadInfoPanel _infoPanel;

    /**
     * ScrollPane
     */
    private JScrollPane _scrollPane;

    /**
     * Splitpane
     */
    private JSplitPane splitPane;

    /**
     * The Edit menu which we will use
     */
    protected Menu_Edit _editMenu;

    /**
     * Constructor
     * @param editMenu the Edit Menu for the Vocabulary Model that we will View/Edit
     */
    public VocabularyTablePanel(Menu_Edit editMenu) {
        _editMenu = editMenu;

        setLayout(new BorderLayout());

        // Create new ScrollPane
        _scrollPane = new JScrollPane();
        // Match background colour
        _scrollPane.getViewport().setBackground(UIManager.getColor("Table.background"));

        // Create new Info panel
        _infoPanel = new ReloadInfoPanel();

        // Set up a Split Pane
        splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        splitPane.setOneTouchExpandable(true);
        splitPane.setTopComponent(_scrollPane);
        splitPane.setBottomComponent(_infoPanel);
        add(splitPane, BorderLayout.CENTER);
    }

    /**
     * Set the view up to display the Vocabulary Document
     * @param vocabulary the vocabulary document
     */
    public void setDocument(Vocabulary vocabulary) {
        // Set up a TreeTable and Model
        VocabularyTreeTableModel model = new VocabularyTreeTableModel(vocabulary);
        _treeTable = new VocabularyTreeTable(model, _infoPanel, _editMenu);

        // Add it to the JScrollPane
        _scrollPane.setViewportView(_treeTable);

        // Select Root Node
        _treeTable.selectNode((VocabularyTreeNode)model.getRoot());
    }

    /**
     * Clean up
     */
    public void destroy() {
        if(_treeTable != null) _treeTable.destroy();
        if(_infoPanel != null) _infoPanel.destroy();
    }


    /**
     * Refresh the View
     */
    public void refresh() {
        _treeTable.refresh();
    }

    /**
     * Called after Panel is shown so we can init stuff
     */
    public void initView() {
        splitPane.setDividerLocation(0.7);
        _infoPanel.initView();
    }

}
