/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.ldeditor;

import java.io.*;

import javax.swing.*;

import org.jdom.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.moonunit.*;
import uk.ac.reload.moonunit.schema.*;


/**
 * The Reload Editor Schema Controller
 *
 * @author Phillip Beauvoir
 * @version 2003-10-15
 */
public abstract class LDEditorSchemaController
extends ReloadSchemaController
{
    /**
     * Constructor
     */
    public LDEditorSchemaController() {
        super();
    }

    /**
     * Constructor
     */
    public LDEditorSchemaController(File fileSchema, File fileHelper, File fileVocab) {
        loadSchema(fileSchema);
        loadSchemaHelper(fileHelper);
        loadVocab(fileVocab);
    }

    /**
     * Load the Schema and warn if need be
     */
    public void loadSchema(File fileSchema) {
        try {
            super.loadSchema(fileSchema);
        }
        catch(ReloadSchemaException ex) {
            ErrorDialogBox.showWarning("Could not load Schema: ", "Load Schema", ex);
        }
    }

    /**
     * Load the Schema Helper and warn if need be
     */
    public void loadSchemaHelper(File fileHelper) {
        try {
            super.loadSchemaHelper(fileHelper);
        }
        catch(ReloadSchemaException ex) {
            ErrorDialogBox.showWarning("Could not load Schema Helper: ", "Load SchemaHelper", ex);
        }
    }

    /**
     * Load the Vocab and warn if need be
     */
    public void loadVocab(File fileVocab) {
        try {
            super.loadVocab(fileVocab);
        }
        catch(ReloadSchemaException ex) {
            ErrorDialogBox.showWarning("Could not load Vocabulary: ", "Load Schema", ex);
        }
    }

    /**
     * @return an appropriate ReloadSchemaController for a given Namespace in an Instance ReloadDocument
     */
    public static ReloadSchemaController getReloadSchemaController(Document doc, Namespace ns) {
        // Try to correct the Namespace
        ns = IMSDocumentHandler.correctAdditionalNamespace(doc, ns);

        String version = IMSDocumentHandler.getSchemaVersionFromNamespace(ns);
        return getReloadSchemaController(version);
    }

    /**
     * @return a Schema Controller depending on a Schema version
     */
    public static ReloadSchemaController getReloadSchemaController(String schemaVersion) {
        ReloadSchemaController schemaController = null;

        if(schemaVersion != null) {
            // LEARNING DESIGN
            if(schemaVersion.equals(IMSDocumentHandler.IMSLD_100)) {
                schemaController = new LDSchemaController(LDEditorDocumentHandler.fileSchemaLDA1_0,
                    LDEditorDocumentHandler.fileHelperLD, LDEditorDocumentHandler.fileVocabLD);
            }

            else if(schemaVersion.equals(IMSDocumentHandler.IMSLD_100A)) {
                schemaController = new LDSchemaController(LDEditorDocumentHandler.fileSchemaLDA1_0,
                    LDEditorDocumentHandler.fileHelperLD, LDEditorDocumentHandler.fileVocabLD);
            }

            else if(schemaVersion.equals(IMSDocumentHandler.IMSLD_100B)) {
                schemaController = new LDSchemaController(LDEditorDocumentHandler.fileSchemaLDB1_0,
                    LDEditorDocumentHandler.fileHelperLD, LDEditorDocumentHandler.fileVocabLD);
            }

            else if(schemaVersion.equals(IMSDocumentHandler.IMSLD_100C)) {
                schemaController = new LDSchemaController(LDEditorDocumentHandler.fileSchemaLDC1_0,
                    LDEditorDocumentHandler.fileHelperLD, LDEditorDocumentHandler.fileVocabLD);
            }

        }

        return schemaController;
    }

    /**
     * @return a Leaf Icon for an Element
     */
    public abstract Icon getLeafIcon(Element element);

    /**
     * Return a Closed Icon for an Element
     */
    public abstract Icon getClosedIcon(Element element);

    /**
     * Return an Open Icon for an Element
     */
    public abstract Icon getOpenIcon(Element element);

    /**
     * @return a Leaf Icon for an Element
     */
    public abstract Icon getLeafIcon(ReloadSchemaElement schemaElement);

    /**
     * Return a Closed Icon for an Element
     */
    public abstract Icon getClosedIcon(ReloadSchemaElement schemaElement);

    /**
     * Return an Open Icon for an Element
     */
    public abstract Icon getOpenIcon(ReloadSchemaElement schemaElement);


 }