/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.ldeditor.menu;

import java.awt.event.*;
import javax.swing.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.menu.*;
import uk.ac.reload.moonunit.schema.*;
import uk.ac.reload.ldeditor.*;
import uk.ac.reload.ldeditor.schemaviewer.*;

/**
 * A Menu Item to display the Schema Node Viewer
 *
 * @author Phillip Beauvoir
 * @version 2003-10-15
 */
public class MenuAction_DebugSchema extends MenuAction {

    String _version;
    String _name;

    public MenuAction_DebugSchema(String version, String name) {
        super(name);
        _version = version;
        _name = name;
    }

    /**
     * Show a new window - we put this on a thread so we can close the menu item
     * @param e
     */
    public void actionPerformed(ActionEvent e) {
        Thread thread = new Thread() {
            public void run() {
                try {
                    LDEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

                    // Get ReloadSchemaController
                    ReloadSchemaController schemaController = LDEditorSchemaController.getReloadSchemaController(_version);

                    // New Internal window
                    final SchemaWindow testWindow = new SchemaWindow(schemaController, _name);

                    // Show the Window later in the Event queue
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            LDEditor.getInstance().addInternalFrame(testWindow);
                            testWindow.show();
                        }
                    });

                }
                catch(Exception ex) {
                    if(LDEditor.DEBUG) ex.printStackTrace();
                }
                finally {
                    LDEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
                }
            }
        };

        thread.start();
    }
}

