/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.ldeditor.prefs;

import java.io.*;
import java.util.*;

import uk.ac.reload.dweezil.prefs.*;

/**
 * The User Prefs file.  Implementers will have to fulfil the abstract methods
 *
 * @author Phillip Beauvoir
 * @version 2003-10-06
 */
public class LDEditorPrefs extends UserPrefs {

    // ======================== GENERAL KEYS ===================================

    // The default Open folder
    public static final String GENERAL_PREFS_DEFAULT_FOLDER = "default_folder";
    // Look and Feel
    public static final String GENERAL_PREFS_LF = "look_and_feel";
    // Recent File History
    public static final String GENERAL_HISTORY = "history";


    /**
     * The Reload Prefs Filename
     */
    public static final String USER_PREFS_FILENAME = "ld_prefs.xml";

    /**
     * The Reload Prefs root element name
     */
    public static final String USER_PREFS_ELEMENTNAME = "reload_prefs";

    /**
     * Our singleton User Prefs
     */
    private static LDEditorPrefs _userPrefs = new LDEditorPrefs();

    /**
     * @return the Singleton
     */
    public static LDEditorPrefs getInstance() {
        return _userPrefs;
    }

    /**
     * Set some default values
     */
    protected void setDefaultValues() {

    }

    /**
     * @return the Preferences File
     */
    public File getPrefsFile() {
        File reloadHome = new File(System.getProperty("user.home"), "reload");
        reloadHome.mkdirs();
        return new File(reloadHome, USER_PREFS_FILENAME);
    }

    /**
     * @return the root Element name
     */
    public String getElementRootName() {
        return USER_PREFS_ELEMENTNAME;
    }

    /**
     * @return the Recent File History as an array of Strings
     */
    public String[] getFileHistory() {
        Vector v = new Vector();

        String val = getValue(GENERAL_HISTORY);

        if(val != null) {
            StringTokenizer t = new StringTokenizer(val, ";");
            while(t.hasMoreElements()) {
                String path = t.nextToken();
                v.addElement(path);
            }
        }

        String[] vals = new String[v.size()];
        v.copyInto(vals);
        return vals;
    }

    /**
     * Add a new entry to the Recent File list, or if already present move to
     * front of list, and limit to 8 entries if need be
     */
    public void addFileToHistory(File file) {
        String val = file.getPath();
        int count = 1;

        String[] history = getFileHistory();

        for(int i = 0; i < history.length && count < 8; i++) {
            if(!history[i].equalsIgnoreCase(file.getPath())) {
                val += ";" + history[i];
                count++;
            }
        }

        putValue(GENERAL_HISTORY, val);
    }
}