/**
 *  Reload Moon Unit Library - Handles JDOM, Castor, Schema and IMS Document functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */


package uk.ac.reload.moonunit;

import java.io.*;
import java.util.*;

import org.jdom.*;
import uk.ac.reload.dweezil.util.*;

/**
 * Static fields for Namespaces, IMS versions and routines for handling these
 *
 * @author Phillip Beauvoir
 * @version 2003-10-15
 */
public class IMSDocumentHandler implements MoonUnitConstants {
    // Schema Versions

    // MD Schema Versions
    public static String IMSMD_110 = "imsmd_110";
    public static String IMSMD_120 = "imsmd_120";
    public static String IMSMD_121 = "imsmd_121";
    public static String IMSMD_122 = "imsmd_122";

	// Tina Manoharan 2003-08-11
    // VX Schema Profile Type Versions (bogus)
	public static String IMSVX_000 = "imsvx_000";
	public static String IMSVX_LAX_000 = "imsvx_lax_000";
	public static String IMSVX_FLAT_000 = "imsvx_flat_000";
	public static String IMSVX_HIERARCHICAL_000 = "imsvx_hierarchical_000";
	public static String IMSVX_GLOSSARY_000 = "imsvx_glossary_000";
	public static String IMSVX_THESAURUS_000 = "imsvx_thesaurus_000";

	// VX Schema Profile Type Versions (correct)
	public static String IMSVX_100 = "imsvx_100";
	public static String IMSVX_LAX_100 = "imsvx_lax_100";
	public static String IMSVX_FLAT_100 = "imsvx_flat_100";
	public static String IMSVX_HIERARCHICAL_100 = "imsvx_hierarchical_100";
	public static String IMSVX_GLOSSARY_100 = "imsvx_glossary_100";
	public static String IMSVX_THESAURUS_100 = "imsvx_thesaurus_100";

    // CP Schema Versions
    public static String IMSCP_111 = "imscp_111";
    public static String IMSCP_112 = "imscp_112";
    public static String IMSCP_113 = "imscp_113";

    // ADL SCORM Schema Versions
    public static String ADLCP_120_112 = "adlcp_120_112";
    public static String ADLCP_120_113 = "adlcp_120_113";
    public static String ADLCP_130 = "adlcp_130";

    // SS Schema Versions
    public static String IMSSS_100 = "imsss_100";

    // LD Schema Versions
    public static String IMSLD_100 = "imsld_100";
    public static String IMSLD_100A = "imsld_100A";
    public static String IMSLD_100B = "imsld_100B";
    public static String IMSLD_100C = "imsld_100C";


    // =========================================================================
    // ================================ METADATA ===============================
    // =========================================================================

    // Namespace prefix
    public static String IMSMD_NAMESPACE_PREFIX = "imsmd";

    // MD Version 1.2.2
    public static String IMSMD_NAMESPACE_URI_122 = "http://www.imsglobal.org/xsd/imsmd_v1p2";          // Correct
    // MD Version 1.2.1
    public static String IMSMD_NAMESPACE_URI_121 = "http://www.imsglobal.org/xsd/imsmd_rootv1p2p1";    // Correct
    public static String IMSMD_NAMESPACE_URI_121A = "http://www.imsproject.org/xsd/imsmd_rootv1p2p1";    // Bogus
    // MD Version 1.2 Variations
    public static String IMSMD_NAMESPACE_URI_12A = "http://www.imsproject.org/xsd/imsmd_rootv1p2";     // Bogus
    public static String IMSMD_NAMESPACE_URI_12B = "http://www.imsglobal.org/xsd/imsmd_rootv1p2";      // Bogus
    public static String IMSMD_NAMESPACE_URI_12C = "http://www.imsproject.org/xsd/ims_md_rootv1p2";    // Bogus
    // MD Version 1.1 Variations
    public static String IMSMD_NAMESPACE_URI_11 = "http://www.imsproject.org/xsd/ims_md_rootv1p1";
    public static String IMSMD_NAMESPACE_URI_112 = "http://www.imsproject.org/xsd/imsmd_rootv1p1p2";   // Bogus

    // Enumeration of Supported MD Namespace URIs
    public static String[] IMSMD_NAMESPACE_URIS = {
        IMSMD_NAMESPACE_URI_122,
        IMSMD_NAMESPACE_URI_121,
    };


    // =========================================================================
    // ================================ VOCABULARY =============================
    // =========================================================================

    // Namespace prefix
    public static String IMSVX_NAMESPACE_PREFIX = "imsvdex";

    // VX Version 0.0.0
    public static String IMSVX_NAMESPACE_URI_000 = "http://www.imsglobal.org/xsd/imsvdex_v0p0"; // Bogus
    // VX Version 1.0.0
    public static String IMSVX_NAMESPACE_URI_100 = "http://www.imsglobal.org/xsd/imsvdex_v1p0"; // Correct

    // Enumeration of Supported VX Namespace URIs
    public static String[] IMSVX_NAMESPACE_URIS = {
        IMSVX_NAMESPACE_URI_100
    };


    // =========================================================================
    // ================================ IMS CP =================================
    // =========================================================================

    // Namespace prefix
    public static String IMSCP_NAMESPACE_PREFIX = "imscp";

    // CP Version 1.1.3
    public static String IMSCP_NAMESPACE_URI_113 = "http://www.imsglobal.org/xsd/imscp_v1p1";           // Correct
    public static String IMSCP_NAMESPACE_URI_113A = "http://www.imsproject.org/xsd/imscp_rootv1p1p3";   // Bogus
    // CP Version 1.1.2
    public static String IMSCP_NAMESPACE_URI_112 = "http://www.imsproject.org/xsd/imscp_rootv1p1p2";    // Correct
    public static String IMSCP_NAMESPACE_URI_112A = "http://www.imsglobal.org/xsd/imscp_rootv1p1p2";    // Bogus
    // CP Version 1.1.x Variations
    public static String IMSCP_NAMESPACE_URI_111A = "http://www.imsglobal.org/xsd/imscproot_v1p1";      // Bogus
    public static String IMSCP_NAMESPACE_URI_111B = "http://www.imsglobal.org/xsd/imscp_rootv1p1";      // Bogus
    public static String IMSCP_NAMESPACE_URI_111C = "http://www.imsproject.org/xsd/ims_cp_rootv1p1";    // Bogus

    // Enumeration of Supported CP Namespace URIs
    public static String[] IMSCP_NAMESPACE_URIS = {
        IMSCP_NAMESPACE_URI_113,
        IMSCP_NAMESPACE_URI_112,
    };

    /**
     * The Manifest name
     */
    public static final String MANIFEST_NAME = "imsmanifest.xml";

    // =========================================================================
    // ================================ SCORM =================================
    // =========================================================================

    // Namespace prefix
    public static String ADLCP_NAMESPACE_PREFIX = "adlcp";

    // ADL SCORM Version 1.2
    public static String ADLCP_NAMESPACE_URI_12 = "http://www.adlnet.org/xsd/adlcp_rootv1p2";
    public static Namespace ADLCP_NAMESPACE_12 = Namespace.getNamespace("adlcp", ADLCP_NAMESPACE_URI_12);

    // ADL SCORM Version 1.3
    public static String ADLCP_NAMESPACE_URI_13 = "http://www.adlnet.org/xsd/adlcp_rootv1p3";
    public static Namespace ADLCP_NAMESPACE_13 = Namespace.getNamespace("adlcp", ADLCP_NAMESPACE_URI_13);

    // Enumeration of ADL SCORM Namespace URIs
    public static String[] ADLCP_NAMESPACE_URIS = {
        ADLCP_NAMESPACE_URI_12,
        ADLCP_NAMESPACE_URI_13
    };

    // =========================================================================
    // ==================== SIMPLE SEQUENCING ==================================
    // =========================================================================

    // Namespace prefix
    public static String IMSSS_NAMESPACE_PREFIX = "imsss";

    // SS Version 1.0
    public static String IMSSS_NAMESPACE_URI_10 = "http://www.imsglobal.org/xsd/imsss";

    // Enumeration of SS Namespace URIs
    public static String[] IMSSS_NAMESPACE_URIS = {
        IMSSS_NAMESPACE_URI_10
    };

    // =========================================================================
    // ====================== LEARNING DESIGN ==================================
    // =========================================================================

    // Namespace prefix
    public static String IMSLD_NAMESPACE_PREFIX = "imsld";

    // LD Version 1.0
    public static String IMSLD_NAMESPACE_URI_10 = "http://www.imsglobal.org/xsd/imsld_v1p0";

    // Enumeration of LD Namespace URIs
    public static String[] IMSLD_NAMESPACE_URIS = {
        IMSLD_NAMESPACE_URI_10
    };

    // =========================================================================
    // ========================== NAMESPACING ==================================
    // =========================================================================

    /**
     * The XSI Namespace
     */
    public static Namespace XSI_Namespace = Namespace.getNamespace("xsi", "http://www.w3.org/2001/XMLSchema-instance");

    /**
     * The Old XSI Namespace
     */
    public static Namespace XSI_NamespaceOLD = Namespace.getNamespace("xsi", "http://www.w3.org/2000/10/XMLSchema-instance");

    /**
     * The schemaLocation String
     */
    public static String XSI_SchemaLocation = "schemaLocation";

    // Schema Namespace URIs mapped to Version numbers
    static String[] _namespaceURIs = new String[] {
        // MD
        IMSMD_NAMESPACE_URI_122, IMSMD_122,
        IMSMD_NAMESPACE_URI_121, IMSMD_121,
        IMSMD_NAMESPACE_URI_12A, IMSMD_120,
        IMSMD_NAMESPACE_URI_12B, IMSMD_120,
        IMSMD_NAMESPACE_URI_12C, IMSMD_120,
        // VX
        IMSVX_NAMESPACE_URI_100, IMSVX_100,
        // CP
        IMSCP_NAMESPACE_URI_113, IMSCP_113,
        IMSCP_NAMESPACE_URI_113A, IMSCP_113,
        IMSCP_NAMESPACE_URI_112, IMSCP_112,
        IMSCP_NAMESPACE_URI_112A, IMSCP_112,
        IMSCP_NAMESPACE_URI_111A, IMSCP_111,
        IMSCP_NAMESPACE_URI_111B, IMSCP_111,
        IMSCP_NAMESPACE_URI_111C, IMSCP_111,
        // SS
        IMSSS_NAMESPACE_URI_10, IMSSS_100,
        // LD
        IMSLD_NAMESPACE_URI_10, IMSLD_100
    };

    // Enumeration of Bogus Namespace URIs mapped to supported ones
    public static String[] BOGUS_NAMESPACE_MAPPING = {
        IMSCP_NAMESPACE_URI_113A, IMSCP_NAMESPACE_URI_113,
        IMSCP_NAMESPACE_URI_112A, IMSCP_NAMESPACE_URI_112,
        IMSCP_NAMESPACE_URI_111A, IMSCP_NAMESPACE_URI_113,
        IMSCP_NAMESPACE_URI_111B, IMSCP_NAMESPACE_URI_113,
        IMSCP_NAMESPACE_URI_111C, IMSCP_NAMESPACE_URI_113,
        IMSMD_NAMESPACE_URI_12A, IMSMD_NAMESPACE_URI_122,
        IMSMD_NAMESPACE_URI_12B, IMSMD_NAMESPACE_URI_122,
        IMSMD_NAMESPACE_URI_12C, IMSMD_NAMESPACE_URI_122,
        IMSMD_NAMESPACE_URI_11, IMSMD_NAMESPACE_URI_122,
        IMSMD_NAMESPACE_URI_121A, IMSMD_NAMESPACE_URI_121,
        IMSMD_NAMESPACE_URI_112, IMSMD_NAMESPACE_URI_121,
        IMSVX_NAMESPACE_URI_000, IMSVX_NAMESPACE_URI_100
    };


    // =========================================================================
    // ======================= SCHEMA VERSION HANDLING =========================
    // =========================================================================

    /**
     * Get a (root) Schema version depending on an Instance ReloadDocument
     * @param doc the ReloadDocument instance
     * @return a (root) Schema version depending on an Instance ReloadDocument
     */
    public static String getSchemaVersion(Document doc) {
        // Get the correct Namespace
        Namespace ns = getCorrectRootNamespace(doc);

        // Get root schema version
        String schemaVersion = getSchemaVersionFromNamespace(ns);

        // If it's a CP, check for ADL SCORM
        if(schemaVersion != null && schemaVersion.startsWith("imscp")) {
            Namespace nsSCORM = getDocumentNamespace(doc, ADLCP_NAMESPACE_PREFIX);
            if(nsSCORM != null) {
                // If SCORM 1.2
                if(nsSCORM.getURI().equals(ADLCP_NAMESPACE_URI_12)) {
                    if(schemaVersion.equals(IMSCP_112)) schemaVersion = ADLCP_120_112;
                    else if(schemaVersion.equals(IMSCP_113)) schemaVersion = ADLCP_120_113;
                }
                else schemaVersion = null;
            }
        }

		// Tina Manoharan, 2003-08-13
        // if it's a Vdex vocabulary
        if(schemaVersion != null && schemaVersion.startsWith("imsvx")) {

			schemaVersion = getSchemaProfileVersion(doc);
		}

        return schemaVersion;
    }

    /**
     * Get a version of the Schema depending on Namespace URI
     * @param ns the Namespace URI
     * @return a version of the Schema depending on Namespace URI
     */
    public static String getSchemaVersionFromNamespace(Namespace ns) {
        if(ns != null) {
            String nameSpaceURI = ns.getURI();
            for(int i = 0; i < _namespaceURIs.length; i += 2) {
                if(nameSpaceURI.equalsIgnoreCase(_namespaceURIs[i])) return _namespaceURIs[i + 1];
            }
        }
        return null;
    }


	// Tina Manoharan, 2003-08-13

    /**
     * Get a (root) Schema Profile Version
     * @param doc the ReloadDocument instance
     * @return a (root) Schema Profile Version
     */
    public static String getSchemaProfileVersion(Document doc) {

		String schemaProfileVersion = "", profileType = "";

		Element root = doc.getRootElement();

		profileType = root.getAttributeValue("profileType");

		if(profileType.equals("lax")) schemaProfileVersion = IMSVX_LAX_100;
		else if(profileType.equals("flatTokenTerms")) schemaProfileVersion = IMSVX_FLAT_100;
		else if(profileType.equals("hierarchicalTokenTerms")) schemaProfileVersion = IMSVX_HIERARCHICAL_100;
		else if(profileType.equals("glossaryOrDictionary")) schemaProfileVersion = IMSVX_GLOSSARY_100;
		else if(profileType.equals("thesaurus")) schemaProfileVersion = IMSVX_THESAURUS_100;

        return schemaProfileVersion;
    }



    /**
     * Check for bogus root Namespace and correct it
     * @param doc the ReloadDocument instance
     * @return the correct Namespace
     */
    public static Namespace getCorrectRootNamespace(Document doc) {
        // Get the Root Namespace for the Instance Document
        Namespace ns = getDocumentNamespace(doc);
        Element element = doc.getRootElement();
        if(element != null && ns != null) {
            String nameSpaceURI = ns.getURI();
            for(int i = 0; i < BOGUS_NAMESPACE_MAPPING.length; i += 2) {
                if(nameSpaceURI.equalsIgnoreCase(BOGUS_NAMESPACE_MAPPING[i])) {
                    Namespace newNamespace = Namespace.getNamespace(BOGUS_NAMESPACE_MAPPING[i + 1]);
                    replaceNamespaces(element, ns, newNamespace);
                    if(DEBUG) System.out.println("Corrected namespace: " + ns.getURI() + " ----> " + newNamespace.getURI());
                    return newNamespace;
                }
            }
        }
        return ns;
    }

    /**
     * Check for bogus additional Namespaces and correct them
     * @param doc the ReloadDocument instance
     */
    public static void correctAdditionalNamespaces(Document doc) {
        Element element = doc.getRootElement();
        // Use a copy of the list so we don't get a concurrent modification exception
        Vector v = new Vector();
        Iterator it = element.getAdditionalNamespaces().listIterator();
        while(it.hasNext()) {
            Namespace ns = (Namespace) it.next();
            v.add(ns);
        }
        it = v.iterator();
        while(it.hasNext()) {
            Namespace ns = (Namespace) it.next();
            correctAdditionalNamespace(doc, ns);
        }
    }

    /**
     * Check for bogus additional Namespace and correct it
     * @param doc the ReloadDocument instance
     *		  ns the document Namespace
     * @return the correct Namespace
     */
    public static Namespace correctAdditionalNamespace(Document doc, Namespace ns) {
        if(doc != null && ns != null) {
            Element element = doc.getRootElement();
            String nameSpaceURI = ns.getURI();
            for(int i = 0; i < BOGUS_NAMESPACE_MAPPING.length; i += 2) {
                if(nameSpaceURI.equalsIgnoreCase(BOGUS_NAMESPACE_MAPPING[i])) {
                    Namespace newNamespace = Namespace.getNamespace(ns.getPrefix(), BOGUS_NAMESPACE_MAPPING[i + 1]);
                    element.removeNamespaceDeclaration(ns);
                    element.addNamespaceDeclaration(newNamespace);
                    replaceNamespaces(element, ns, newNamespace);
                    if(DEBUG) System.out.println("Corrected namespace: " + ns.getURI() + " ----> " + newNamespace.getURI());
                    return newNamespace;
                }
            }
        }
        return ns;
    }

    /**
     * Set child namespaces to MD Namespace.  Will not affect other Namespaces
     * @param element	the element whose Namespace has to be changed
     *		  oldNamespace	the old Namespace of the element
     *		  newNamespace	the new Namespace of the element
     */
    public static void replaceNamespaces(Element element, Namespace oldNamespace, Namespace newNamespace) {
        if(element.getNamespace().getURI().equalsIgnoreCase(oldNamespace.getURI()))
            element.setNamespace(newNamespace);

        Iterator children = element.getChildren().iterator();
        while(children.hasNext()) {
            Element child = (Element)children.next();
            replaceNamespaces(child, oldNamespace, newNamespace);
        }
    }

    /**
     * Replace the old "record" element with "lom" on MD
     * @param element	the concerned element
     */
    public static void correctLomElement(Element element) {
        if(element != null) {
            if(element.getName().equals("record")) {
                element.setName("lom");
                if(DEBUG) System.out.println("Corrected \"record\" ---> \"lom\" element");
            }
            Iterator children = element.getChildren().iterator();
            while(children.hasNext()) {
                Element child = (Element)children.next();
                correctLomElement(child);
            }
        }
    }

    /**
     * Get the schema location for a given Namespace in a JDOM Document or null if not found
     * @param doc the JDOM document
     *		  ns the Namespace in the document
     * @return the schema location for a given Namespace in a JDOM Document or null if not found
     */
    public static String getSchemaLocation(Document doc, Namespace ns) {
        if(doc.hasRootElement() && ns != null) {
            String namespaceURI = ns.getURI();
            if(namespaceURI != null) {
                Element root = doc.getRootElement();

                // Get Schema Location
                String str = root.getAttributeValue(XSI_SchemaLocation, XSI_Namespace);
                // Try an older one
                if(str == null)
                    str = root.getAttributeValue(XSI_SchemaLocation, XSI_NamespaceOLD);

                if(str != null) {
                    int index = str.indexOf(namespaceURI);
                    if(index != -1) {
                        str = str.substring(index + namespaceURI.length());
                        StringTokenizer t = new StringTokenizer(str, " ");
                        try {
                            return t.nextToken();
                        }
                        catch(NoSuchElementException ex) {
                            return null;
                        }
                    }
                }
            }
        }

        return null;
    }

    /**
     * Get the root Namespace of the Document or null if not found
     * @param doc the JDOM document
     * @return the root Namespace of the Document or null if not found
     */
    public static Namespace getDocumentNamespace(Document doc) {
        Namespace ns = null;
        if(doc.hasRootElement()) ns = doc.getRootElement().getNamespace();
        return ns;
    }

    /**
     * Get a given Namespace of the Document given its prefix or null if not found
     * @param doc the JDOM document
     *		  prefix the Namespace prefix
     * @return a given Namespace of the Document given its prefix or null if not found
     */
    public static Namespace getDocumentNamespace(Document doc, String prefix) {
        Namespace ns = null;
        if(doc.hasRootElement()) ns = doc.getRootElement().getNamespace(prefix);
        return ns;
    }

    /**
     * @return true if nameSpaceURI is in our list of accepted Metadata Namespace URIs
     */
    public static boolean isAcceptedMetadataNamespaceURI(String nameSpaceURI) {
        for(int i = 0; i < IMSDocumentHandler.IMSMD_NAMESPACE_URIS.length; i++) {
            if(nameSpaceURI.equals(IMSDocumentHandler.IMSMD_NAMESPACE_URIS[i])) {
                return true;
            }
        }
        return false;
    }

    /**
     * Looks for imsmanifest.xml in a zip archive
     * @param zipFile the zip archive file
     * @return either true or false
     * @throws DweezilZipException
     */
    public static boolean containsManifest(File zipFile) throws DweezilZipException {
        String manifestFile = ZipUtils.extractZipEntry(zipFile, MANIFEST_NAME);
        return manifestFile != null;
    }

}