/**
 *  Reload Moon Unit Library - Handles JDOM, Castor, Schema and IMS Document functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.moonunit.dom;

import org.jdom.*;


/**
 * A ReloadDocument has changed , this is the Event that is fired Listeners.
 *
 * @author Phillip Beauvoir
 * @version 2003-04-30
 */
public class ReloadDocumentListenerEvent {

	/**
	 * Who started this event in the first place
	 */
    protected Object _source;

    /**
     * The document for the element
     */
    protected ReloadSchemaDocument _reloadDocument;

    /**
     * The element concerned
     */
    protected Element _element;

    /**
     * A hint to the listener that they should select the Element that changed
     */
    protected boolean _doSelect;

    /**
     * Constructor
     * @param source Who started this event in this first place
     * @param reloadDocument The Document for the Element
     * @param element The Element concerned
     * @param doSelect A hint to the listener that they should select the Element that changed
     */
    public ReloadDocumentListenerEvent(Object source, ReloadSchemaDocument reloadDocument, Element element, boolean doSelect) {
        _source = source;
        _reloadDocument = reloadDocument;
        _element = element;
        _doSelect = doSelect;
    }

	/**
	 * Get the source who started this event
	 * @return the source object
	 */
    public Object getSource() {
        return _source;
    }

	/**
	 * Get the document for the element
	 * @return the ReloadSchemaDocument concerned
	 */
    public ReloadSchemaDocument getReloadDocument() {
        return _reloadDocument;
    }

	/**
	 * Get the element concerned
	 * @return the element concerned
	 */
    public Element getElement() {
        return _element;
    }

	/**
	 * Get the select option for the element that changed
	 * @return true for selecting the element, false otherwise
	 */
    public boolean doSelect() {
        return _doSelect;
    }
}