/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.moonunit.schema;

import org.jdom.*;


/**
 * A ReloadVocabularyList has changed , this is the Event that is fired Listeners.
 *
 * @author Tina Manoharan
 * @version 2003-10-29
 */
public class ReloadVocabularyListenerEvent {

	/**
	 * Who started this event in the first place
	 */
    protected Object _source;

    /**
     * The Vocabulary List
     */
    protected ReloadVocabularyList _rvList;

    /**
     * A String array of list items
     */
    protected String[] _list;


    /**
     * Constructor
     * @param source Who started this event in the first place
     * @param rvList the Vocabulary list
     * @param list a String array of list items
     */
    public ReloadVocabularyListenerEvent(Object source, ReloadVocabularyList rvList, String[] list) {
        _source = source;
        _rvList = rvList;
        _list = list;
    }

	/**
	 * Get the source object that started this event
	 * @return the source object that started this event
	 */
    public Object getSource() {
        return _source;
    }

	/**
	 * Get the Vocabulary list
	 * @return the vocabulary list
	 */
    public ReloadVocabularyList getReloadVocabularyList() {
        return _rvList;
    }

	/**
	 * Get the array of list items
	 * @return the array of list items
	 */
    public String[] getList() {
        return _list;
    }
}