/**
  *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.scormplayer.client.contentpackaging;

import java.io.*;
import java.util.*;
import org.jdom.*;

import uk.ac.reload.moonunit.*;
import uk.ac.reload.moonunit.dom.*;

import uk.ac.reload.dweezil.util.*;

import uk.ac.reload.scormplayer.client.gui.*;

 /**
  * A class to wrap and use the IMSDocumentHandler lib, to find out if
  * a manifest is a (correctly namespaced) valid scorm 1.2 package
  * @author Paul Sharples
  * @version 2003-09-29
  */
public class ReloadManifestHandler extends ReloadDocument{

    public static final String ARCHIVE_EXTENSION = ".zip";
    public static final String XML_EXTENSION = ".xml";

    /*
    * our document handler to use
    */
    protected IMSDocumentHandler _docHandler;

    /**
     * Constructor - with a file parameter, used to load the manifest
     * into JDOM via ReloadDocument.
     * @param manifest
     */
    public ReloadManifestHandler(File manifest) {
        try {
           // Load the Document
            super.loadDocument(manifest);
            _docHandler = new IMSDocumentHandler();
        }
        catch (DweezilXMLException ex) {
            System.out.println("Error: could not parse manifest: " + ex);
        }
    }

    /**
     * startParse() - parse the manifest and try to figure out what it is
     * Uses the moonunit library IMSDocumentHandler to do this. If its a
     * valid scorm 1.2 package, then we can instantiate a new ScormPackageHandler
     * and start to build our sco xml files and navigation for the javascript stuff
     * on the server side.
     */
    public void startParse() {
        try {
            ScormPackageHandler scormPackage = new ScormPackageHandler(_file,
                _doc);
            scormPackage.buildSettings();
        }
        catch (Exception ex) {
            System.out.println("An error occured while parsing the manifest: " +
                               ex);

        }
    }

    public boolean testManifest() {
        String version = _docHandler.getSchemaVersion(_doc);

        if (version == _docHandler.ADLCP_120_112 ||
            version == _docHandler.ADLCP_120_113)
            return true;
        else
            return false;
    }
}