/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.contentpackaging;

import java.io.*;
import java.util.*;
import org.jdom.*;

import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.dweezil.util.*;

import uk.ac.reload.scormplayer.client.contentpackaging.*;

/**
 * A class used to interrogate the navigation file created when a scorm package
 * has been imported.  Used to show the tree in the organizations panel and
 * to figure out which items are scos/assets
 * @author Paul Sharples
 * @version 2003-09-29
 */
public class ReloadNavigationViewer extends ReloadDocument{

    public Vector _organizationList;

    /**
     * Constructor with file param, to load nav file as JDOM doc.
     * @param file
     */
    public ReloadNavigationViewer(File file) {
        try {
            super.loadDocument(file);
        }
        catch (DweezilXMLException ex) {
            System.out.println("Error loading navigation setting for package" + ex);
        }
    }

    /**
     * A method to get the default organization from the navigation file
     * @return
     */
    public String getDefaultOrg(){
        String def = _doc.getRootElement().getAttribute(ScormPackageHandler.NAVIGATION_ROOT_DEFAULT).getValue();
        if (def != "" && def != null){
            return def;
        }
        else{
            return "";
        }
    };

    /**
     * Method to ascertain which organization is the default, by index
     * @param index
     * @return
     */
    public boolean getDefaultOrgByIndex(int index){
        String theDefaultOrg = getDefaultOrg();
        Element[] orgs = getOrganizationList();
            if (orgs[index].getAttributeValue(ScormPackageHandler.NAVIGATION_ORG_ID).equals(theDefaultOrg)){
              return true;
            }
        return false;
    }

    /**
     * Method to ascertain if an item is a sco, asset or does not
     * have a referenced resource
     * @param element
     * @return
     */
    public static String findScoType(Element element){
        if (element.getName() == ScormPackageHandler.NAVIGATION_ITEM){
            String scoType = element.getChild(ScormPackageHandler.NAVIGATION_TYPE).getText();
            if (scoType == null || scoType == ""){
                return "";
            }
            else {
                return scoType;
            }
        }
        return null;
    }

    /**
     * A method to get an array of all the organization elements in the
     * navigation file.
     * @return an array of <org> elements
     */
    public Element[] getOrganizationList() {
         _organizationList = new Vector();
        parseForOrgs(_doc.getRootElement());
        Element[] orgs = new Element[_organizationList.size()];
        _organizationList.copyInto(orgs);
        return orgs;
    }


    /**
     * Method to grab all organizations and copy into vector. called
     * by getOrganizationList above.
     * @param element
     */
    private void parseForOrgs(Element element) {
        String name = element.getName();
        if (name.equals(ScormPackageHandler.NAVIGATION_ORG)) {
           _organizationList.add(element);
        }
        Iterator it = element.getChildren().iterator();
        while (it.hasNext()) {
             Element child = (Element) it.next();
              parseForOrgs(child);
        }
    }


    /**
     * Method to change the default organization in the navigation file
     * @param index
     */
    public void updateDefaultOrg(int index){
        Element[] orgs = getOrganizationList();
        String newDefault = orgs[index].getAttributeValue(ScormPackageHandler.NAVIGATION_ORG_ID);
        _doc.getRootElement().setAttribute(ScormPackageHandler.NAVIGATION_ROOT_DEFAULT, newDefault);
        try {
            saveDocument();
        }
        catch (Exception ex) {
            System.out.println("Error saving changes to default organization: " + ex);
        }
    }
}