/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import uk.ac.reload.scormplayer.client.gui.menu.*;
import uk.ac.reload.scormplayer.client.gui.mainwindow.*;
import uk.ac.reload.scormplayer.client.gui.prefs.*;

import uk.ac.reload.dweezil.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.util.*;

/**
 * The ReloadScormPlayer Frame - the main graphical application. Builds the GUI
 *
 * @author Phillip Beauvoir
 * @author Paul Sharples
 * @version 2002-03-10
 */
public class ReloadScormPlayer
extends JFrame
{
    /**
    * Set this to true for a Release Build, keep false for debugging
    */
   public static boolean ISBUILD = true;

   //public static boolean STATUS_WINDOW = true;  // Send Output to Status Window
   public static boolean STATUS_WINDOW = ISBUILD; // Send Output to Status Window
   public static boolean DEBUG = !ISBUILD; // Debug Trace

   // Information Constants
   public static String APP_NAME = "Reload Scorm 1.2 Runtime Environment";
   public static String VERSION = "1.0";
   public static String BUILD_DATE = "5th November 2003";
   public static String COPYRIGHT = "Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir";
   public static String AUTHORS = "Paul Sharples & Phillip Beauvoir";
   public static String CONTRIBUTORS =
       "Warwick Bailey, Pierre Gorissen, " +
       "Colin Milligan, Mark Power, Ben Ryan, Scott Wilson, " +
       "and all the Glasgow code-bashers";

   public static String EMAIL_CONTACT = "p.sharples@bolton.ac.uk";
   public static String WEB_PAGE = "http://www.reload.ac.uk";
   public static String CR = System.getProperty("line.separator");

   // ========================================================================
   // ============================== ICONS ===================================
   // ========================================================================
   public static String ICON_PATH = "/resources/images/";
   public static String ICON_APP16 = ICON_PATH + "app16.gif";
   public static String ICON_APP32 = ICON_PATH + "app32.gif";
   public static String ICON_CP_SCORM_CLOSED = ICON_PATH + "cp_scorm.gif";
   public static String ICON_CP_SCORM_OPENED = ICON_PATH + "cp_scorm_open.gif";
   public static String ICON_DELETE = ICON_PATH + "delete.gif";
   public static String ICON_FOLDER_CLOSED = ICON_PATH + "folder_closed.gif";
   public static String ICON_FOLDER_OPEN = ICON_PATH + "folder_open.gif";
   public static String ICON_OPEN = ICON_PATH + "open.gif";
   public static String ICON_REFRESH = ICON_PATH + "refresh.gif";
   public static String ICON_RESET = ICON_PATH + "reset.gif";
   public static String ICON_RESOURCES = ICON_PATH + "resources.gif";
   public static String ICON_TOMCAT_START = ICON_PATH + "tomcatup.gif";
   public static String ICON_TOMCAT_END = ICON_PATH + "tomcatdown.gif";
   public static String ICON_START_SCORM_PACKAGE = ICON_PATH + "start.gif";
   public static String ICON_SCORM_FOLDER_OPENED = ICON_PATH +
       "menu_folder_open.gif";
   public static String ICON_SCORM_FOLDER_CLOSED = ICON_PATH +
       "menu_folder_closed.gif";
   public static String ICON_SCORM_ITEM = ICON_PATH + "menu_link_default.gif";
   public static String ICON_CMI_ROOT_OPEN = ICON_PATH + "cmi_root_open.gif";
   public static String ICON_CMI_ROOT_CLOSED = ICON_PATH + "cmi_root_closed.gif";
   public static String ICON_CMI_ELEMENT_FOLDER_CLOSED = ICON_PATH +
       "cmi_element_folder_closed.gif";
   public static String ICON_CMI_ELEMENT_FOLDER_OPEN = ICON_PATH +
       "cmi_element_folder_open.gif";
   public static String ICON_CMI_ELEMENT_LEAF = ICON_PATH + "cmi_element_leaf.gif";
   public static String ICON_HIDDEN_ITEM_LEAF = ICON_PATH + "hiddenitemleaf.gif";
   public static String ICON_HIDDEN_ITEM_CLOSED = ICON_PATH + "hiddenitemclosed.gif";

   public static String IMAGE_SPLASH = ICON_PATH + "splash.gif";
   public static String IMAGE_EASTER_EGG = ICON_PATH + "easter_egg.jpg";

   /**
    * The Status Window
    */
    private ReloadStatusWindow _statusWindow;

    /**
     * The singleton instance of the app
     */
    private static ReloadScormPlayer _instance;

    /**
     * The desktop pane.
     */
    protected JDesktopPane _desktop;

    /**
     * The main internal window
     */
    protected static MainWindow _mainWindow;

    /**
     * Gets the Singleton Instance of the Editor
     * @return The Singleton Instance of the Editor
     */
    public static ReloadScormPlayer getInstance() {
        return _instance;
    }

    /**
     * @return The Status Window
     */
    public ReloadStatusWindow getStatusWindow() {
        return _statusWindow;
    }


    /**
     * Gets the Singleton Instance of the MainWindow
     * @return - singleton instance of the mian window
     */
    public static MainWindow getMainWindow(){
        return _mainWindow;
    }

    /**
     * Constructor method for new Reload SCORM Viewer
     */
    public ReloadScormPlayer() {
        _instance = this;

        // UI Defaults
        ReloadUIManager.setMacUIDefaults(APP_NAME);

        // SplashScreen
        SplashScreen splash = new SplashScreen(ReloadUIManager.getIcon(IMAGE_SPLASH));

        // Get the Status Window going
         _statusWindow = new ReloadStatusWindow(APP_NAME, VERSION, BUILD_DATE, ReloadUIManager.getIcon(ICON_APP32), !DEBUG);

       // Set Look And Feel from User Prefs
       String lf = ReloadScormPlayerPrefs.getInstance().getValue(ReloadScormPlayerPrefs.GENERAL_PREFS_LF);
       if(lf == null) lf = UIManager.getSystemLookAndFeelClassName();
       ReloadUIManager.setLookAndFeel(lf, new Component[] {getInstance(), getStatusWindow()});

       // Set default folder from User Prefs
       String defFolder = ReloadScormPlayerPrefs.getInstance().getValue(ReloadScormPlayerPrefs.GENERAL_PREFS_DEFAULT_FOLDER);
       if(defFolder != null) ReloadFileChooser.setDefaultFolder(new File(defFolder));

        // assign our main window var
        _mainWindow = new MainWindow();

       // Set application title in title bar
       super.setTitle(APP_NAME);

        // Set application Icon
        ImageIcon icon = ReloadUIManager.getIcon(ICON_APP32);
        if(icon != null) setIconImage(icon.getImage());

        // Trap window closing event for our Window listener
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

        // Add a listener to Close our window down on exit
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
            	closeApp();
            }
        });

        // Set up desktop
        JPanel panel = new JPanel(new BorderLayout());
        _desktop = new JDesktopPane();
        panel.add(_desktop, BorderLayout.CENTER);
        setContentPane(panel);

        // Create Menu Bar (after setting up the desktop, please!)
        setJMenuBar(MainMenu.sharedInstance);

        // Create Toolbar
        getContentPane().add(MainMenu.sharedInstance.getReloadToolBar(), BorderLayout.NORTH);
        // add the main window
        getContentPane().add(_mainWindow, BorderLayout.CENTER);
        splash.close();

        // Show the main Frame Window, centred
        ReloadUtils.centreWindowProportional(this, 0.9, 0.9);
        setVisible(true);
    }

    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    /**
     * Main program entry point
     * @param args Command line arguments
     */
    public static void main(String[] args) {
        ReloadSetupHandler handler = new ReloadSetupHandler();
        if (handler.checkUserFolderExists()){
          new ReloadScormPlayer();
        }
        else{
          System.exit(0);
        }
    }

    /**
     * Quit the application
     * Put any cleanup code here.
     */
    public void closeApp() {
    	// Tell any registered listeners
        boolean result = fireApplicationClosing();
        if(result) System.exit(0);
    }

    /**
     * Set the Title Bar String
     * @param title The Title Bar String
     */
    public void setTitle(String title) {
    	super.setTitle(ReloadScormPlayer.APP_NAME + " - " + title);
    }


    ////////////////////////////////////////////////////////////////////////////
    //   APPLICATION LISTENERS  ////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    /**
     * List of Listeners
     */
    private Vector listeners = new Vector();

    /**
     * Add an Application Listener
     * @param listener The Application Listener
     */
    public synchronized void addApplicationListener(ApplicationListener listener) {
        if(!listeners.contains(listener)) listeners.addElement(listener);
    }

    /**
     * Remove an Application Listener
     * @param listener The Application Listener
     */
    public synchronized void removeApplicationListener(ApplicationListener listener) {
        listeners.removeElement(listener);
    }

    /**
     * Tell our listeners that we are closing the App
     * They must reply by answering with true for OK, or false for not OK
     */
    public boolean fireApplicationClosing() {
        boolean ok = true;
        // Go from last to first in case anyone removes themselves
        for(int i = listeners.size() - 1; i >= 0; i--) {
            ApplicationListener listener = (ApplicationListener)listeners.elementAt(i);
            boolean result = listener.applicationClosing();
            if(result == false) ok = false;
        }
        return ok;
    }
}