/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui;

import java.io.*;
import javax.swing.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.scormplayer.*;
import uk.ac.reload.scormplayer.tomcat.TomcatConstants;

/**
 * A class used for windows versions of the player.  Ensures that a project
 * folder has been created for each user.  Will only run once on initial launch
 * of application.
 *
 * @author Paul Sharples
 * @version 2003-07-29
 */


public class ReloadSetupHandler {

  public ReloadSetupHandler() {}

  public boolean checkUserFolderExists(){
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName() );
    }
    catch (Exception ex) {}
    File file = TomcatConstants.FULL_APP_PATH;
    if (file.exists()){
        return true;
    }
    else{
      int answer = JOptionPane.showConfirmDialog(null,
          "You appear to have no project folder setup.\n" +
          "The player will not be able continue without one.\nWould you like to set this now?.", "Setup user project folder",
          JOptionPane.YES_NO_CANCEL_OPTION,
          JOptionPane.QUESTION_MESSAGE);
      if (answer == JOptionPane.YES_OPTION) {
        // copy it all across
        if (TomcatConstants.GENERIC_PROJECT_PATH.exists()){
          ReloadProgressMonitor progressMonitor = null;
          try {
            progressMonitor = new ReloadProgressMonitor(null,
                "Setting up files for initial use",
                "Copying required files...", "Searching...", true, null);

            TomcatConstants.USER_HOME.mkdirs();

            boolean result = ControlStation.copyFolder(TomcatConstants.
                GENERIC_PROJECT_PATH,
                new File(TomcatConstants.USER_HOME, "server"), progressMonitor);

            //Pressed Cancel
            if (!result) {
              // need to rollback here
              ControlStation.deleteFolder(TomcatConstants.USER_HOME);
              JOptionPane.showMessageDialog(null, "User pressed cancel, setup not complete. The player will now exit.");
              return false;
            }
          }
          catch (Exception ex) {
                JOptionPane.showMessageDialog(null, "Cannot find setup files. Please reinstall this software.");
                return false;
          }
          finally {
           if (progressMonitor != null)
               progressMonitor.close();
          }
          JOptionPane.showMessageDialog(null, "User setup complete.");
          return true;
        }
        else{
          JOptionPane.showMessageDialog(null, "Cannot find setup files. Please reinstall this software.");
          return false;
        }
      }
      else{
        JOptionPane.showMessageDialog(null,
           "The player cannot continue without a user project folder and will now exit.");
        return false;
      }
    }
  }
}