/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.mainwindow.itemwindow;

import java.io.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import org.jdom.*;


import uk.ac.reload.scormplayer.client.gui.mainwindow.*;
import uk.ac.reload.scormplayer.client.gui.mainwindow.navigationwindow.*;
import uk.ac.reload.scormplayer.client.gui.mainwindow.packagewindow.*;
import uk.ac.reload.scormplayer.client.contentpackaging.*;
import uk.ac.reload.dweezil.util.*;

import javax.swing.border.LineBorder;

public class ItemPanel extends JPanel
implements TreeSelectionListener {

    protected static CMIModelTreeTable _cmiTree;

    protected static DetailPanel _detailPanel;

    protected static JScrollPane _scrollPane;

    protected static JSplitPane jsp;

    protected static JLabel _scoNotSelected;

    protected static JPanel _scoEmpty;

    public ItemPanel() {
        setLayout(new BorderLayout());

        _scrollPane = new JScrollPane();
        _scrollPane.getViewport().setBackground(Color.WHITE);

        _scoNotSelected = new JLabel(" No sco currently selected");
        _scoNotSelected.setBackground(Color.WHITE);

        _scoEmpty = new JPanel();
        _scoEmpty.setLayout(new BorderLayout());
        _scoEmpty.add(_scoNotSelected, BorderLayout.NORTH);
        _scoEmpty.setBackground(Color.WHITE);

        _scrollPane.setViewportView(_scoEmpty);
        this.add(_scrollPane, BorderLayout.CENTER);

        _detailPanel = new DetailPanel();
        _detailPanel.setBorder(new LineBorder(Color.WHITE, 1));
        this.add(_detailPanel, BorderLayout.SOUTH);
    }

    public void valueChanged(TreeSelectionEvent event) {
        if (event.getSource() == MainWindow._navPanel.getOrgPanel().getNavTree()){
             TreePath selPath = event.getPath();
             NavigationTreeNode selectedNode = (NavigationTreeNode) selPath.getLastPathComponent();
             String scoType = selectedNode.whatIsThis();

             String[] tester = new String[5];
             // item title
             tester[0] = selectedNode.toString();
             // item id
             tester[1] = selectedNode.getIdentifier();
             // sco type
             tester[2] = scoType;
             // launch
             tester[3] = selectedNode.getLaunch();
             // prereq
             tester[4] = selectedNode.getPrerequistes();

             if (MainWindow._treePanel.getPackageTree().getSelectedNode().isRoot()){
                 _detailPanel.updateTable(getEmptyIteminfo());
             }
             else{
                 _detailPanel.updateTable(tester);
             }


             if (scoType == "sco"){
                 // need to get file ref to cmi xml file
                 ReloadItemViewer item = new ReloadItemViewer();
                try {
                    this.remove(_scrollPane);
                    item.loadDocument(selectedNode.getSCOFile());
                     CMIModelTreeTableModel model = new CMIModelTreeTableModel(item.getRootElement());
                    _cmiTree = new  CMIModelTreeTable(model);
                    _cmiTree.setBackground(Color.WHITE);
                    _scrollPane = new JScrollPane();
                    _scrollPane.setViewportView(_cmiTree);
                    _scrollPane.getViewport().setBackground(Color.WHITE);
                    this.add(_scrollPane, BorderLayout.CENTER);
                    this.updateUI();
                }
                catch (DweezilXMLException ex) {
                    System.out.println("Error reading SCO xml: " + ex);
                }
             }
             else {
                 this.remove(_scrollPane);
                 _scrollPane = new JScrollPane();
                 _scrollPane.getViewport().setBackground(Color.WHITE);
                 _scrollPane.setViewportView(_scoEmpty);
                 this.add(_scrollPane, BorderLayout.CENTER);
                 this.updateUI();
             }
         }
    }

    protected String[] getEmptyIteminfo(){
            String[] empty = {"","","","",""};
            return empty;
    }


}



