/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.mainwindow.navigationwindow;

import java.io.*;
import java.util.*;
import org.jdom.*;

import uk.ac.reload.dweezil.gui.tree.*;

//import uk.ac.reload.scormplayer.client.dom.*;
import uk.ac.reload.scormplayer.client.contentpackaging.*;

/**
 * Our tree model for the navigation tree (shows the organizations)
 * @author Paul Sharples
 * @version 2003-09-29
 */
public class NavigationTreeModel extends ReloadTreeModel {

    /**
     * A temporary hashtable used to work out the tree structure
     * for each organization. Its cleared once the model is loaded
     */
    protected static Hashtable items = new Hashtable();

    /**
     * default constructor
     */
    public NavigationTreeModel() {}

    /**
     * Give the tree an organization element and build the view
     * @param theOrg
     */
    public void setTree(Element theOrg) {
        if (theOrg != null){
            root = new NavigationTreeNode(theOrg);
            String orgID = theOrg.getAttributeValue(ScormPackageHandler.NAVIGATION_ORG_ID);
            items.put(orgID, root);
            buildView( (NavigationTreeNode) root);
        }
        else{
            this.setRoot(null);
        }
         reload();
         items.clear();
    }

    /**
     * Build up child nodes
     */
    public void buildView(NavigationTreeNode node) {
        if (node == null)
            return;
        Iterator children = node.getElement().getChildren().iterator();
        while (children.hasNext()) {
            Element child = (Element) children.next();
            String name = child.getName();
                if (name.equals(ScormPackageHandler.NAVIGATION_ITEM)) {
                    NavigationTreeNode newNode = new NavigationTreeNode(child);
                    String itemID = child.getAttributeValue(ScormPackageHandler.NAVIGATION_ITEM_ID);
                    items.put(itemID, newNode);
                    String parentID = newNode.getElement().getChild(ScormPackageHandler.NAVIGATION_LEVEL).getText();
                    NavigationTreeNode theParent = (NavigationTreeNode)items.get(parentID);
                    theParent.add(newNode);
                    buildView(newNode);
                }
        }
    }

}

