/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.mainwindow.navigationwindow;

import java.io.*;

import org.jdom.*;

import uk.ac.reload.dweezil.gui.tree.*;

import uk.ac.reload.scormplayer.client.gui.mainwindow.*;
import uk.ac.reload.scormplayer.client.gui.mainwindow.packagewindow.*;

import uk.ac.reload.scormplayer.client.contentpackaging.*;


/**
 * A class to hold a JDOM element as a Navigation tree node.
 * @author Paul Sharples
 * @version 2003-09-29
 */
public class NavigationTreeNode extends ReloadTreeNode{

    protected File _CMIModel = null;

    public NavigationTreeNode(Element name) {
        super(name);
        String scoType = whatIsThis();

        if (scoType == "sco" && name.getName().equals("item")) {
            // get the item ID
            String id = name.getAttributeValue(ScormPackageHandler.
                                               NAVIGATION_ITEM_ID);

            // get the path
            PackageTreeNode node = (PackageTreeNode) MainWindow._treePanel.
                getPackageTree().getSelectionPath().getLastPathComponent();
            // now build file reference
            File scoFile = new File(node.getFilePath().toString() +
                                    File.separatorChar +
                                    File.separatorChar + "reload-settings" +
                                    File.separatorChar + id + ".xml");
            if (scoFile.exists()) {
                _CMIModel = scoFile;
            }
        }
    }

    public String getLaunch(){
        Element aNode = (Element) getUserObject();
        String name = aNode.getName();
        if (name.equals(ScormPackageHandler.NAVIGATION_ITEM)){
            String launch = aNode.getChild(ScormPackageHandler.NAVIGATION_LAUNCH).getText();
            if (launch != null) {
                if (launch != ""){
                    return launch;
                }
                else return "none";
            }
            else return "none";
        }
        else return "none";
    }


    public String getPrerequistes(){
        Element aNode = (Element) getUserObject();
        String name = aNode.getName();
        if (name.equals(ScormPackageHandler.NAVIGATION_ITEM)){
            String prereq = aNode.getChild(ScormPackageHandler.NAVIGATION_PREREQUISITES).getText();
            if (prereq != null) {
                if (prereq != "") {
                    return prereq;
                }
                else return "none";
            }
            else return "none";
        }
        else return "none";
    }



    /**
     * Accessor method to return the Identifier for this item/org
     * @return
     */
    public String getIdentifier(){
        Element aNode = (Element) getUserObject();
        String name = aNode.getName();
       // If this element is an ITEM..
       if (name.equals(ScormPackageHandler.NAVIGATION_ITEM)){
           String id = aNode.getAttributeValue(ScormPackageHandler.NAVIGATION_ITEM_ID);
           if (id != null) return id;
           else return "";
       }
       // If this is the organization
       else if (name.equals(ScormPackageHandler.NAVIGATION_ORG)){
           return aNode.getAttributeValue(ScormPackageHandler.NAVIGATION_ORG_ID);
       }
       else return "";
    }


    /**
     * Accessor Method to return the file reference to the CMI xml file for this
     * sco.
     * @return - file reference
     */
    public File getSCOFile(){
        return _CMIModel;
    }


    /**
     * Accessor Method to return if an item is visible or not
     * @return
     */
    public boolean isVisible() {
        Element aNode = (Element) getUserObject();
        String name = aNode.getName();
        // If this element is an ITEM..
        if (name.equals(ScormPackageHandler.NAVIGATION_ITEM)) {
            String answer = aNode.getAttributeValue("isvisible");
            if (answer != null) {
                if (answer.equals("false")) {
                    return false;
                }
                else {
                    return true;
                }
            }
            else {
                return true;
            }
        }
        else {
            return true;
        }
    }

    /**
    * Get the JDOM Element
    * @return The JDOM Element
    */
   public Element getElement() {
       return (Element)getUserObject();
   }


   /**
    * Method to find out if this is a sco/asset/neither
    * @return - a string identifying what this is
    */
   public String whatIsThis(){
       Element aNode = (Element) getUserObject();
       String name = aNode.getName();
       // If this element is an ITEM..
       if (name.equals(ScormPackageHandler.NAVIGATION_ITEM)){
           String scoType = ReloadNavigationViewer.findScoType(aNode);
           if (scoType != "" && scoType != null){
               if (scoType.equals("sco")){
                   return "sco";
               }
               else if (scoType.equals("asset")){
                   return "asset";
               }
               else{
                   return "none";
               }
           }
       }
       return "none";
   }

   /**
    * Accessor method to get the title of the item/org
    * @return a string representing the title of this item/org
    */
   public String toString(){
       Element aNode = (Element) getUserObject();
       String name = aNode.getName();
       // If this element is an ITEM..
       if (name.equals(ScormPackageHandler.NAVIGATION_ITEM)){
           String scoType = ReloadNavigationViewer.findScoType(aNode);
           // is this a sco/asset/neither
           if (scoType != "" && scoType != null){
               return aNode.getChild(ScormPackageHandler.NAVIGATION_TITLE).
                   getText() + " (" + scoType + ")";
           }
           else{
               return aNode.getChild(ScormPackageHandler.NAVIGATION_TITLE).getText();
           }
        // else is this an Org
       }else if (name.equals(ScormPackageHandler.NAVIGATION_ORG)){
           return aNode.getAttributeValue(ScormPackageHandler.NAVIGATION_ORG_TITLE);
       // shouldn't get to here, but just in case...
       }else{
           return "unknown";
       }
   }

}

