/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.mainwindow.packagewindow;

import java.io.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import uk.ac.reload.dweezil.gui.tree.*;

import uk.ac.reload.scormplayer.client.gui.menu.*;

/**
 * A class derived from a JTree, used to show which packages have been imported
 * into the app (web document root)
 * @author Paul Sharples
 * @version 2003-09-29
 */
public class PackageTree extends ReloadTree implements TreeSelectionListener {

    /**
     * The Tree Model
     */
    public PackageTreeModel _treeModel;

    /**
     * The main menu
     */
    protected MainMenu _mainMenu;

    /**
     * Keeping note of the current selected tree node means we don't have to do
     * unneccesary updating
     */
    protected TreePath _currentTreePath;

    /**
     * Default Constructor
     */
    public PackageTree() {
        _mainMenu = MainMenu.sharedInstance;
        _treeModel = new PackageTreeModel();
        setTree();
        setModel(_treeModel);
        addTreeSelectionListener(this);
        setCellRenderer(new PackageTreeRenderer());
        getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
    }

    public void setTree() {
        _treeModel.setTree();
    }

    public File getSelectedPackage() {
        PackageTreeNode fileSysEntity =
            (PackageTreeNode) _currentTreePath.getLastPathComponent();
        // Need to make sure this does not pass the root node back!
        if (!fileSysEntity.isRoot()) {
            File aPackage = (File) fileSysEntity.getUserObject();
            return aPackage;
        }
        else {
            return null;
        }
    }



    public void valueChanged(TreeSelectionEvent event) {
        TreePath selPath = event.getPath();
        if (selPath != _currentTreePath) {
            updateMenus(selPath);
            _currentTreePath = selPath;
        }
    }

    /**
     * Update the menus according to the selected node on the tree.
     * This has to be synchronized so different threads don't access it at the same time
     */
    protected synchronized void updateMenus(TreePath selPath) {
        if (selPath == null)
            return;

        Object selectedNode = selPath.getLastPathComponent();
        if (selectedNode instanceof PackageTreeNode) {
            // Cast it
            PackageTreeNode treeNode = (PackageTreeNode) selectedNode;
            if (!treeNode.isRoot()) {
                _mainMenu._startPackage.setEnabled(true);
                _mainMenu._removePackage.setEnabled(true);
                _mainMenu._resetPackage.setEnabled(true);
            }
            else {
                _mainMenu._startPackage.setEnabled(false);
                _mainMenu._removePackage.setEnabled(false);
                _mainMenu._resetPackage.setEnabled(false);
            }

        }
    }



}