/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.menu;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.menu.*;
import uk.ac.reload.scormplayer.client.gui.*;


/**
 * The Main Menu for the Application.  It extends JMenuBar.  It also takes care
 * of the Toolbar and any main Popup menus.  It can listen to any refresh requests
 * so that for example, a change will set enabled and disabled menu items and buttons.
 * This class is tied in with the MainMenuAction and MenuAction classes.
 *
 * @author Phillip Beauvoir
 * @author Paul Sharples
 * @version 2003-06-02
 */
public class MainMenu
extends JMenuBar
{
    /**
     * The Singleton instance
     */
    public static MainMenu sharedInstance = new MainMenu();
    public MenuAction_StartServer _startServer = new MenuAction_StartServer();
    public MenuAction_StopServer _stopServer = new MenuAction_StopServer();
    public MenuAction_StartPackage _startPackage = new MenuAction_StartPackage();
    public MenuAction_Refresh _stopPackage = new MenuAction_Refresh();
    public MenuAction_ResetPackage _resetPackage = new MenuAction_ResetPackage();
    public MenuAction_RemovePackage _removePackage = new MenuAction_RemovePackage();

    /**
     * The main application toolbar
     */
    private ReloadToolBar toolBar;

    /**
     * The Key Mask will be 'Meta' for Mac and 'Ctrl' for PC/Unix
     */
    private int keyMask = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();

    /**
     * Constructor
     */
    private MainMenu() {
        // Attach Actions to Menu Items
        createMenuBar();

        // Create the ToolBar
        toolBar = createToolBar();
    }


    // =========================== MENU BAR ==================================
    /**
     * The File Menu
     */
    public JMenu fileMenu;

   /**
     * The Debug Menu
     */
    public JMenu runMenu;

    /**
     * The Debug Menu
     */
    public JMenu serverMenu;

    /**
     * The Options Menu
     */
    public JMenu optionsMenu;

    /**
     * The View Menu
     */
    public JMenu viewMenu;

    /**
     * The Help Menu
     */
    public JMenu helpMenu;

    /**
     * Create the Menu Bar
     */
    private void createMenuBar() {
        // File Menu
        fileMenu = createFileMenu();
        //run menu
        runMenu = createRunMenu();
        // Debug menu
        serverMenu = createServerMenu();
        // Options menu
        optionsMenu = createOptionsMenu();
        // View Menu
        viewMenu = createViewMenu();
        // Help Menu
        helpMenu = createHelpMenu();
    }

    /**
     * Create the File Menu
     * @return The JMenu
     */
    private JMenu createFileMenu() {
        JMenu jmenu = add(new JMenu("File"));
        jmenu.setMnemonic('f');


        // Import CP

        JMenuItem item = jmenu.add(new MenuAction_OpenContentPackage());
        item.setMnemonic('i');


        // remove package
        item = jmenu.add(_removePackage);
        item.setMnemonic('e');
        jmenu.addSeparator();


        // Exit
        item = jmenu.add(new MenuAction_Exit());
        item.setMnemonic('x');

        return jmenu;
    }


      /**
        * Create the Run Menu
        * @return The JMenu
        */
       private JMenu createRunMenu() {
           JMenu jmenu = add(new JMenu("Run"));
           jmenu.setMnemonic('r');

           JMenuItem item = new JMenuItem(_startPackage);
           jmenu.add(item);
           item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F9, 0));

           jmenu.addSeparator();

           item = new JMenuItem(_resetPackage);
           jmenu.add(item);
           item.setMnemonic('t');

           return jmenu;
       }




    /**
     * Create the Debug Menu
     * @return The JMenu
     */
    private JMenu createServerMenu() {
        JMenu jmenu = add(new JMenu("Server"));
        jmenu.setMnemonic('s');

        JMenuItem item = jmenu.add(_startServer);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));

       // item = jmenu.add(_stopServer);
       // item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, 0));

        return jmenu;
    }

    /**
     * Create the Options Menu
     * @return The JMenu
     */
    private JMenu createOptionsMenu() {
        JMenu jmenu = add(new JMenu("Options"));
        jmenu.setMnemonic('o');
        JMenuItem item = jmenu.add(new MenuAction_Prefs());
        item.setMnemonic('c');
        return jmenu;
    }

    /**
     * Create the View Menu
     * @return The JMenu
     */
    private JMenu createViewMenu() {
        JMenu jmenu = add(new JMenu("View"));
        jmenu.setMnemonic('v');

        // Look & Feels
        JMenu subMenu = new JMenu("Look & Feel");
        addLookFeels(subMenu);
        jmenu.add(subMenu);
        subMenu.setMnemonic('k');

        JMenuItem item = jmenu.add(new MenuAction_StatusWindow());
        item.setMnemonic('d');

       // item = new JMenuItem(_stopPackage);
        //jmenu.add(item);
       // item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F12, 0));


        return jmenu;
    }

    /**
     * Add installed Look & Feels to the menu
     * @param menu The menu to add the L&Fs to.
     */
    protected void addLookFeels(JMenu menu) {
        // Add some Menu Items for each installed L&F
        ButtonGroup bg = new ButtonGroup();

        // Get the look and feels
        UIManager.LookAndFeelInfo[] lfs = UIManager.getInstalledLookAndFeels();

        for(int i = 0; i < lfs.length; i++) {
            UIManager.LookAndFeelInfo lfInfo = lfs[i];

            // Add menu item
            JCheckBoxMenuItem jcb = new JCheckBoxMenuItem(new MenuAction_LookAndFeel(lfInfo));
            JMenuItem item = menu.add(jcb);
            bg.add(item);

            // Select currently set L&F as ticked
            if(UIManager.getLookAndFeel().getName().equals(lfInfo.getName())) jcb.setSelected(true);
        }

    }

    /**
     * Create the Help Menu
     * @return The JMenu
     */
    private JMenu createHelpMenu() {
        JMenu jmenu = add(new JMenu("Help"));
        jmenu.setMnemonic('h');

        // Help
        JMenuItem item = jmenu.add(new MenuAction_Help());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));

        jmenu.addSeparator();

        // About
        item = jmenu.add(new MenuAction_About());
        item.setMnemonic('a');

        return jmenu;
    }

    // =========================== TOOLBAR ==================================

    /**
     * Create the Toolbar.
     * @return The Toolbar
     */
    private ReloadToolBar createToolBar() {
      ReloadToolBar tBar = new ReloadToolBar();

      // import a SCORM CP
      MenuAction_OpenContentPackage actionOpenCp = new
          MenuAction_OpenContentPackage();
      tBar.add(actionOpenCp);

      // remove a SCORM CP
      tBar.add(_removePackage);

      // Add a separator
      tBar.addSeparator();

      // Run a SCORM Package
      tBar.add(_startPackage);

      // Reset a SCORM Package
      tBar.add(_resetPackage);

      // Add a separator
      tBar.addSeparator();

      // Start Tomcat Server
      tBar.add(_startServer);

      // Stop Tomcat Server
    //  tBar.add(_stopServer);

      // Refresh screen
      tBar.add(_stopPackage);

      return tBar;
    }

    /**
     * Get the ReloadToolBar.
     * @return The main toolbar
     */
    public ReloadToolBar getReloadToolBar() {
      return toolBar;
    }

}

