/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.menu;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import uk.ac.reload.dweezil.menu.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.dweezil.gui.*;

import uk.ac.reload.scormplayer.client.gui.*;




/**
 * The "About" Menu Item for the Application.
 * Displays Version, Author, Contact details etc.
 * Extends MenuAction.
 *
 * @author Phillip Beauvoir
 * @version 2003-04-25
 */
public class MenuAction_About
extends MenuAction
{

    /**
     * Default Constructor
     */
    public MenuAction_About() {
        super("About " + ReloadScormPlayer.APP_NAME + "...");
    }

    /**
     * The Menu Item was selected.
     * @param e ActionEvent
     */
    public void actionPerformed(ActionEvent e) {
        String javaVersion = ReloadUtils.getJavaVersion();

        MessageLabel box = new MessageLabel();

        JOptionPane.showMessageDialog(ReloadScormPlayer.getInstance(), box, "About " + ReloadScormPlayer.APP_NAME,
            JOptionPane.PLAIN_MESSAGE, ReloadUIManager.getIcon(ReloadScormPlayer.ICON_APP32));

    }

    class MessageLabel extends JLabel {
        MessageLabel() {
            final Icon icon = ReloadUIManager.getIcon(ReloadScormPlayer.IMAGE_EASTER_EGG);
            setPreferredSize(new Dimension(icon.getIconWidth(), icon.getIconHeight()));
            setVerticalAlignment(SwingConstants.TOP);

            final String msg = "<html>" +
                "<font size=\"+1\" color=blue><b>" + ReloadScormPlayer.APP_NAME + "</b></font><br>" +
                "<font color=black>" +
                "Version " + ReloadScormPlayer.VERSION + " (" + ReloadScormPlayer.BUILD_DATE + ")" + "<br>" +
                ReloadScormPlayer.COPYRIGHT + "<br><br>" +
                "Authors - " + ReloadScormPlayer.AUTHORS + "<br><br>" +
                "Advisers & Contributors - " + ReloadScormPlayer.CONTRIBUTORS + "<br><br>" +
                "<u>" + ReloadScormPlayer.EMAIL_CONTACT + "</u>" + "<br>" +
                "<u>" + ReloadScormPlayer.WEB_PAGE + "</u>" +"<br>";

            setText(msg);

            addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    if(e.getClickCount() == 2 && e.isShiftDown()) {
                        setIcon(getIcon() == null ? icon : null);
                        setText(getText() == null ? msg : null);
                    }
                }
            });

        }
    }
}
