/**
  *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.scormplayer.server.manager;

import java.io.*;
import java.net.*;
import java.util.*;

import org.jdom.*;

import uk.ac.reload.dweezil.util.*;

import uk.ac.reload.moonunit.dom.*;

import uk.ac.reload.scormplayer.server.sequence.*;
import uk.ac.reload.scormplayer.tomcat.*;


public class ScormManager
    extends ReloadDocument {

    public boolean _showTreeWidget;

    public boolean _showNavigation;

    public ReloadSequenceManager _sequence;

    public static String _packageName;

    /*
     * boolean value used to flag that navigation nav file has been found
     */
    public boolean _error_found;

    /*
     * A count used to order the items in an org
     */
    private int sequenceCount = 0;

    public Vector theLinks = new Vector();

    public ScormManager() {}

    /*
     * Keep hold of the default org
     */
    public Element _defaultorg;

    public ScormManager(String packageName, boolean showTreeWidget, boolean showNavigation) {
        _error_found = false;
        File pathToNavFile = new File(TomcatConstants.FULL_COURSES_PATH.
                                      toString() +
                                      File.separatorChar +
                                      packageName +
                                      File.separatorChar +
                                      "reload-settings" +
                                      File.separatorChar +
                                      "settings.xml");
        // Make sure its there
        if (pathToNavFile.exists()) {
            _file = pathToNavFile;
            _packageName = packageName;
            _showNavigation = showNavigation;
            _showTreeWidget = showTreeWidget;
            _sequence = new ReloadSequenceManager();
            // it exists so try to load it...
            try {
                loadDocument(pathToNavFile);
                initialize();
            }
            catch (Exception ex) {
                _error_found = true;
            }
        }
        else {
           // problems if we cannot find the nav file...
           _error_found = true;
        }

    }


    public boolean isTreeVisible(){
        return _showTreeWidget;
    }

    public boolean isNavVisible(){
        return _showNavigation;
    }


    public void initialize(){
        try {
           if (_doc == null){
               return;
           }
           theLinks.clear();
           // grab the default org
           Element theOrg = getDefaultOrganization(_doc.getRootElement());
           if (theOrg != null){
               // build the menu
               addLinks(theOrg, "menu");
           }
       }
       catch (IOException ex) {
           return;
       }
    }


    public String getProjectName() {
        return _packageName;
    }


    public void updateNavStatus(String Sco, String status){
        try {
            Element element = getElementByIdentifier(_defaultorg, Sco);
            Element stat = element.getChild("status");
            stat.setText(status);
            saveDocument();
            // update our table
            _sequence.addtoPrereqTable(Sco, status);
            //_sequence.showPreReqTable();
        }
        catch (Exception ex) {
            System.out.println("Error: could not save changes to navigation file:" + ex);
        }
    }


    /**
     * @return an Element by its IDENTIFIER attribute starting at Parent element
     * This will only search one-level deep from parent
     */
    public Element getElementByIdentifier(Element parent, String identifier) {
        Iterator it = parent.getChildren().iterator();
        while (it.hasNext()) {
            Element child = (Element) it.next();
            if (isDocumentNamespace(child)) {
                String id = child.getAttributeValue("id");
                if (id != null) {
                    if (id.equals(identifier))
                        return child;
                }
            }
        }

        return null;
    }

    public Element getDefaultOrganization(Element orgsElement) {
        Element element = null;

        if (orgsElement != null && orgsElement.getName().equals("course")) {
            // Get ORGANIZATION by default attribute ref
            String defOrg = orgsElement.getAttributeValue("default");
            if (defOrg != null)
                element = getElementByIdentifier(orgsElement, defOrg);
                // Not found so get first ORGANIZATION Element
            if (element == null)
                element = orgsElement.getChild("org", orgsElement.getNamespace());
        }
        _defaultorg = element;
        return element;
    }


    public String[][] getLinks() {
        String[][] links = new String[theLinks.size()][];
        theLinks.copyInto(links);
        return links;
    }


    private void addLinks(Element element, String menuParent) throws
        IOException {
        String name = element.getName();
        // ORGANIZATION
        if (name.equals("org")) {
            String title = "Organization";
            // Display Title if there is one
            String titleElement = element.getAttributeValue("title");
            if (titleElement != null) {
                title = titleElement;
            }
            writeItem(title, "", "", menuParent);
        }
        // ITEM
        else if (name.equals("item")) {

            // Display Title if there is one
            String title = "Item";
            Element titleElement = element.getChild("title");
            if (titleElement != null) {
                if (!titleElement.getText().equals(""))
                    title = titleElement.getText();
            }
            // get the identifier
            String scoID = "";
            scoID = element.getAttributeValue("id");


            String isvisible = element.getAttributeValue("isvisible");
            if (isvisible == null){
                isvisible = "true";
            }
            // get the launch url
            String launch = "";
            Element launchUrl = element.getChild("launch");
            if (launchUrl != null) {
                if (!launchUrl.getText().equals(""))
                    launch = launchUrl.getText();
            }

            String level = "";
            Element indentLevel = element.getChild("level");
            if (indentLevel != null) {
                if (!indentLevel.getText().equals(""))
                    level = indentLevel.getText();
            }

            // get the type, sco, asset or null (doesn't reference anything)
            String scoType = "";
            Element scoStatus = element.getChild("type");
            if (scoStatus != null) {
                if (!scoStatus.getText().equals(""))
                    scoType = scoStatus.getText();
            }

            // only add the item to the sequence if it references something
            if (launch != "" && isvisible.equals("true")) {
                sequenceCount++;

                String prerequisites = "";
                Element prereq = element.getChild("prerequisites");
                prerequisites = prereq.getText();
                if (prerequisites == null){
                    prerequisites = "";
                }
                String currentCount = sequenceCount + "";

                _sequence.addNewItem(scoID, launch, currentCount, scoType, title, prerequisites);
                // let the prerequisite class know about this too
                String statText = "";
                Element stat = element.getChild("status");
                statText = stat.getText();
                _sequence.addtoPrereqTable(scoID, statText);

            }
            if (isvisible.equals("true")){
                writeItem(title, launch, scoID, level);
            }
            else{
                writeItem("* hidden", "#", "", level);
            }
        }

        Iterator it = element.getChildren().iterator();
        while (it.hasNext()) {
            Element child = (Element) it.next();
            addLinks(child, menuParent);
        }

    }

    private void writeItem(String title, String url, String scoId,
                           String parentMenu) throws IOException {
        String pm = parentMenu.replace('-', '_');
        String link;
        if (url.equals("#")){
            link = "javascript:void(0)";
        }
        else if (url != "") {
            link = "javascript:onclick=top.window.frames.code.NavigateItems('" +
                scoId + "');";
        }
        else {
            link = "";
        }
        String[] aMenuItem = {
            title, link, pm};
        theLinks.add(aMenuItem);
    }

    /**
     * @return the Root element name
     */
//    public String getRootElementName() {
//        return "manifest";
//    }

}