/**
  *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.scormplayer.server.sequence;

import java.util.*;
import java.util.regex.*;
import java.util.StringTokenizer;

import bsh.*;


 /**
  * A class to handle prerequisites found for items in the manifest.
  * Uses http://www.beanshell.org/
  * @author Paul Sharples
  * @version 2003-10-20
  */
public class PrerequisiteManager {

    /*
     * A hashtable of all key (scoIDs) and values (status)
    */
    private Hashtable _prereqTable = new Hashtable();

    /**
     * Deafult constructor
     */
    public PrerequisiteManager(){}

    /**
     * A method to allow us to keep our prerequisite table up-to-date
     * @param sco - the id of the sco
     * @param status -  its current status
     */
    public void updatePrerequisites(String sco, String status){
         _prereqTable.put(sco, status);
    }

    /**
     * Method to find if an item should be launched
     * 1. has it already been completed
     * 2. does it rely on any prerequisites
     * @param sco
     * @param prerequisites
     * @return
     */
    public boolean canLaunchItem(String sco, String prerequisites){

        String stat = (String)_prereqTable.get(sco);
        if (stat.equals("completed") || stat.equals("passed")){
            return false;
        }
        else{
            if (!prerequisites.equals("")){
                return checkPrerequisites(prerequisites);
            }
            else{
                return true;
            }
        }

    }


    public boolean checkPrerequisites(String prereq) {
            try {
              Interpreter i = new Interpreter(); // Construct an interpreter
              System.out.println(prereq);
              StringTokenizer st1 = new StringTokenizer(prereq, "&|()~");
              StringTokenizer itemAndValue;
              String aToken = "";
              while (st1.hasMoreTokens()){
                  aToken = st1.nextToken();
                  // if theres no value in quotes to check against..
                  if (aToken.indexOf("=") == -1 && aToken.indexOf("<>") == -1) {
                      // get boolean status
                      if (!doesItemExist(aToken)) {
                          System.out.println("item does not exist"+ aToken);
                          return true;// identifer does not exist, so junk.
                      }
                      else {
                      // item exists in prerequisites table. Has it been completed
                      // or passed -grab the answer and declare the var for the interpreter
                          i.set(aToken.replaceAll("-", "_"), checkStatus(aToken));
                      }
                  }
                  else {
                      itemAndValue = new StringTokenizer(aToken, "=<>");
                      String anToken = itemAndValue.nextToken();
                      if (!doesItemExist(anToken)) {
                          return true; // identifer does not exist, so junk..
                      }
                      else {
                          // item exists in prerequisites table. Has it been
                          // completed or passed -grab the answer and declare
                          // the var for the interpreter...
                          i.set(anToken.replaceAll("-", "_"), getStatus(anToken));
                      }
                  }
              }
              prereq = prereq.replaceAll("-", "_");
              prereq = prereq.replaceAll("&", "&&");
              prereq = prereq.replaceAll("\\|", "||");
              prereq = prereq.replaceAll("~", "!");
              prereq = prereq.replaceAll("<>\\\"", "@");
              prereq = prereq.replaceAll("=\\\"", "^");
              prereq = prereq.replaceAll("\\\"", "\")");
              prereq = prereq.replaceAll("\\^", "=\\\"");
              prereq = prereq.replaceAll("=", ".equals(");

              Vector v = new Vector();
              StringBuffer sb = new StringBuffer();
              sb.append(prereq);
              boolean notSymbFound = false;
              // go backwards char at a time. if you find &}"|& then insert a !)
              for (int j = sb.length() - 1; j > -1; j--) {
                if (sb.charAt(j) == '@') {
                  notSymbFound = true;
                }
                if (notSymbFound) {
                  if (sb.charAt(j) == '|' || sb.charAt(j) == '&' || sb.charAt(j) == '"' ||
                      sb.charAt(j) == '}' || sb.charAt(j) == ')') {
                    notSymbFound = false;
                    v.add(j + 1 + "");
                  }
                }
              }
              Iterator it = v.iterator();
              while (it.hasNext()) {
                sb.insert(Integer.parseInt(it.next().toString()), '!');
              }
              prereq = sb.toString();
              prereq = prereq.replaceAll("@", ".equals(\"");
              Object result = i.eval(prereq);
             // System.out.println("eval: "+prereq);
              String a = result.toString();
              boolean retVal = Boolean.valueOf(a).booleanValue();
              return retVal;
            }
            catch (EvalError ex) {
              System.out.println("Could not parse prerequisites: "+ex);
              return true;
            }
    }

    private boolean doesItemExist(String sco){
        if (_prereqTable.containsKey(sco)) return true;
        else return false;
    }

    private String getStatus(String sco){
        return (String)_prereqTable.get(sco);
    }

    private boolean checkStatus(String sco) {
        String stat = (String) _prereqTable.get(sco);
        if (stat.equals("completed") || stat.equals("passed")) return true;
        else return false;
    }


    /**
     * A utillity method for testing purposes - prints out the current state
     * of the prerequisites table.
     */
    public void showPreReqTable(){
       System.out.println("-----------------------");
       System.out.println("-------prereq table ---");
       System.out.println("-----------------------");
       Enumeration keys = _prereqTable.keys();
       while(keys.hasMoreElements()) {
           String scoID = (String)keys.nextElement();
           String theStatus = (String)_prereqTable.get(scoID);
           System.out.println("SCO ID: " + scoID + "  status: " + theStatus);
       }
        System.out.println("-----------------------\n\n");
    }


   /**
    * A utillity method to test a prerequisite string to see if it is legal.
    * (mainly to check that there are no illegal chars in it)
    * More rubust checking will be made once the prerequisite is parsed.
    * Note: Because sets are not yet supported, ie 2*{S1,S2,S3}
    * we will not flag for ,*{} characters.  This will be addressed in a later
    * version.
    * @param aprereq
    * @return true or false
    */
   public static boolean isValid(String aprereq){
      Pattern pattern;
      Matcher matcher;
      // now lets do a little pattern matching - look for illegal chars
      //String pat = "[^a-zA-Z0-9\\&\\|\\~\\=\\<\\>\\\"\\*\\,\\{\\}\\-\\_\\(\\)]+"; // with sets
      String pat = "[^a-zA-Z0-9\\&\\|\\~\\=\\<\\>\\\"\\-\\_\\(\\)]+"; // without sets
      pattern = Pattern.compile(pat);
      matcher = pattern.matcher(aprereq);
      // make sure there is an even number of quotes
      int quoteCount = (new StringTokenizer(aprereq, "\"")).countTokens() - 1;
      if (quoteCount % 2 != 0){
          //System.out.println("odd number of quotes");
          return false;
      }
      /*
      // Sets not yet supported
      // make sure there is an even number of opening and closing curly braces
      int openCurlyCount = (new StringTokenizer(aprereq, "{")).countTokens() - 1;
      int closeCurlyCount = (new StringTokenizer(aprereq, "}")).countTokens() - 1;
      if (openCurlyCount != closeCurlyCount){
         //System.out.println("incorrect number of opening and closing curly brackets: "
         //+ openCurlyCount + " :" + closeCurlyCount);
          return false;
      }
      */
      // make sure there is an even number of opening and closing bracketss
      int openBracketCount = (new StringTokenizer(aprereq, "(")).countTokens() - 1;
      int closeBracketCount = (new StringTokenizer(aprereq, ")")).countTokens() - 1;
      if (openBracketCount != closeBracketCount){
         //System.out.println("incorrect number of opening and closing brackets: "
         //+ openBracketCount + " :" + closeBracketCount);
          return false;
      }
      // if illegal chars found
      if (matcher.find()){
         //System.out.println("illegal string");
         return false;
      }
  return true;
  }

}

