/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.server.sequence;

import java.util.*;
//import org.jdom.*;


import uk.ac.reload.scormplayer.server.servermodels.ReloadScoDocument;

public class ReloadItemSequence {

    private boolean _hasPrerequisites;
    private String _prerequisites;
    private String _scoitemID;
    private String _title;
    private String _launchUrl;
    private String _sequence;
    private String _scormType;
    private ReloadScoDocument _scoDataModel;

    /**
     * Constructor
     */
    public ReloadItemSequence() {}

    public void setTitle(String title){
         _title = title;
    }


    public String getTitle(){
         return _title;
    }


    public void setLaunchUrl(String id){
        _launchUrl = id;
    }

    public String getLaunchUrl(){
        return _launchUrl;
    }

    public void setSequence(String seq){
        _sequence = seq;
    }

    public String getSequence(){
        return _sequence;
    }

    public String[][] getScoModel(){
        return _scoDataModel.getScoModel();
    }

    public void updateClientModel(String[][] entries){
        _scoDataModel.doLmsCommit(entries);
    }

    /**
     * Method to set the sco type for this class and if its an
     * sco, then load in the xml cmi model for it.
     * @param scoType string which is the ID of the item
     */
    public void setScoType(String scoType){
        _scormType = scoType;
    }

    public void loadInModel() {
        // if this is a sco then we need to access the
        // cmi data model we created when importing the package
        if (isItemSco()) {
            if (_scoitemID != null) {
                ReloadScoDocument anSco = new ReloadScoDocument();
                anSco.loadDocument(_scoitemID);
                _scoDataModel = anSco;
            }
        }
    }

    /**
     * Method to ascertain if an item is an SCO or not
     * @return true/false
     */
    public boolean isItemSco(){
        if (_scormType.compareTo("sco")!=0)
            return false;
        else
            return true;
    }


    public void setItemId(String id){
        _scoitemID = id;
    }

    public String getItemId(){
        return _scoitemID;
    }


    public String getPrerequisites(){
        return _prerequisites;
    }

    public void setPrerequisites(String aprereq){
        if (!aprereq.equals("")){
            // get rid of any spaces
            String pre = aprereq.replaceAll(" ", "");
            // check the prerequisites for legality
            if (PrerequisiteManager.isValid(pre)){
                _prerequisites = pre;
                _hasPrerequisites = true;
            }
            // if they are not legal, then junk them...
            else{
                _hasPrerequisites = false;
            }
        }
        else{
            _hasPrerequisites = false;
        }
    }

    public boolean hasPrerequisites(){
        return _hasPrerequisites;
    }

}