/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.server.sequence;

import java.util.*;


public class ReloadSequenceManager {

    private int _itemCount = 0;

    private Hashtable _itemSequences = new Hashtable();

    private String _currentSco;

    private PrerequisiteManager _prereqManager = new PrerequisiteManager();

    /**
     * Default constructor
     */
    public ReloadSequenceManager() {}


    public String findFirstItemToLaunch(){
        for (int i=1; i <= _itemCount; i++){
            String id = findItemFromIndex(i+"");
            ReloadItemSequence anItem = (ReloadItemSequence)_itemSequences.get(id);
            String preReqs = "";
            if (anItem.hasPrerequisites()){
                preReqs = anItem.getPrerequisites();
            }
            boolean answer = _prereqManager.canLaunchItem(id, preReqs);
            if (answer == true){
                return id;
            }
        }
        return "completed";
    }

    public boolean checkItemsPrerequisites(){
        if (getItem(_currentSco).hasPrerequisites()){
            return _prereqManager.checkPrerequisites(getItem(_currentSco).getPrerequisites());
        }
        else{
            return true;
        }
    }

    public void addtoPrereqTable(String sco, String status){
        _prereqManager.updatePrerequisites(sco, status);
    }


    public void showPreReqTable(){
       _prereqManager.showPreReqTable();
    }


    public void setCurrentSco(String _sco){
        _currentSco=_sco;
    }

    public String getCurrentSco(){
        return _currentSco;
    }


    public ReloadItemSequence getItem(String scoIdentifier){
        ReloadItemSequence anItem = (ReloadItemSequence)_itemSequences.get(scoIdentifier);
        return anItem;
    }

    public void addNewItem(String scoId, String launchUrl, String sequence, String scoType, String title, String prereqs){
        ReloadItemSequence anItem = new ReloadItemSequence();
        anItem.setItemId(scoId);
        anItem.setLaunchUrl(launchUrl);
        anItem.setSequence(sequence);
        anItem.setScoType(scoType);
        anItem.setTitle(title);
        anItem.setPrerequisites(prereqs);
        _itemSequences.put(scoId, anItem);
        _itemCount++;
    }

    public String getLaunchFromId(String scoIdentifier){
        ReloadItemSequence anItem = (ReloadItemSequence)_itemSequences.get(scoIdentifier);
        String theURL = anItem.getLaunchUrl();
        if (theURL != null){
            return theURL;
        }
        else{
            return "";
        }
    }


    public String findItemFromIndex(String index){
          Enumeration keys = _itemSequences.keys();
          while(keys.hasMoreElements()) {
              String scoID = (String)keys.nextElement();
              ReloadItemSequence  anItem = (ReloadItemSequence)_itemSequences.get(scoID);
              if (anItem.getSequence().compareTo(index) == 0){
                  return anItem.getItemId();
              }
          }
          return null;
      }


    /**
    * Method to find out of requested sco is the first item in the course
    * @param scoIdentifier
    * @return true or false
    */
    public boolean isFirstItem(String scoIdentifier){
       ReloadItemSequence anItem = (ReloadItemSequence)_itemSequences.get(scoIdentifier);
       String seqPosition = anItem.getSequence();
       if (seqPosition.compareTo("1") == 0)
           return true;
       else
           return false;
    }

    /**
  * Method to find out of requested sco is the last item in the course
  * @param scoIdentifier
  * @return true or false
  */
  public boolean isLastItem(String scoIdentifier){
     int numberOfItems = _itemSequences.size();
     String numberOfItemsAsStr = numberOfItems + "";
     ReloadItemSequence anItem = (ReloadItemSequence)_itemSequences.get(scoIdentifier);
     String seqPosition = anItem.getSequence();
     if (seqPosition.compareTo(numberOfItemsAsStr) == 0)
         return true;
     else
         return false;
  }

}