/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */


package uk.ac.reload.scormplayer.tomcat;

import java.util.*;
import java.net.URL;

import org.apache.catalina.Connector;
import org.apache.catalina.Context;
import org.apache.catalina.Deployer;
import org.apache.catalina.Engine;
import org.apache.catalina.Host;
import org.apache.catalina.logger.SystemOutLogger;
import org.apache.catalina.startup.Embedded;
import org.apache.catalina.Container;
import org.apache.catalina.Logger;
import org.apache.catalina.valves.*;
import org.apache.catalina.core.StandardHost;
import org.apache.catalina.*;

/**
 * An embedded version of Tomcat server 4.1.18.  This class is used
 * to invoke Tomcat and is based on James Goodwill example found at
 * http://www.onjava.com/pub/a/onjava/2002/04/03/tomcat.html
 *
 * @author James Goodwill. Modifications made by Paul Sharples.
 * @version 2003-09-17
 */

public final class EmbeddedTomcat {

    // The singleton instance of Tomcat
    public static EmbeddedTomcat sharedInstance = new EmbeddedTomcat();

    // keep a track of all loaded contexts
    private Vector loadedContexts;

    // Boolean to see if server already up & running
    private boolean isAlive = false;

    private String path = null;
    private Embedded embedded = null;
    private Host host = null;
    private Engine engine = null;
    private Connector connector = null;

    /**
     * Default Constructor
     */
    private EmbeddedTomcat() {}

    public boolean isAlive() {
        return isAlive;
    }

    /**
     * This method Starts the Tomcat server.
     */
    public void startTomcat()  {

        try {
            // Set the home directory
            System.setProperty("catalina.home", TomcatConstants.TOMCAT_PATH);
            // Create an embedded server
            embedded = new Embedded();
            // print all log statments to standard error
            embedded.setDebug(0);
            embedded.setLogger(new SystemOutLogger());

            // Create an engine
            engine = embedded.createEngine();
            engine.setDefaultHost(TomcatConstants.HOST);
            // Create a default virtual host
            host = embedded.createHost(TomcatConstants.HOST,
                                       TomcatConstants.TOMCAT_PATH
                                       + TomcatConstants.SERVER_APP_FOLDER);

            engine.addChild(host);

            // A little bit of security is needed here. We'll block all
            // addresses requesting server content, bar our own machine
            // on the local loop 127.0.0.1
            RemoteAddrValve remoteAddresses = new RemoteAddrValve();
            remoteAddresses.setDeny("");
            remoteAddresses.setAllow("127.0.0.1");
            ( (StandardHost) host).addValve(remoteAddresses);

            // next add our default contexts
            loadedContexts = new Vector();
            Context context;
            Hashtable defaultContexts = TomcatConstants.getDefaultContexts();
            Enumeration keys = defaultContexts.keys();
            while (keys.hasMoreElements()) {
                String path = (String) keys.nextElement();
                String docBase = (String) defaultContexts.get(path);
                context = embedded.createContext(path,
                                                 TomcatConstants.TOMCAT_PATH +
                                                 docBase);
                host.addChild(context);
                loadedContexts.add(context);
            }
            // Install the assembled container hierarchy
            embedded.addEngine(engine);
            // Assemble and install a default HTTP connector
            connector =
                embedded.createConnector(null, TomcatConstants.PORT, false);
            embedded.addConnector(connector);
            // Start the embedded server
            embedded.start();
            isAlive = true;
        }
        catch (LifecycleException ex) {
            System.out.println("Error: server could not be started: "+ex);
        }
    }

    /**
     * This method Stops the Tomcat server.
     */
    public void stopTomcat() throws Exception {
        // Stop the embedded server
        Iterator listElement;
        listElement = loadedContexts.iterator();
        while (listElement.hasNext()) {
            embedded.removeContext( (Context) listElement.next());
        }
        embedded.removeConnector(connector);
        embedded.removeEngine(engine);
        embedded.removeHost(host);
        embedded.stop();
        isAlive = false;
    }

}