/**
 *  RELOAD SCORM Runtime Environment - A SCORM package runtime player
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.tomcat;

import java.util.*;
import java.io.*;
//import uk.ac.reload.dweezil.util.ReloadUtils;

/**
 * A class to hold all of the constant values needed to use the Tomcat server
 * @author Paul Sharples
 * @version 2003-09-17
 */
public final class TomcatConstants {

   // Hostname - set to default "localhost"
    public static String HOST = "127.0.0.1";

    public TomcatConstants() {}

    public static String SERVER_INFO = "Apache Tomcat Server 4.1.18";

    // Port number to run on
    public static int PORT = 8080;

    // Folder where web applications are be found
    public static String SERVER_APP_FOLDER = "/webapps";

    // Our SCORM application folder
    public static String SCORM_APP_PATH = "/reload-scorm-player";

    // Path to generic project folder
    public static File GENERIC_PROJECT_PATH = new File (System.getProperty("user.dir") + "/server");

    // Path to Tomcat user installation
    public static String TOMCAT_PATH = System.getProperty("user.home") + SCORM_APP_PATH + "/server";

    // Path to users profile files.
    public static File USER_HOME = new File(System.getProperty("user.home"), SCORM_APP_PATH);

    // where the courses are...
    public static String COURSES_FOLDER = "/course-packages";

    // Full path to where our web application is located on disk
    public static File FULL_APP_PATH = new File(TOMCAT_PATH + SERVER_APP_FOLDER +
                                                SCORM_APP_PATH);

    // Full path to where our course information is located on disk
    public static File FULL_COURSES_PATH = new File(FULL_APP_PATH +
        COURSES_FOLDER);

    // Contexts are Web applications themselves.  We need to declare
    // which ones the user will have access to by default
    public static Hashtable getDefaultContexts() {
        Hashtable defaultContexts = new Hashtable();
        defaultContexts.put("", SERVER_APP_FOLDER + "/ROOT");
        defaultContexts.put(SCORM_APP_PATH, SERVER_APP_FOLDER + SCORM_APP_PATH);
        return defaultContexts;
    }

}