/**
 *  Reload Dweezil Library - Handles GUI, Undo and Utility functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */


package uk.ac.reload.dweezil.gui.widgets;

import javax.swing.event.*;


/**
 * A ReloadWidget behaviour Interface
 *
 * @author Phillip Beauvoir
 * @version 2003-07-31
 */
public interface ReloadWidget
{
    /**
     * Get the value of this Widget
     * @return The value contained in the Widget
     */
    public String getTextValue();

    /**
     * Set the text value of the Widget
     * @param text the value to be set for this widget
     */
    public void setTextValue(String text);

    /**
     * Insert some text
     * @param text the text to be inserted
     */
    public void insertTextValue(String text);

    /**
     * Set the max length of the Widget
     * @param maxLength the max length of the widget
     */
    public void setMaxLength(int maxLength);

    /**
     * Determine whether this widget only needs a single line to display itself
     * @return true if this widget needs only a single line to display itself, false otherwise
     */
    public boolean isSingleLine();

    /**
     * Add a Document Listener
     * @param dl the DocumentListener to add
     */
    public void addDocumentListener(DocumentListener dl);

    /**
     * Remove a Document Listener
     * @param dl the DocumentListener to remove
     */
    public void removeDocumentListener(DocumentListener dl);
}