/**
 *  Reload Dweezil Library - Handles GUI, Undo and Utility functions
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.dweezil.menu;

import java.util.*;

import java.awt.*;
import javax.swing.*;

/**
 * An abstract JMenu which allows us to add additional menu items
 *
 * @author Phillip Beauvoir
 * @version 2003-09-17
 */
public abstract class ReloadMenu extends JMenu {

	/**
	 * Additional Menu Items
	 */
    Vector _additionalItems;

	/**
	 * Constructor
	 * @param s the text for the Menu Label
	 */
    public ReloadMenu(String s) {
        super(s);
    }

	/**
	 * Add additional items to the Menu
	 * @param c the component to be added to the menu
	 */
    public void addAdditionalItem(Component c) {
        if(_additionalItems == null) _additionalItems = new Vector();

        _additionalItems.add(add(c));
    }

	/**
	 * Add additional items to the menu
	 * @param action the action of the component to be added to the menu
	 */
    public void addAdditionalItem(Action action) {
        if(_additionalItems == null) _additionalItems = new Vector();

        Component c = add(action);
        _additionalItems.add(c);
    }

	/**
	 * Remove additional menu items that were created
	 */
    public void removeAdditionalItems() {
        // Remove any menu items we created
        if(_additionalItems != null) {
            for(int i = 0; i < _additionalItems.size(); i++) {
                Component c = (Component) _additionalItems.elementAt(i);
                remove(c);
            }
            _additionalItems.clear();
        }
    }

}