/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor;

import java.beans.*;
import java.io.*;

import java.awt.*;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;

import org.jdom.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.editor.contentpackaging.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.editor.prefs.*;
import uk.ac.reload.moonunit.*;
import uk.ac.reload.moonunit.schema.*;
import uk.ac.reload.moonunit.dom.*;

// Tina Manoharan, 2003-08-12

import uk.ac.reload.editor.vocabulary.*;
import uk.ac.reload.editor.vocabulary.importexport.*;


/**
 * The Reload Editor Document/Editor opening routines
 *
 * @author Phillip Beauvoir
 * @version 2003-10-08
 */
public final class ReloadEditorDocumentHandler
extends IMSDocumentHandler
{

    /**
     * Schema root Folder
     */
    public static File SCHEMAROOTFOLDER = new File(System.getProperty("user.dir"), "schema");

    /**
     * Schema shipped Folder
     */
    public static File SCHEMASHIPPEDFOLDER = new File(SCHEMAROOTFOLDER, "shipped");

    /**
     * Schema model Folder
     */
    public static File SCHEMAMODELFOLDER = new File(SCHEMAROOTFOLDER, "model");

    /**
     * Helper root Folder
     */
    public static File HELPERFOLDER = new File(System.getProperty("user.dir"), "helpers");

    // =========================================================================
    // ================================ METADATA ===============================
    // =========================================================================

    // Schema files
    public static File fileSchemaMD1_2_1 = new File(new File(SCHEMAMODELFOLDER, IMSMD_121), "imsmd_rootv1p2p1.xsd");
    public static File fileSchemaMD1_2_2 = new File(new File(SCHEMAMODELFOLDER, IMSMD_122), "imsmd_v1p2p2.xsd");

    // Helper root folder
    public static File folderMDHelper = new File(HELPERFOLDER, "md");

    // Profile helper folder
    public static File folderMDProfile = new File(folderMDHelper, "profile");
    public static File fileDefaultMDProfile = new File(folderMDProfile, "ims-lrm_en_profile.xml");

    // Schema helper folder
    public static File folderMDSchemahelper = new File(folderMDHelper, "schemahelper");

    // Vocab helper folder
    public static File folderMDVocab = new File(folderMDHelper, "vocab");


 	// Tina Manoharan, 2003-08-13

    // =========================================================================
    // ============================== VOCABULARY ===============================
    // =========================================================================

    // Schema files
    public static File fileSchemaVX_LAX_1_0_0 = new File(new File(SCHEMAMODELFOLDER, IMSVX_100), "imsvdex_lax_v1p0pd.xsd");
    public static File fileSchemaVX_FLAT_1_0_0 = new File(new File(SCHEMAMODELFOLDER, IMSVX_100), "imsvdex_flat_v1p0pd.xsd");
    public static File fileSchemaVX_HIERARCHICAL_1_0_0 = new File(new File(SCHEMAMODELFOLDER, IMSVX_100), "imsvdex_hierarchical_v1p0pd.xsd");
    public static File fileSchemaVX_GLOSSARY_1_0_0 = new File(new File(SCHEMAMODELFOLDER, IMSVX_100), "imsvdex_glossary_v1p0pd.xsd");
    public static File fileSchemaVX_THESAURUS_1_0_0 = new File(new File(SCHEMAMODELFOLDER, IMSVX_100), "imsvdex_thesaurus_v1p0pd.xsd");

    // Helper root folder
    public static File folderVXHelper = new File(HELPERFOLDER, "vx");

	// Profile helper folder
    public static File folderVXProfile = new File(folderVXHelper, "profile");
    public static File fileDefaultVXProfile = new File(folderVXProfile, "ims-vdex_lax_en_profile.xml");
    public static File fileProfileVX_LAX = new File(folderVXProfile, "ims-vdex_lax_en_profile.xml");
    public static File fileProfileVX_FLAT = new File(folderVXProfile, "ims-vdex_flat_en_profile.xml");
    public static File fileProfileVX_HIERARCHICAL = new File(folderVXProfile, "ims-vdex_hierarchical_en_profile.xml");
    public static File fileProfileVX_GLOSSARY = new File(folderVXProfile, "ims-vdex_glossary_en_profile.xml");
    public static File fileProfileVX_THESAURUS = new File(folderVXProfile, "ims-vdex_thesaurus_en_profile.xml");

	// Schema helper folder
	public static File folderVXSchemahelper = new File(folderVXHelper, "schemahelper");

	// Vocab helper folder
    public static File folderVXVocab = new File(folderVXHelper, "vocab");

    // MD Vdex Vocab folder
    public static File folderMDVdexVocab = new File(folderMDHelper, "vdexVocab");

    // MD Zthes Vocab folder
    public static File folderMDZthesVocab = new File(folderMDHelper, "zthesVocab");

    // MD Other Vocab folder
    public static File folderMDOtherVocab = new File(folderMDHelper, "otherVocab");

    // Stylesheet folder
    public static File folderStylesheets = new File(HELPERFOLDER, "stylesheets");
    public static File VDEX_BROWSER_STYLESHEET = new File(folderStylesheets, "vdex_browser.css");
    public static File ZTHES_TO_VDEX_STYLESHEET = new File(folderStylesheets, "zthes_to_vdex.xsl");
    public static File VDEX_TO_ZTHES_STYLESHEET = new File(folderStylesheets, "vdex_to_zthes.xsl");
    public static File VDEX_TO_TEMP_STYLESHEET = new File(folderStylesheets, "vdex_to_temp.xsl");
    public static File VDEX_TO_KEYWORDS_STYLESHEET = new File(folderStylesheets, "vdex_to_keywords.xsl");
    public static File VDEX_TO_ENTRIES_STYLESHEET = new File(folderStylesheets, "vdex_to_entries.xsl");
    public static File VDEX_METADATA_STYLESHEET = new File(folderStylesheets, "vdex_metadata.xsl");
    public static File TERM_METADATA_STYLESHEET = new File(folderStylesheets, "term_metadata.xsl");


    // =========================================================================
    // ================================ IMS CP =================================
    // =========================================================================

    // Schema files
    public static File fileSchemaCP1_1_2 = new File(new File(SCHEMAMODELFOLDER, IMSCP_112), "imscp_rootv1p1p2.xsd");
    public static File fileSchemaCP1_1_3 = new File(new File(SCHEMAMODELFOLDER, IMSCP_113), "imscp_v1p1.xsd");

    // Helper root folder
    public static File folderCPHelper = new File(HELPERFOLDER, "cp");

    // Profile helper folder
    public static File folderCPProfile = new File(folderCPHelper, "profile");
    public static File fileDefaultCPProfile = new File(folderCPProfile, "ims-cp_en_profile.xml");

    // Schema helper folder
    public static File folderCPSchemahelper = new File(folderCPHelper, "schemahelper");

    // Vocab helper folder
    public static File folderCPVocab = new File(folderCPHelper, "vocab");


    // =========================================================================
    // ================================ SCORM =================================
    // =========================================================================

    // Schema files
    public static File fileSchemaSCORM1_2_IMCP1_1_2 = new File(new File(SCHEMAMODELFOLDER, ADLCP_120_112), "imscp_rootv1p1p2.xsd");
    public static File fileSchemaSCORM1_2_IMCP1_1_3 = new File(new File(SCHEMAMODELFOLDER, ADLCP_120_113), "imscp_v1p1.xsd");

    // Helper root folder
    public static File folderSCORMHelper = new File(HELPERFOLDER, "scorm");

    // Profile helper folder
    public static File folderSCORMProfile = new File(folderSCORMHelper, "profile");
    public static File fileDefaultSCORMProfile = new File(folderSCORMProfile, "scorm12_en_profile.xml");

    // Schema helper folder
    public static File folderSCORMSchemahelper = new File(folderSCORMHelper, "schemahelper");

    // Vocab helper folder
    public static File folderSCORMVocab = new File(folderSCORMHelper, "vocab");


    // =========================================================================
    // ==================== SIMPLE SEQUENCING ==================================
    // =========================================================================

    // Schema files
    public static File fileSchemaSS1_0 = new File(new File(SCHEMAMODELFOLDER, IMSSS_100), "imsss_v1p0.xsd");

    // Helper root
    public static File folderSSHelper = new File(HELPERFOLDER, "ss");

    // Schema helper folder
    public static File folderSSSchemahelper = new File(folderSSHelper, "schemahelper");
    public static File fileHelperSS = new File(folderSSSchemahelper, "imsss_en_helper.xml");

    // Vocab helper folder
    public static File folderSSVocab = new File(folderSSHelper, "vocab");
    public static File fileVocabSS = new File(folderSSVocab, "imsss_en_vocab.xml");



    // =========================================================================
    // ========================== DOCUMENT HANDLING ============================
    // =========================================================================

    /**
     * Ask and Open a file, determine what it is, load it, get the SchemaController
     * and open an appropriate Editor
     */
    public static void openFile(Component parent) {
        // Ask for file
        ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xml", "zip"}, "XML or Archive Files");
        File file = ReloadUtils.askFileNameOpen(parent, "Open XML or Archive file", filter);
        if(file == null) return;

        openFile(parent, file);
    }

    /**
     * Open a file, determine what it is, load it, get the SchemaController
     * and open an appropriate Editor
     */
    public static void openFile(Component parent, File file) {
        ReloadProgressMonitor progressMonitor = null;

        // If we already have it open select and maximise it
        ReloadInternalFrame frame = ReloadInternalFrameManager.getReloadInternalFrame(file);
        if(frame != null) {
            try {
                frame.setIcon(false);
                frame.setSelected(true);
            }
            catch(PropertyVetoException ex) {}
            return;
        }

        try {
            // If user opens a CP zip file then we ask to unzip it
            if(file.getName().toLowerCase().endsWith(".zip")) {
                // First check that archive contains a manifest file
                if(!containsManifest(file)) {
                    // no manifest found in archive
                    JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                              "Manifest file " + MANIFEST_NAME +
                              " cannot be located in archive.", "Archive error", JOptionPane.WARNING_MESSAGE);
                    return;
                }
                // Ask for new project folder
                File targetFolder = askNewContentPackageFolder("Select Folder to Unzip " + file.getName());
                // If the folder already contains the manifest or there was some other problem then return
                if(targetFolder == null) return;

                progressMonitor = new ReloadProgressMonitor(ReloadEditor.getInstance(), "Unzipping Content Package",
                    "Unzipping...", "Searching...", true, ReloadUIManager.getIcon(ReloadEditor.ICON_APP16));

                // Unzip and get imsmanifest
                file = unzipContentPackage(file, targetFolder, progressMonitor);
                if(file == null) return;
            }

            // Wait cursor
            ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

            // Load the Document
            Document doc = XMLUtils.readXMLFile(file);

            // OK, what is it?
            String version = getSchemaVersion(doc);

            // Unknown version
            if(version == null) {
                ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
                showWarningMessage("This is an unsupported version.  Sorry, cannot open file.");
                return;
            }

            // Get the Schema Controller
            ReloadSchemaController schemaController = ReloadEditorSchemaController.getReloadSchemaController(version);

            // Oops!
            if(schemaController == null) {
                ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
                showWarningMessage("This is an unsupported version.  Sorry, cannot open file.");
                return;
            }

            // Get rid of bogus Namespaces
            correctAdditionalNamespaces(doc);

            // Get rid of old "record" elements
            correctLomElement(doc.getRootElement());

            // Now get the appropriate Editor and ReloadDocument

            // MD Editor
            if(schemaController instanceof MetadataSchemaController) {
                MetadataEditorFrame mdEditorFrame = new MetadataEditorFrame();

				// Tina Manoharan, 2003-11-25

                progressMonitor = new ReloadProgressMonitor(ReloadEditor.getInstance(), "Launching New Metadata Editor",
                "Opening file:  " + file.getName(), "Loading external vocabularies, Please wait..." , true, ReloadUIManager.getIcon(ReloadEditor.ICON_APP16));

                mdEditorFrame.setDocument(file, doc, (MetadataSchemaController)schemaController);

                launchEditor(mdEditorFrame);
            }

		 	// Tina Manoharan, 2003-08-12

		 	// VX Editor
		 	else if(schemaController instanceof VocabularySchemaController) {
		 		VocabularyEditorFrame vxEditorFrame = new VocabularyEditorFrame();
		 		vxEditorFrame.setDocument(file, doc, (VocabularySchemaController)schemaController);
		 		launchEditor(vxEditorFrame);
			}

            // ADL SCORM 1.2 Editor
            else if(schemaController instanceof SCORM1_2SchemaController) {
                SCORM1_2PackageEditor scormEditorFrame = new SCORM1_2PackageEditor();
                scormEditorFrame.setDocument(file, doc, (SCORM1_2SchemaController)schemaController);
                launchEditor(scormEditorFrame);
            }

            // IMSP CP Editor
            else if(schemaController instanceof ContentPackageSchemaController) {
                ContentPackageEditor cpEditorFrame = new ContentPackageEditor();
                cpEditorFrame.setDocument(file, doc, (ContentPackageSchemaController)schemaController);
                launchEditor(cpEditorFrame);
            }

            // Add to Recent File List
            ReloadEditorPrefs.getInstance().addFileToHistory(file);
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Error opening file: " + file.getPath(), "Could not open file", ex);
        }
        finally {
            if(progressMonitor != null) progressMonitor.close();
            ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
        }
    }


	// Tina Manoharan, 2003-10-10

    /**
     * Load the given file, get the SchemaController and create the ReloadDocument, the Editor is not launched
     * @param file the XML file
     * @return the created ReloadDocument instance of the file
     */
    public static ReloadDocument openFile(File file) {

		try {

            // Load the Document
            Document doc = XMLUtils.readXMLFile(file);

            // OK, what is the doc?
            String version = getSchemaVersion(doc);

            // Unknown version
            if(version == null) {
                ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
                showWarningMessage("This is an unknown version.  Sorry, cannot open file.");
                return null;
            }

            // Get the Schema Controller
            ReloadSchemaController schemaController = ReloadEditorSchemaController.getReloadSchemaController(version);

            // Oops!
            if(schemaController == null) {
                ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
                showWarningMessage("This is an unknown version.  Sorry, cannot open file.");
                return null;
            }

            // Get rid of bogus Namespaces
            correctAdditionalNamespaces(doc);

            // Now get the appropriate ReloadDocument

		 	// VX Editor
		 	if(schemaController instanceof VocabularySchemaController) {

				Vocabulary vocabulary = new Vocabulary(file, doc, (VocabularySchemaController)schemaController);

		 		return vocabulary;
			}

        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Error opening file: " + file.getName(), "Could not open file", ex);
        }
        finally {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
        }

        return null;
    }



	// Tina Manoharan, 2003-10-20

    /**
     * Load the given file, get the SchemaController and open an appropriate Editor
     * @param file the file to open
     */
    public static void openFile2(File file) {

		try {

            // Load the Document
            Document doc = XMLUtils.readXMLFile(file);

            // OK, what is the doc?
            String version = getSchemaVersion(doc);

            // Unknown version
            if(version == null) {
                ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
                showWarningMessage("This is an unknown version.  Sorry, cannot open file.");
                return;
            }

            // Get the Schema Controller
            ReloadSchemaController schemaController = ReloadEditorSchemaController.getReloadSchemaController(version);

            // Oops!
            if(schemaController == null) {
                ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
                showWarningMessage("This is an unknown version.  Sorry, cannot open file.");
                return;
            }

            // Get rid of bogus Namespaces
            correctAdditionalNamespaces(doc);

            // Now get the appropriate Editor and ReloadDocument

		 	// VX Editor
		 	if(schemaController instanceof VocabularySchemaController) {
		 		VocabularyEditorFrame vxEditorFrame = new VocabularyEditorFrame();
		 		vxEditorFrame.setDocument(file, doc, (VocabularySchemaController)schemaController);
		 		launchEditor(vxEditorFrame);
			}

        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Error opening file: " + file.getName(), "Could not open file", ex);
        }
        finally {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
        }
    }



	// Tina Manoharan, 2003-08-13

    /**
     * Get a Schema Profile File
     * @param schemaFileName the name of the schema file for which we want the profile file
     * @return a Schema Profile File
     */
    public static File getVXSchemaProfileFile(String schemaFileName) {

		if(schemaFileName.equals(fileSchemaVX_LAX_1_0_0.getName())) return fileProfileVX_LAX;
		else if(schemaFileName.equals(fileSchemaVX_FLAT_1_0_0.getName())) return fileProfileVX_FLAT;
		else if(schemaFileName.equals(fileSchemaVX_HIERARCHICAL_1_0_0.getName())) return fileProfileVX_HIERARCHICAL;
		else if(schemaFileName.equals(fileSchemaVX_GLOSSARY_1_0_0.getName())) return fileProfileVX_GLOSSARY;
		else if(schemaFileName.equals(fileSchemaVX_THESAURUS_1_0_0.getName())) return fileProfileVX_THESAURUS;

		return null;
	}


    // =========================================================================
    // ============================ DIALOG/WINDOW HANDLING =====================
    // =========================================================================

    /**
     * New Metadata Standalone Editor with New blank Document
     * @return the MetadataEditorFrame with new blank document
     */
    public static MetadataEditorFrame launchNewMetadataEditor() {
        MetadataEditorFrame mdEditorFrame = null;
        ReloadProgressMonitor progressMonitor = null;

        try {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

            mdEditorFrame = new MetadataEditorFrame();


			// Tina Manoharan, 2003-11-25

            progressMonitor = new ReloadProgressMonitor(ReloadEditor.getInstance(),
            "Launching New Metadata Editor", "Opening New File", "Loading external vocabularies, Please wait...", true, ReloadUIManager.getIcon(ReloadEditor.ICON_APP16));

            mdEditorFrame.newDocument(MetadataSchemaController.getDefaultSchemaController());

            launchEditor(mdEditorFrame);
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            if(mdEditorFrame != null) mdEditorFrame.dispose();
            ErrorDialogBox.showWarning("Error in New Metadata", "Could not create Metadata file", ex);
            mdEditorFrame = null;
        }
        // This had to be commented out for JDK1.4.2 and higher versions
        // Warning message: finally clause cannot complete normally
        //finally {

            if(progressMonitor != null) progressMonitor.close();

            ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
            return mdEditorFrame;
        //}
    }


	// Tina Manoharan, 2003-08-12

    /**
     * New Vocabulary Standalone Editor with New blank Document
     * @return the VocabularyEditorFrame with new blank document
     */
    public static VocabularyEditorFrame launchNewVocabularyEditor() {
        VocabularyEditorFrame vxEditorFrame = null;

        try {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

            vxEditorFrame = new VocabularyEditorFrame();
            vxEditorFrame.newDocument(VocabularySchemaController.getDefaultSchemaController());

            launchEditor(vxEditorFrame);
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            if(vxEditorFrame != null) vxEditorFrame.dispose();
            ErrorDialogBox.showWarning("Error in New Vocabulary", "Could not create Vocabulary file", ex);
            vxEditorFrame = null;
        }
        //finally {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
            return vxEditorFrame;
        //}
    }


    /**
     * New Content Package and Editor
     * @return the ContentPackageEditor with new content package
     */
    public static ContentPackageEditor launchNewContentPackageEditor() {
        ContentPackageEditor cpEditorFrame = null;

        try {
            // Ask for New CP Folder
            File cpFolder = askNewContentPackageFolder("Select Folder for New Content Package");
            // User cancelled or Project Folder not valid
            if(cpFolder == null) return null;

            // Load CP Editor
            ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

            cpEditorFrame = new ContentPackageEditor();
            cpEditorFrame.newContentPackage(cpFolder);

            launchEditor(cpEditorFrame);

            // Add to Recent File List
            ReloadEditorPrefs.getInstance().addFileToHistory(cpEditorFrame.getContentPackage().getFile());
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            if(cpEditorFrame != null) cpEditorFrame.dispose();
            ErrorDialogBox.showWarning("Error in New Content Package", "Could not create Content Package", ex);
            cpEditorFrame = null;
        }
        //finally {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
            return cpEditorFrame;
        //}
    }

    /**
     * Brand new SCORM Package and Editor
     * @return the SCORM1_2PackageEditor with new SCORM package
     */
    public static SCORM1_2PackageEditor launchNewSCORM1_2PackageEditor() {
        SCORM1_2PackageEditor scormEditorFrame = null;

        try {
            // Ask for New CP Folder
            File cpFolder = askNewContentPackageFolder("Select Folder for new SCORM Package");
            // User cancelled or Project Folder not valid
            if(cpFolder == null) return null;

            // Load SCORM Editor
            ReloadEditor.getInstance().setCursor(ReloadUIManager.WAIT_CURSOR);

            scormEditorFrame = new SCORM1_2PackageEditor();
            scormEditorFrame.newContentPackage(cpFolder);

            launchEditor(scormEditorFrame);

            // Add to Recent File List
            ReloadEditorPrefs.getInstance().addFileToHistory(scormEditorFrame.getContentPackage().getFile());
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            if(scormEditorFrame != null) scormEditorFrame.dispose();
            ErrorDialogBox.showWarning("Error in New SCORM Package", "Could not create SCORM Package", ex);
            scormEditorFrame = null;
        }
        //finally {
            ReloadEditor.getInstance().setCursor(ReloadUIManager.DEFAULT_CURSOR);
            return scormEditorFrame;
        //}
    }

    /**
     * Launch and Show an Editor Window later in the Event queue
     * @param editorWindow the ReloadInternalFrame to be launched
     */
    protected static void launchEditor(final ReloadInternalFrame editorWindow) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                ReloadEditor.getInstance().addInternalFrame(editorWindow);
                editorWindow.show();
            }
        });
    }

    /**
     * Ask for a file to open - xml or archive files
     * @return the xml or archive file
     */
    public static File askManifestFileOpen(Component parent) {
        // Make a Filter
        javax.swing.filechooser.FileFilter filter = new javax.swing.filechooser.FileFilter() {
            public boolean accept(File file) {
                if(file.isDirectory()) return true;
                return file.getName().equalsIgnoreCase(MANIFEST_NAME) || file.getName().toLowerCase().endsWith(".zip");
            }
            public String getDescription() {
                return "XML or Archive Files";
            }
        };
        return ReloadUtils.askFileNameOpen(parent, "Open XML or Archive file", filter);
    }

    /**
     * Ask to Unzip a Content Package Archive File
     * @param zipFile the content package archive file
     * @return the Unzipped imsmanifest.xml file or null
     * @throws ReloadEditorException
     */
    public static File unzipContentPackage(File zipFile, File targetFolder, ReloadProgressMonitor progressMonitor) throws ReloadEditorException {
        File manifestFile = null;

        try {
            boolean result = ZipUtils.unpackZip(zipFile, targetFolder, progressMonitor);

            // Pressed Cancel
            if(!result) return null;

            // Get the unzipped manifest
            manifestFile = new File(targetFolder, MANIFEST_NAME);

            // Make sure it is there in case something went wrong...
            if(!manifestFile.exists()) return null;
        }
        catch(Exception ex) {
            throw new ReloadEditorException(ex);
        }
        finally {
            if(progressMonitor != null) progressMonitor.close();
        }

        return manifestFile;
    }

    /**
     * Ask for a New CP Project Folder
     * @param title the title displayed in the dialog's border; a null value results in an empty title
     * @return the Folder or null if cancelled or folder not valid
     */
    public static File askNewContentPackageFolder(String title) {
        // Ask for the File Name
        ReloadFolderChooser chooser = new ReloadFolderChooser();
        chooser.setDialogTitle(title);
        int returnVal = chooser.showOpenDialog(ReloadEditor.getInstance());

        // User Cancelled
        if(returnVal != ReloadFolderChooser.APPROVE_OPTION) return null;

        // Get the chosen File
        File file = chooser.getSelectedFileAndStore();

        // Create if it doesn't exist
        file.mkdirs();

        // Sanity check
        if(!file.exists() || !file.isDirectory()) {
            JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                "Folder doesn't exist", "New Content Package", JOptionPane.WARNING_MESSAGE);
            return null;
        }

        // If there is already an imsmanifest.xml file warn the user
        File imsManifest = new File(file, "imsmanifest.xml");
        if(imsManifest.exists()) {
            JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                "Folder already contains imsmanifest.xml", "New Content Package", JOptionPane.WARNING_MESSAGE);
            return null;
        }

        return file;
    }

    /**
     * Warn the user that we can't cope with an old version
     * @param msg the warning message to be displayed
     */
    protected static void showWarningMessage(String msg) {
        JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                                      msg,
                                      "Reload Editor",
                                      JOptionPane.WARNING_MESSAGE);
    }



	// Tina Manoharan, 2003-10-20

    // =========================================================================
    // ======================== Vocabulary Import/Export HANDLING ==============
    // =========================================================================


    /**
     * Dialogue routine to allow user to import a new Zthes vocabulary as Vdex
     */
    public static void importZthesVocabulary(){

        // Ask for file
        ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xml"}, "XML Files");
        File zthesFile = ReloadUtils.askFileNameOpenUsingDirName(ReloadEditor.getInstance(), "Open XML Zthes Vocabulary file", filter, folderMDZthesVocab);
        if(zthesFile == null) return;

        try {
            // Call routine
            showWarningMessage("Possible loss of information when importing Zthes to Vdex");

            ZthesImporter zthesImporter = new ZthesImporter(zthesFile);
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Import Zthes Vocabulary as Vdex", "Error importing", ex);
        }

    }

    /**
     * Dialogue routine to allow user to import other vocabulary as Vdex
     */
    public static void importOtherVocabulary(){

        // Ask for file
        ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xml"}, "XML Files");
        File otherFile = ReloadUtils.askFileNameOpenUsingDirName(ReloadEditor.getInstance(), "Open XML Vocabulary file", filter, folderMDOtherVocab);
        if(otherFile == null) return;

        try {
            // Call routine
            showWarningMessage("Possible loss of information when importing to Vdex");

            OtherImporter otherImporter = new OtherImporter(otherFile);
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Import Other Vocabulary as Vdex", "Error importing", ex);
        }

    }

    /**
     * Dialogue routine to allow user to export a vdex vocabulary as Zthes
     */
    public static void exportZthesVocabulary(){

        // Ask for file
        ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xml"}, "XML Files");
        File vdexFile = ReloadUtils.askFileNameOpenUsingDirName(ReloadEditor.getInstance(), "Open XML Vdex Vocabulary file", filter, folderMDVdexVocab);
        if(vdexFile == null) return;

        try {
            // Call routine
            showWarningMessage("Possible loss of information when exporting Vdex to Zthes");

			openFile2(vdexFile);

            ZthesExporter zthesExporter = new ZthesExporter(vdexFile);
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Export Vdex Vocabulary as Zthes", "Error exporting", ex);
        }

    }

    /**
     * Dialogue routine to allow user to export a vdex vocabulary as another vocabulary
     */
    public static void exportOtherVocabulary(){

        // Ask for file
        ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xml"}, "XML Files");
        File vdexFile = ReloadUtils.askFileNameOpenUsingDirName(ReloadEditor.getInstance(), "Open XML Vdex Vocabulary file", filter, folderMDVdexVocab);
        if(vdexFile == null) return;

        try {
            // Call routine
            showWarningMessage("Possible loss of information when exporting Vdex to another vocabulary");

			openFile2(vdexFile);

            OtherExporter otherExporter = new OtherExporter(vdexFile);
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            ErrorDialogBox.showWarning("Export Vdex Vocabulary as another vocabulary", "Error exporting", ex);
        }

    }


}