/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.contentpackaging;

import java.io.*;
import java.util.*;

import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.prefs.*;


/**
 * The SCORM 1.2 Helper Profile
 * This reads in a SCORM 1.2 User Profile XML File for helper stuff
 *
 * @author Phillip Beauvoir
 * @version 2003-10-02
 */
public class SCORM1_2HelperProfile
extends ContentPackageHelperProfile
{
    /**
     * Static Profiles - we only want to create these just once and re-use them
     */
    protected static Hashtable _scormProfilesTable = new Hashtable();

    /**
     * Constructor
     */
    public SCORM1_2HelperProfile(File fileProfile) throws ReloadEditorException {
        super(fileProfile);
    }

    /**
     * Factory method for getting a static re-usable SCORM Profile
     */
    public static SCORM1_2HelperProfile getSCORM1_2Profile(File fileProfile) throws ReloadEditorException {
        if(fileProfile == null) return null;
        // Do we have it already?
        SCORM1_2HelperProfile scormProfile = (SCORM1_2HelperProfile)_scormProfilesTable.get(fileProfile);
        // No, first time creation, create and store in table
        if(scormProfile == null) {
            // Does it exist?
            if(fileProfile.exists()) {
                scormProfile = new SCORM1_2HelperProfile(fileProfile);
                _scormProfilesTable.put(fileProfile, scormProfile);
            }
        }
        return scormProfile;
    }

    /**
     * @return all available MetadataProfiles
     */
    public static SCORM1_2HelperProfile[] getSCORM1_2HelperProfiles() {
        // Set up filter to only display xml files
        FileFilter filter = new FileFilter() {
            public boolean accept(File file) {
                if(file.isDirectory()) return false;
                if(ReloadUtils.getFileExtension(file).equals("xml")) return true;
                return false;
            }
        };

        Vector v = new Vector();

        // Get the Files
        File[] files = ReloadEditorDocumentHandler.folderSCORMProfile.listFiles(filter);
        if(files != null) {
            for(int i = 0; i < files.length; i++) {
                try {
                    SCORM1_2HelperProfile profile = getSCORM1_2Profile(files[i]);
                    v.addElement(profile);
                }
                catch(ReloadEditorException ex) {
                }
            }
        }

        SCORM1_2HelperProfile[] profiles = new SCORM1_2HelperProfile[v.size()];
        v.copyInto(profiles);

        return profiles;
    }

    /**
     * @return the Vocabulary File
     */
    public File getVocabFile() {
        return new File(ReloadEditorDocumentHandler.folderSCORMVocab, getVocabularyFileName());
    }

    /**
     * @return the Schema Helper File
     */
    public File getSchemaHelperFile() {
        return new File(ReloadEditorDocumentHandler.folderSCORMSchemahelper, getSchemaHelperFileName());
    }

    /**
     * @return the default MetadataHelperProfile
     */
    public static SCORM1_2HelperProfile getDefaultSCORM1_2HelperProfile() throws ReloadEditorException {
        // Try User Prefs
        String defProfileName = ReloadEditorPrefs.getInstance().getValue(ReloadEditorPrefs.SCORM_DEFAULT_PROFILE);

        // Nope, so use default
        if(defProfileName == null) return getSCORM1_2Profile(ReloadEditorDocumentHandler.fileDefaultSCORMProfile);

        // Try this
        File file = new File(ReloadEditorDocumentHandler.folderSCORMProfile, defProfileName);
        if(file.exists()) return getSCORM1_2Profile(file);

        // Default
        return getSCORM1_2Profile(ReloadEditorDocumentHandler.fileDefaultSCORMProfile);
    }

}