/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.contentpackaging.resourceview;

import java.io.*;

import uk.ac.reload.dweezil.gui.tree.*;
import uk.ac.reload.editor.contentpackaging.*;


/**
 * A Tree Model for the CP Resources Tree
 *
 * @author Phillip Beauvoir
 * @author Paul Sharples
 * @version 2003-10-07
 */
public class CPResourcesTreeModel
extends ReloadTreeModel
{
    /**
     * The Root Folder
     */
    protected ReloadResource _rootFolder;

    /**
     * Constructor
     */
    public CPResourcesTreeModel() {
        super();
    }

    /**
     * Set a new CP Project to be displayed
     */
    public void setFileView(File rootFolder) {
        _rootFolder = new ReloadResource(rootFolder);
        refresh();
    }

    /**
     * Refresh the model from scratch
     */
    public void refresh() {
        root = new CPResourcesTreeNode(_rootFolder);
        buildView((CPResourcesTreeNode)root);
        reload();
    }

    /**
     * Build up child nodes
     */
    private void buildView(CPResourcesTreeNode node) {
        if(node == null) return;

        ReloadResource reloadResource = node.getReloadResource();
        ReloadResource[] children = reloadResource.getChildren();

        if(children != null) {
            for(int i = 0; i < children.length; i++) {
                CPResourcesTreeNode newNode = new CPResourcesTreeNode(children[i]);
                node.add(newNode);
                if(children[i].isDirectory()) buildView(newNode);
            }
        }
    }
}
