/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.gui;

import java.awt.*;
import javax.swing.*;

import uk.ac.reload.editor.*;
import uk.ac.reload.dweezil.util.*;


/**
 * A Panel to hold the Element and Attribute Editors
 *
 * @author Phillip Beauvoir
 * @version 2003-08-04
 */
public class ReloadElementAttributePanel extends JPanel {
    /**
     * The ReloadElementEditor
     */
    public ReloadElementEditor _elementPanel;

    /**
     * The ReloadAttributeEditor
     */
    protected ReloadAttributeEditor _attrPanel;

    /**
     * SplitPane
     */
    protected JSplitPane _splitPane;

    /**
     * Default constructor
     */
    public ReloadElementAttributePanel() {
        setOpaque(false);
        setLayout(new BorderLayout());
        setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));

        // Value Panel
        _elementPanel = createElementEditor();

        // Attribute Panel
        _attrPanel = createAttributeEditor();

        // Splitter
        _splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        _splitPane.setOneTouchExpandable(true);
        _splitPane.setTopComponent(_elementPanel);
        _splitPane.setBottomComponent(_attrPanel);
        add(_splitPane, BorderLayout.CENTER);
    }

    /**
     * Called after Panel is shown so we can init stuff
     */
    public void initView() {
        // For some reason this figure has to be set much lower than it should be
        _splitPane.setDividerLocation(0.12);
    }

    /**
     * Setup the element editor Panel
     * @return the element editor panel
     */
    protected ReloadElementEditor createElementEditor() {
        return new ReloadElementEditor();
    }

    /**
     * Setup the Attribute editor Panel
     * @return the attribute editor panel
     */
    protected ReloadAttributeEditor createAttributeEditor() {
        return new ReloadAttributeEditor();
    }

    /**
     * Get the element editor Panel
     * @return the element editor panel
     */
    public ReloadElementEditor getElementEditor() {
        return _elementPanel;
    }

    /**
     * Get the attribute editor panel
     * @return the attribute editor panel
     */
    public ReloadAttributeEditor getAttributeEditor() {
        return _attrPanel;
    }

    /**
     * Get the split pane
     * @return the Splitpane
     */
    public JSplitPane getSplitpane() {
        return _splitPane;
    }

    /**
     * Set The Panels to display the data for the Element
     * @param elementBinding The ReloadElementBinding object containing the Element, its
     * ReloadSchemaElement and the ReloadDocument
     */
    public void setElement(ReloadElementBinding elementBinding) {
        // Update Value Panel
        _elementPanel.setElement(elementBinding);

        // Update Attribute panel
        _attrPanel.setElement(elementBinding);
    }

    /**
     * Clear the Settings
     */
    public void clear() {
        //_elementPanel.clear();
        //_attrPanel.clear();
    }

    /**
     * Clean up
     */
    public void destroy() {
        _elementPanel.destroy();
        _attrPanel.destroy();
    }
}
