/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.metadata;

import java.util.*;

import org.jdom.*;

import uk.ac.reload.moonunit.dom.*;

/**
 * A Metadata Profile Element
 *
 * @author Phillip Beauvoir
 * @version 2003-03-10
 */
public class MetadataProfileElement {
    /**
     * The Path of the Element
     */
    protected DomPath domPath;

    /**
     * The name to display for this Element
     */
    protected String name;

    /**
     * The Parent, or null if no parent
     */
    protected MetadataProfileElement parent;

    /**
     * This Element's children
     */
    protected MetadataProfileElement[] children;

    /**
     * Whether this is a Grouping
     */
    protected boolean isGroup;

	/**
	 * Constructor
	 * @param element the JDOM element
	 */
    public MetadataProfileElement(Element element) {
        name = element.getAttributeValue("name");
        domPath = new DomPath(element.getAttributeValue("path"));
        isGroup = element.getName().equals("group");
        children = makeChildren(element);
    }

    /**
     * Get the path for this Profile Element
     * @return the path for this profile element
     */
    public DomPath getDomPath() {
        return domPath;
    }

    /**
     * Get the name to display for this Element
     * @return the display name of this profile element
     */
    public String getName() {
        return name;
    }

    /**
     * Find out if this Element is merely a container for others
     * @return true if this element is a container, false otherwise
     */
    public boolean isGroup() {
        return isGroup;
    }

    /**
     * Return the Children of this Profile Element
     * @return the children of this profile element
     */
    public MetadataProfileElement[] getChildren() {
        return children;
    }

    /**
     * Get the parent
     * @return the parent of this profile element
     */
    public MetadataProfileElement getParent() {
        return parent;
    }

    /**
     * Wrap each child Element in a MetadataProfileElement
     * @param element the JDOM parent profile element
     * @return the children of this profile element
     */
    private MetadataProfileElement[] makeChildren(Element element) {
        Vector v = new Vector();

        // Get all "element" children
        Iterator elementlist = element.getChildren().iterator();

        // Enumerate through them
        while(elementlist.hasNext()) {
            // Get the Element
            Element child = (Element) elementlist.next();
            // New MetadataProfileField
            MetadataProfileElement mdProfileChildElement = new MetadataProfileElement(child);
            mdProfileChildElement.parent = this;
            v.addElement(mdProfileChildElement);
        }

        MetadataProfileElement[] array = new MetadataProfileElement[v.size()];
        v.copyInto(array);
        return array;
    }
}