/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.metadata.vdexBrowser;

import java.awt.*;
import java.io.*;
import java.util.Iterator;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.BadLocationException;

import org.jdom.*;

import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.dweezil.gui.widgets.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.dom.vocabulary.*;


/**
 * The Panel used to display the vdex term information.
 *
 * @author Tina Manoharan
 * @version 2003-09-11
 */
public class VdexTermPanel extends JPanel {

	/**
	 * The JSplitPane to display the term tree (left side) and other term information (right side)
	 */
	private JSplitPane splitPane;

	/**
	 * The JScrollPane
	 */
	private JScrollPane left, right;

	/**
	 * The Vdex Tree that displays term id and caption information on its nodes
	 */
	private VdexTree _tree;

	/**
	 * The JPanel
	 */
	private JPanel panel, panel2;

	/**
	 * The Select Button that gets users selection and passes it on to the source ReloadWidget
	 */
	private JButton select;

	/**
	 * The JEditorPane that displays term information (right side)
	 */
	private JEditorPane editor;

	/**
	 * The JEditorPane that displays term metadata information (right side)
	 */
	private JEditorPane mdEditor;

	/**
	 * The Term element
	 */
	private Element element;

	/**
	 * The mine type of the JEditorPane that displays term information
	 */
	private String mimeType = "text/html";

	/**
	 * The mime type of the JEditorPane that displays term metadata information
	 */
	private String mdMimeType = "text/plain";

	/**
	 * The source ReloadWidget that is bound to this MetadataVdexBrowser
	 */
	private ReloadWidget source;

	/**
	 * The MetadataVdexBrowser instance
	 */
	private MetadataVdexBrowser browser;


	/**
	 * Constructor
	 * @param tree the VdexTree that displays term id and caption information on its nodes
	 *		  source the ReloadWidget that is bound to this MetadataVdexBrowser
	 *		  browser the MetadataVdexBrowser instance
	 */
	public VdexTermPanel(VdexTree tree, ReloadWidget source, MetadataVdexBrowser browser) {

		super();

		_tree = tree;

		this.source = source;

		this.browser = browser;

		setLayout(new BorderLayout());

		// LEFT
		left = new JScrollPane(tree);

		// RIGHT
		panel = new JPanel();
		panel.setLayout(new BorderLayout());

		panel2 = new JPanel();
		((FlowLayout)panel2.getLayout()).setAlignment(FlowLayout.LEFT);
		panel2.setBackground(Color.white);
		select = new JButton("Select");
		select.addActionListener(new SelectListener());
		panel2.add(select);

		editor = new JEditorPane();
		editor.setContentType(mimeType);
		editor.setEditable(false);
		editor.addHyperlinkListener(new VdexHyperLinkListener());

		mdEditor = new JEditorPane();
		mdEditor.setContentType(mdMimeType);
		mdEditor.setEditable(false);

		panel.add(panel2, BorderLayout.NORTH);
		panel.add(editor, BorderLayout.CENTER);
		panel.add(mdEditor, BorderLayout.SOUTH);

		right = new JScrollPane(panel);

		// ADD
		splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT , left, right);
		splitPane.setOneTouchExpandable(true);
		splitPane.setDividerLocation(320);

		add(splitPane, BorderLayout.CENTER);

	}

	/**
	 * Set the term element selected on the VdexTree
	 * @param element the selected term element
	 */
	public void setElement(Element element) {

		this.element = element;

		this.updateEditor();
	}

	/**
	 * Update the JEditorPane that displays term information, to reflect the new selection
	 */
	private void updateEditor() {

		String text = "";

		text += "<html><head>";

		//String cssFile = ReloadEditorDocumentHandler.VDEX_BROWSER_STYLESHEET.getAbsolutePath();
		//System.out.println("cssFile = " + cssFile);
		//text += "<link rel=\"stylesheet\" type=\"text/css\" href=\"" + cssFile + "\">";

		text += "<style type=\"text/css\">";
		text += "body { font-size: 10px; font-family: sans-serif; }";
		text += "strong { font-size: 100%; text-align: left; } ";
		text += "tr { text-align: left; vertical-align: top; } ";
		text += "td.alter { background-color: #eeeeee }";
		text += "</style>";

		text += "</head><body>";

		text += "<table width=100% cellpadding=5>";

		text += "<tr>";
		text += "<td class=\"alter\" width=\"20%\">" + "<strong>Description</strong>" + "</td>";
		text += "<td class=\"alter\">" + this.getDescription() + "</td>";
		text += "</tr>";

		text += "<tr>";
		text += "<td>" + "<strong>Media Descriptor</strong>" + "</td>";
		text += "<td>" + this.getMediaDescriptor() + "</td>";
		text += "</tr>";

		text += "<tr>";
		text += "<td class=\"alter\">" + "<strong>Relationship</strong>" + "</td>";
		text += "<td class=\"alter\">" + this.getRelationship() + "</td>";
		text += "</tr>";

		text += "<tr>";
		text += "<td>" + "<strong>Metadata</strong>" + "</td>";
		text += "<td>" + this.getMetadata() + "</td>";
		text += "</tr>";

		text += "</table>";

		text += "</body></html>";


		editor.setText(text);

		repaint();
	}


	/**
	 * Get the term description information
	 * @return the term description if available, or the text 'Not available'
	 */
	public String getDescription() {

		java.util.List children = element.getChildren(VocabularyCore.TERM_DESCRIPTION, element.getNamespace());

		String description = "", val="";

		Element child;

		if(children.size() == 0) {

			description += "Not available.";
		}
		else {
			for(int i = 0; i < children.size(); i++) {

    			child = (Element) children.get(i);

				if(child != null) {

					val = child.getChildTextTrim(VocabularyCore.LANGSTRING, child.getNamespace());

					if(val != null && !val.equals("")) {

						if(i == 0) description += val;
						else description += "<br>" + val;
					}
					else {

						if(i == 0) description += "Not available.";
						else description += "<br>" + "Not available.";
					}
				}
				else {

					if(i == 0) description += "Not available.";
					else description += "<br>" + "Not available.";
				}
        	}
		}

		return description;
	}

	/**
	 * Get the term media descriptor information
	 * @return the term media description if available, or the text 'Not available'
	 */
	public String getMediaDescriptor() {

		java.util.List children = element.getChildren(VocabularyCore.TERM_MEDIA_DESCRIPTOR, element.getNamespace());

		String mediaDescriptor = "", val="";

		Element child;

		if(children.size() == 0) {

			mediaDescriptor += "Not available.";
		}
		else {
			for(int i = 0; i < children.size(); i++) {

    			child = (Element) children.get(i);

				if(child != null) {

					mediaDescriptor += "<i>" + "Media Locator: " + "</i>";

					val = child.getChildTextTrim(VocabularyCore.TERM_MEDIA_LOCATOR, child.getNamespace());

					if(val != null && !val.equals("")) {

						mediaDescriptor += val + "<br>";
					}
					else {

						mediaDescriptor += "Not available." + "<br>";
					}


					mediaDescriptor += "<i>" + "Interpretation Note: " + "</i>";

					Element child2 = child.getChild(VocabularyCore.TERM_INTERPRETATION_NOTE, child.getNamespace());

					if(child2 != null) {

						val = child2.getChildTextTrim(VocabularyCore.LANGSTRING, child2.getNamespace());

						if(val != null && !val.equals("")) {

							if(i < children.size()-1) mediaDescriptor += val + "<br>";
							else mediaDescriptor += val;
						}
						else {

							if(i < children.size()-1) mediaDescriptor += "Not available." + "<br>";
							mediaDescriptor += "Not available.";
						}
					}
					else {

						if(i < children.size()-1) mediaDescriptor += "Not available." + "<br>";
						mediaDescriptor += "Not available.";
					}
				}
				else {

					mediaDescriptor += "<i>" + "Media Locator: " + "</i>" + "Not available." + "<br>";

					if(i < children.size()-1) mediaDescriptor += "<i>" + "Interpretation Note: " + "</i>" + "Not available." + "<br>";
					else mediaDescriptor += "<i>" + "Interpretation Note: " + "</i>" + "Not available.";
				}

        	}
		}

		return mediaDescriptor;

	}

	/**
	 * Get the term relationship information
	 * @return the term relationship if available, or the text 'Not available'
	 */
	public String getRelationship() {

		java.util.List children = element.getChildren(VocabularyCore.TERM_RELATIONSHIP, element.getNamespace());

		String relationship = "", val="";

		Element child;

		if(children.size() == 0) {

			relationship += "Not available.";
		}
		else {
			for(int i = 0; i < children.size(); i++) {

    			child = (Element) children.get(i);

				if(child != null) {

					val = child.getChildTextTrim(VocabularyCore.TERM_RELATIONSHIP_TYPE, child.getNamespace());

					if(val != null && !val.equals("")) {

						relationship += val + ": ";
					}
					else {

						relationship += "__: ";
					}

					val = child.getChildTextTrim(VocabularyCore.TERM_TARGET_TERM, child.getNamespace());

					if(val != null && !val.equals("")) {

						if(i < children.size()-1) relationship += "<a href=\"relationship_target\">" + val + "</a>" + "<br>";
						else relationship += "<a href=\"relationship_target\">" + val + "</a>";
					}
					else {

						if(i < children.size()-1) relationship += "__" + "<br>";
						else relationship += "__";
					}

				}
				else {

					relationship += "__: ";

					if(i < children.size()-1) relationship += "__" + "<br>";
					else relationship += "__";
				}

        	}
		}

		return relationship;

	}

	/**
	 * Get the term metadata information
	 * @return the term metadata if available, or the text 'Not available'
	 */
	public String getMetadata() {

		String metadata = "";

		Element termIDElem = element.getChild(VocabularyCore.TERM_IDENTIFIER, element.getNamespace());

		File folderTemp = new File(System.getProperty("user.dir"), "temp");


		if(termIDElem != null) {

			File stylesheet =  ReloadEditorDocumentHandler.TERM_METADATA_STYLESHEET;

			if(MetadataVdexBrowser._file != null && stylesheet != null) {

				String filename = "termMD_" + termIDElem.getText() + "_" + MetadataVdexBrowser._file.getName();

				File mdFile = null;

				if(folderTemp.exists()) {

					File[] files = folderTemp.listFiles();

					for(int i = 0; i < files.length; i++) {

						mdFile = files[i];

						if(mdFile.getName().equals(filename)) break;
						else mdFile = null;
					}
				}

				if(mdFile == null) {

					try {

						XMLUtils.getMetadataFile(MetadataVdexBrowser._file, stylesheet, filename);
					}
					catch(DweezilXMLException ex) {}
				}

				if(mdFile != null) {

					try {

						BufferedReader bufferedReader = new BufferedReader(new FileReader(mdFile));

						String line = bufferedReader.readLine();

						int index;

						String termID = "", check="";

						while(line != null) {

							line.trim();

							if(line.startsWith("termIdentifier=")) {

								index = line.indexOf('=');

								termID = line.substring(index+1);

								if(termID.equals(termIDElem.getText())) break;
							}

							line = bufferedReader.readLine();
						}

						if(line != null) line = bufferedReader.readLine();

						while(line != null) {

							check = new String(line.trim());

							if(!check.startsWith("termIdentifier=")) metadata += line + "\n";

							line = bufferedReader.readLine();
						}
					}
					catch(FileNotFoundException ex) {

						if(ReloadEditor.DEBUG) ex.printStackTrace();
					}
					catch(IOException ex) {

						if(ReloadEditor.DEBUG) ex.printStackTrace();
					}
				}
			}
		}


		if(metadata.equals("")) {

			mdEditor.setText("");

			metadata = "Not available.";
		}
		else {

			mdEditor.setText(metadata);

			metadata = "";
		}

		return metadata;
	}


	/**
	 * Listener class that listens to the 'Select' button press. It gets the term id and caption
	 * information from the VdexTree and passes it on to the source ReloadWidget that is bound
	 * to this MetadataVdexBrowser.
	 */
	protected class SelectListener implements ActionListener {

		/**
		 * The action to be performed when the 'Select' button is pressed. The term id and caption
		 * information of the node selected in the tree is passed on to the source ReloadWidget.
		 */
		public void actionPerformed(ActionEvent ae) {

			// get the selected id and caption from tree
			// send it back to the source

			if(source instanceof ReloadButtonTextField) {

				ReloadButtonTextField source1 = (ReloadButtonTextField) source;

				source1.setVdexValuesToDocument(_tree.getAllNodeNames());
			}
			else if (source instanceof ReloadButtonTextPane) {

				ReloadButtonTextPane source2 = (ReloadButtonTextPane) source;

				source2.setVdexValuesToDocument(_tree.getAllNodeNames());
			}
			else if(source instanceof ReloadButtonTextArea) {

				ReloadButtonTextArea source3 = (ReloadButtonTextArea) source;

				source3.setVdexValuesToDocument(_tree.getAllNodeNames());
			}

			// close the browse window

			browser.dispose();
		}
	}


	/**
	 * The Listener class that listens to hyperlink selection in the term relationship.
	 */
	protected class VdexHyperLinkListener implements HyperlinkListener {

		/**
		 * Action to be taken when a term relationship hyperlink is pressed. For example, if a thesaurus
		 * relationship is selected it jumps to the appropriate term on the tree.
		 */
		public void hyperlinkUpdate(HyperlinkEvent e) {

			HyperlinkEvent.EventType type = e.getEventType();

			if (type == HyperlinkEvent.EventType.ACTIVATED) {

				if(e.getDescription().equals("relationship_target")) {

					// Highlight the selected Relationship Target on the Tree

					int startIndex = e.getSourceElement().getStartOffset();

					int endIndex = e.getSourceElement().getEndOffset();

					int length = endIndex - startIndex;

					try {

						String text = e.getSourceElement().getDocument().getText(startIndex,length);

						_tree.highlightNode(text.trim());
					}
					catch(BadLocationException ble) {

						if(ReloadEditor.DEBUG) System.out.println("BadLocationException when extracting text from document.");
					}
				}
			}
		}
	}
}





