/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */


package uk.ac.reload.editor.metadata.vdexBrowser;

import java.util.ArrayList;
import java.io.*;

import uk.ac.reload.editor.*;


/**
 * This class holds information about the Vdex Vocabulary such as name, id, default language, metadata.
 *
 * @author Tina Manoharan
 * @version 2003-09-11
 */
public class VdexVocabNode
{
	/**
	 * The vocabulary name
	 */
	private String vocabName;

	/**
	 * The vocabulary identifier
	 */
	private String vocabID;

	/**
	 * The vocabulary default language
	 */
	private String vocabLang;

	/**
	 * The vocabulary Metadata
	 */
	private File vocabMD;

	/**
	 * Default constructor
	 */
	public VdexVocabNode() {

		this.vocabName = "Not available.";

		this.vocabID = "Not available.";

		this.vocabLang = "en";


		this.vocabMD = null;
	}

	/**
	 * Set the vocabulary name
	 * @param vocabName the vocabulary name
	 */
	public void setVocabName(String vocabName) {

		this.vocabName = vocabName;
	}

	/**
	 * Set the vocabulary identifier
	 * @param vocabID the vocabulary identifier
	 */
	public void setVocabID(String vocabID) {

		this.vocabID = vocabID;
	}

	/**
	 * Set the vocabulary default language
	 * @param vocabLang the vocabulary default language
	 */
	public void setVocabLang(String vocabLang) {

		this.vocabLang = vocabLang;
	}

	/**
	 * Set the vocabulary metadata
	 * @param vocabMD the vocabulary metadata
	 */
	public void setVocabMDFile(File vocabMD) {

		this.vocabMD = vocabMD;
	}

	/**
	 * Get the vocabulary name
	 * @return the vocabulary name
	 */
	public String getVocabName() {

		if(vocabName == null || vocabName.equals("")) vocabName = "Not available.";

		return vocabName;
	}

	/**
	 * Get the vocabuarly identifier
	  @return the vocabulary identifier
	 */
	public String getVocabID() {

		if(vocabID == null || vocabID.equals("")) vocabID = "Not available.";

		return vocabID;
	}

	/**
	 * Get the vocabulary default language
	 * @return the vocabulary default language
	 */
	public String getVocabLang() {

		if(vocabLang == null || vocabLang.equals("")) vocabLang = "en";

		return vocabLang;
	}

	/**
	 * Get the vocabulary metadata
	 * @return the vocabulary metadata
	 */
	public String[] getVocabMD() {

		ArrayList result = new ArrayList();

		if(vocabMD != null) {

			try {

				BufferedReader bufferedReader = new BufferedReader(new FileReader(vocabMD));

				String line = bufferedReader.readLine();

				while( line != null) {

					result.add(line);

					line = bufferedReader.readLine();
				}
			}
			catch(FileNotFoundException ex) {

				if(ReloadEditor.DEBUG) ex.printStackTrace();
			}
			catch(IOException ex) {

				if(ReloadEditor.DEBUG) ex.printStackTrace();
			}
		}

		return (String[]) result.toArray(new String[result.size()]);
	}

}