/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.metadata.vdexBrowser;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;


/**
 * The Panel that displays the Vdex Vocabulary information, such as name, id, default language.
 *
 * @author Tina Manoharan
 * @version 2003-09-11
 */

public class VdexVocabPanel extends JPanel {

	/**
	 * The JEditorPane used for displaying the vocabulary information
	 */
	private JEditorPane editor;

	/**
	 * The mine type of the contents in the JEditorPane
	 */
	private String mimeType = "text/html";


	/**
	 * Constructor
	 * @param vocabNode the vocabulary node containing information such as name, id, default language, metadata
	 */
	public VdexVocabPanel(VdexVocabNode vocabNode) {

		super();

		setLayout(new BorderLayout());

		setBackground(Color.white);

		String text = "<html><head>";

		//String cssFile = ReloadEditorDocumentHandler.VDEX_BROWSER_STYLESHEET.getAbsolutePath();
		//System.out.println("cssFile = " + cssFile);
		//text += "<link rel=\"stylesheet\" type=\"text/css\" href=\"" + cssFile + "\">";

		text += "<style type=\"text/css\">";
		text += "body { font-size: 10px; font-family: sans-serif; }";
		text += "h1 { font-size: 120%; text-align: center; } ";
		text += "strong { font-size: 100%; text-align: left; } ";
		text += "</style>";

		text += "</head><body>";

		text += "<h1>" + vocabNode.getVocabName() + "</h1>";

		text += "<p><strong>" + "Vocabulary ID:  " + "</strong>" + vocabNode.getVocabID() + "</p>";

		text += "<p><strong>" + "Language:  " + "</strong>" + vocabNode.getVocabLang() + "</p>";

		text += "</body></html>";

		editor = new JEditorPane(mimeType, text);

		editor.setEditable(false);

		add(editor, BorderLayout.CENTER);
	}

}



		/*
		///////////////Using CSS for display/////////////

		text += "<html><head>";

		// text += "<link rel=\"stylesheet\" type=\"text/css\" href=\"" + ReloadEditorDocumentHandler.VDEX_BROWSER_STYLESHEET.getAbsolutePath() + "\">";

		text += "<style type=\"text/css\">";

		text += "body { font-size: 10px; font-family: sans-serif; }";

		text += "h1 { font-size: 120%; text-align: center; } ";

		text += "strong { font-size: 100%; text-align: left; } ";

		text += "</style>";

		text += "</head><body>";

		text += "<h1>" + vocabNode.getVocabName() + "</h1>";

		text += "<p><strong>" + "Vocabulary ID:  " + "</strong>" + vocabNode.getVocabID() + "</p>";

		text += "<p><strong>" + "Language:  " + "</strong>" + vocabNode.getVocabLang() + "</p>";

		text += "</body></html>";



		///////////////Without CSS///////////////////////

		text += "<html><head></head><body>";

		text += "<p align=\"center\"><font face=\"sans-serif\" size=4><b>" + vocabNode.getVocabName() + "</b></font></p>";

		text += "<br>";

		text += "<font face=\"sans-serif\" size=3><b>" + "Vocabulary ID:  " + "</b></font>" + vocabNode.getVocabID();

		text += "<br>";

		text += "<font face=\"sans-serif\" size=3><b>" + "Language:  " + "</b></font>" + vocabNode.getVocabLang();

		text += "<br>";

		text += "</body></html>";

		*/

