/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.prefs;

import java.io.*;

import javax.swing.*;

import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.dweezil.gui.layout.*;
import uk.ac.reload.dweezil.prefs.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.metadata.*;



/**
 * A Metadata Settings User Prefs Panel
 *
 * @author Phillip Beauvoir
 * @version 2003-10-01
 */
public class MD_PrefsPanel
extends PrefsPanel
{

    /**
     * Combo Box for Default MD Profile
     */
    protected ReloadComboBox _profileCombobox;

    /**
     * Constructor
     */
    public MD_PrefsPanel() {
        setLayout(new XYLayout());
        constructFields();
    }

    /**
     * Set up the fields
     */
    private void constructFields() {
        int x = 10;
        int y = 10;

        // ============ Default Metadata Profile ==================
        JLabel label = new JLabel("Default Profile:");
        add(label, new XYConstraints(x, y + 2, 0, 0));

        _profileCombobox = new ReloadComboBox(false);
        add(_profileCombobox, new XYConstraints(x += 110, y, 300, COMBOBOX_HEIGHT));

        // Populate it
        MetadataHelperProfile[] profiles = MetadataHelperProfile.getMetadataHelperProfiles();
        _profileCombobox.setItems(profiles);
    }

    /**
     * Update UserPrefs according to the controls' settings
     */
    public void saveToUserPrefs(UserPrefs prefs) {
        // Default MD Profile
        MetadataHelperProfile profile = (MetadataHelperProfile)_profileCombobox.getSelectedItem();
        prefs.putValue(ReloadEditorPrefs.MD_DEFAULT_PROFILE, profile.getFile().getName());
    }

    /**
     * Set the controls' settings from those found in UserPrefs
     */
    public void setFields(UserPrefs prefs) {
        // Default MD Profile
        String fileName = prefs.getValue(ReloadEditorPrefs.MD_DEFAULT_PROFILE);
        if(fileName != null) {
            File file = new File(ReloadEditorDocumentHandler.folderMDProfile, fileName);
            try {
                MetadataHelperProfile profile = MetadataHelperProfile.getMetadataProfile(file);
                if(profile != null) _profileCombobox.setSelectedItem(profile);
            }
            catch(ReloadEditorException ex) {
            }
        }
    }

}
