/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.vocabulary;

import java.io.*;
import java.util.*;

import org.jdom.*;

import uk.ac.reload.editor.*;
import uk.ac.reload.moonunit.dom.*;
import uk.ac.reload.moonunit.schema.*;
import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.editor.metadata.*;
import uk.ac.reload.moonunit.dom.metadata.*;


/**
 * The Vocabulary Metadata
 *
 * @author Tina Manoharan
 * @version 2003-11-10
 */

public class VocabularyMetadata extends Metadata {

	/**
	 * The Vocabulary Metadata element
	 * For example, vdex/metadata element
	 */
	protected Element _mdElement;

	/**
	 * The Metadata LOM root element
	 */
	protected Element _lomElement;

	/**
	 * The Vocabulary document
	 */
	protected Vocabulary _vocabulary;


	/**
	 * Constructor
	 * @param mdElement the vocabulary metadata element
	 * @param vocabulary the vocabulary document
	 */
	public VocabularyMetadata(Element mdElement, Vocabulary vocabulary) {

 		super(false);

		_mdElement = mdElement;

		_vocabulary = vocabulary;

		createDocument(_mdElement);
	}


	/**
	 * Create a new document using the vocabulary metadata
	 * @param mdElement the vocabulary metadata element
	 */
	public void createDocument(Element mdElement) {

		Namespace mdNamespace;

		Document doc;

        // See if we have a LOM Element
        if(mdElement != null) _lomElement = mdElement.getChild(MetadataCore.ROOT_NAME);

        // Nope, so use brute force to get the LOM Element without the Namespace
        if(_lomElement == null) {
            Iterator i = mdElement.getChildren().iterator();
            while(i.hasNext()) {
                Element element = (Element) i.next();
                if(element.getName().equals(MetadataCore.ROOT_NAME)) {
                    _lomElement = element;
                    break;
                }
            }
        }

        // Get MD Namespace from LOM Element
        if(_lomElement != null) {

			mdNamespace = _lomElement.getNamespace();

        	doc = new Document();

			// This returns a deep clone of this element. The new element is detached from its parent,
			// and getParent() on the clone will return null
        	doc.setRootElement((Element) _lomElement.clone());

        	// Setting Metadata _doc to this doc
        	_doc = doc;

            // Determine which Schema Controller it is
            if(mdNamespace != null) {

				ReloadSchemaController controller = ReloadEditorSchemaController.getReloadSchemaController(_doc, mdNamespace);

	            // If we got no controller or the wrong one use default
				if(controller != null && controller instanceof MetadataSchemaController) {

					_controller = (MetadataSchemaController) controller;
				}
			}

		}

	}

	/**
	 * Get the vocabulary document
	 * @return the vocabulary document
	 */
	public Vocabulary getVocabulary() {

		return _vocabulary;
	}

	/**
	 * Get the Metadata LOM element
	 * @return the metadata LOM element
	 */
	public Element getLOMElement() {

		return _lomElement;
	}

	/**
	 * Get the vocabulary metadata element
	 * @return the vocabulary metadata element
	 */
	public Element getMDElement() {

		return _mdElement;
	}

	/**
	 * Get the vocabulary metadata schema controller
	 * @return the vocabulary metadata schema controller
	 */
	public MetadataSchemaController getMetadataSchemaController() {

		if(_controller == null) _controller = MetadataSchemaController.getDefaultSchemaController();

		return (MetadataSchemaController) _controller;
	}

}