/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.vocabulary.formview;

import uk.ac.reload.editor.vocabulary.*;
import uk.ac.reload.moonunit.schema.*;

/**
 * The Vocabulary Form View Model
 * This reads in a VX User Profile and provides the Elements and form view mappings
 *
 * @author Tina Manoharan
 * @version 2003-08-11
 */
public class VocabularyFormModel {
    /**
     * The Vocabulary Document
     */
    protected Vocabulary _vocabulary;

    /**
     * The Vocabulary Schema Controller
     */
    protected VocabularySchemaController _schemaController;

    /**
     * The Vocabulary Helper Profile
     */
    protected VocabularyHelperProfile _vxProfile;


    /**
     * Default constructor
     */
    public VocabularyFormModel() {

    }

    /**
     * Update the Document
     * @param vocabulary the vocabulary document
     */
    public void setDocument(Vocabulary vocabulary) {
        _vocabulary = vocabulary;
        _schemaController = (VocabularySchemaController)_vocabulary.getReloadSchemaController();
        _vxProfile = _schemaController.getVocabularyHelperProfile();
    }

    /**
     * Get the Root Element of the Vocabulary Profile
     * @return the root element of the vocabulary profile
     */
    public VocabularyProfileElement getRootElement() {
        return _vxProfile.getRootElement();
    }

    /**
     * Create a suitable component for a Schema Element or Attribute
     * @param profileElement The profileElement for which to make the widget
     * @return The VocabularyField Widget
     */
    public VocabularyField createWidget(VocabularyProfileElement profileElement) {

        ReloadSchemaNode schemaNode = _schemaController.getReloadSchemaNode(profileElement.getDomPath());

        // Attribute
        //if(schemaNode instanceof ReloadSchemaAttribute)
            //return createAttributeWidget((ReloadSchemaAttribute)schemaNode);

        // Element
        if(schemaNode instanceof ReloadSchemaElement) {

            return createElementWidget((ReloadSchemaElement)schemaNode);
		}

        else {
			return null;
		}
    }

    /**
     * Create a widget bound to an Element
     * @param schemaElement The Element to Widgetise
     * @return A VocabularyField widget
     */
    protected VocabularyField createElementWidget(ReloadSchemaElement schemaElement) {

        // Do we have a vocabulary?
        ReloadVocabularyList rvList = _schemaController.getVocabularyList(schemaElement);

        // If we do, make a combo box
        if(rvList != null) return new VocabularyComboBox(_vocabulary, schemaElement, rvList);

        // Else make a Text Field
        else return new VocabularyTextField(_vocabulary, schemaElement);
    }

    /**
     * Create a widget bound to an Element's Attribute
     * @param schemaAttribute The Attribute to Widgetise
     * @return A VocabularyField widget
     */
/*    protected VocabularyField createAttributeWidget(ReloadSchemaAttribute schemaAttribute) {
        // Do we have a vocabulary?
        ReloadVocabularyList rvList = _schemaController.getVocabularyList(schemaAttribute);

        // If we do, make a combo box
        if(rvList != null) return new VocabularyComboBox(_vocabulary, schemaAttribute, rvList);

        // Else make a Text Field
        else return new VocabularyTextField(_vocabulary, schemaAttribute);
    }
*/
}