/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.vocabulary.importexport;

import java.io.*;

import org.jdom.*;

import javax.xml.transform.*;
import javax.xml.transform.stream.*;

//import com.icl.saxon.*;
//import com.icl.saxon.jdom.*;
//import com.icl.saxon.expr.*;
//import com.icl.saxon.om.*;

import org.jdom.transform.*;

import uk.ac.reload.dweezil.util.*;
import uk.ac.reload.dweezil.gui.*;
import uk.ac.reload.editor.*;
import uk.ac.reload.editor.vocabulary.*;
import uk.ac.reload.moonunit.dom.*;


/**
 * Import Other Vocabulary as Vdex Vocabulary
 *
 * @author Tina Manoharan
 * @version 2003-11-15
 */
public class OtherImporter {

	/**
	 * Constructor
	 * @param otherFile the other vocabulary to import as Vdex
	 */
	public OtherImporter(File otherFile) {

		try {

			// create source document

			Document otherDoc = XMLUtils.readXMLFile(otherFile);

        	// Ask for a stylesheet file
        	ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xsl"}, "XSL Files");
        	File stylesheetFile = ReloadUtils.askFileNameOpenUsingDirName(ReloadEditor.getInstance(), "Open XSL Stylesheet file", filter, ReloadEditorDocumentHandler.folderStylesheets);

        	if(stylesheetFile != null) {

			    // Now do a transformation

				Transformer transformer = TransformerFactory.newInstance().newTransformer(new StreamSource(stylesheetFile));

				// create result by performing transformation using stylesheet

				JDOMSource source = new JDOMSource(otherDoc);

				JDOMResult result = new JDOMResult();

				transformer.transform(source, result);

				Document vdexDoc = result.getDocument();


				// save output file

			    File vdexOutputFile = new File(ReloadEditorDocumentHandler.folderMDVdexVocab, otherFile.getName());

			    XMLUtils.write2XMLFile(vdexDoc, vdexOutputFile);


			    // open output file in editor

			    if(vdexOutputFile.canRead()) ReloadEditorDocumentHandler.openFile2(vdexOutputFile);
			}

		}
        catch(TransformerConfigurationException tex) {
            if(ReloadEditor.DEBUG) tex.printStackTrace();
        }
        catch(TransformerException tex2) {
            if(ReloadEditor.DEBUG) tex2.printStackTrace();
        }
        catch(Exception rex) {
            if(ReloadEditor.DEBUG) rex.printStackTrace();
            ErrorDialogBox.showWarning("Import Other Vocabulary as Vdex", "Error importing", rex);
        }
	}
}

/*
	public OtherImporter(File otherFile) {

		try {

			Document otherDoc = XMLUtils.readXMLFile(otherFile);

			// Give it a Saxon wrapper
	    	DocumentWrapper saxonOtherDocWrap = new DocumentWrapper(otherDoc, otherFile.getAbsolutePath());
	    	NamePool pool = NamePool.getDefaultNamePool();
	    	saxonOtherDocWrap.setNamePool(pool);

	    	// Now do a transformation

	    	System.setProperty("javax.xml.transform.TransformerFactory", "com.icl.saxon.TransformerFactoryImpl");
	    	TransformerFactory tfactory = TransformerFactory.newInstance();

        	// Ask for a stylesheet file
        	ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xsl"}, "XSL Files");
        	File stylesheetFile = XMLUtils.askFileNameOpenUsingDirName("Open XSL Stylesheet file", filter, ReloadEditorDocumentHandler.folderStylesheets);

        	if(stylesheetFile != null) {

	    		Templates templates = tfactory.newTemplates(new StreamSource(stylesheetFile.getAbsolutePath()));
	    		Transformer transformer = templates.newTransformer();

	    		File vdexOutputFile = new File(ReloadEditorDocumentHandler.folderMDVdexVocab, otherFile.getName());

	    		transformer.transform(saxonOtherDocWrap, new StreamResult(vdexOutputFile));

	    		if(vdexOutputFile.canRead()) ReloadEditorDocumentHandler.openFile2(vdexOutputFile);
			}

		}
        catch(Exception rex) {
            if(ReloadEditor.DEBUG) rex.printStackTrace();
            ErrorDialogBox.showWarning("Import Other Vocabulary as Vdex", "Error importing", rex);
        }
        catch(TransformerConfigurationException tex) {
            if(ReloadEditor.DEBUG) tex.printStackTrace();
        }
        catch(TransformerException tex2) {
            if(ReloadEditor.DEBUG) tex2.printStackTrace();
        }
	}

*/