/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.editor.vocabulary.tableview;

import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;

import uk.ac.reload.dweezil.gui.tree.*;

/**
 * Extend and over-ride the DefaultTreeCellRenderer so that we can set user icons here.
 *
 * @author Tina Manoharan
 * @version 2003-08-11
 */
class VocabularyTreeRenderer extends DefaultTreeCellRenderer {

	/**
	 * Default constructor
	 */
    public VocabularyTreeRenderer() {
    }

	/**
	 * Configures the renderer based on the passed in components.
	 * @param tree the JTree the receiver is being configured for
	 * 		  value the value to be set for the current tree cell
	 * 		  selected if true, the current tree cell will be drawn as if selected
	 *		  expanded if true, the node is currently expanded
	 *		  leaf if true, the node represents a leaf
	 *		  row the current row of the tree table
	 * 		  hasFocus if true, the node currently has focus
	 * @return the component that the renderer uses to draw the value
	 */
    public Component getTreeCellRendererComponent(JTree tree, Object value,
            boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {

        if(value instanceof VocabularyTreeNode) {
            VocabularyTreeNode selNode = (VocabularyTreeNode) value;
            // Set some icons
            setLeafIcon(selNode == null ? null : selNode.getLeafIcon());
            setClosedIcon(selNode == null ? null : selNode.getClosedIcon());
            setOpenIcon(selNode == null ? null : selNode.getOpenIcon());
        }

        return super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
    }
}

